@@ -37,7 +37,7 @@ trap finish EXIT
 
 if ! [ -x "$(command -v gphoto2)" ]
 then
-	zenity --title="Geeqie camera import" --info --width=200 --text="gphoto2 is not installed" 2> /dev/null
+	zenity --title="Geeqie camera import" --info --text="gphoto2 is not installed" 2> /dev/null
 	exit 0
 fi
 
@@ -48,7 +48,7 @@ fi
 
 if [ "$(gphoto2 --auto-detect | wc -l)" -le 2 ]
 then
-	zenity --error --title="Geeqie camera import" --text="No camera detected" --window-icon=/usr/local/share/pixmaps/geeqie.png --width=250 2> /dev/null
+	zenity --error --title="Geeqie camera import" --text="No camera detected" --window-icon=/usr/local/share/pixmaps/geeqie.png 2> /dev/null
 	exit 0
 fi
 
@@ -72,7 +72,7 @@ then
 		n=$((n + 1))
 	done
 
-	camera_selected=$(printf '%b' "$camera_list" | zenity --width=500 --height=250 --title="Geeqie camera import" --list --text "Select camera" --radiolist --column "Select" --column "Camera" --column "n" --print-column=2 2> /dev/null)
+	camera_selected=$(printf '%b' "$camera_list" | zenity --title="Geeqie camera import" --list --text "Select camera" --radiolist --column "Select" --column "Camera" --column "n" --print-column=2 2> /dev/null)
 
 	if [ $? = 1 ]
 	then
@@ -89,7 +89,7 @@ port="$port_type:$port_address"
 
 script_dir=$(dirname "$0")
 
-if ! zenity --question --title="Geeqie camera import" --text="Camera: $camera\n\nDownloading to folder:\n<b>$PWD</b>" --ok-label="OK" --cancel-label="Cancel" --window-icon=/usr/local/share/pixmaps/geeqie.png --width=350 2> /dev/null
+if ! zenity --question --title="Geeqie camera import" --text="Camera: $camera\n\nDownloading to folder:\n<b>$PWD</b>" --ok-label="OK" --cancel-label="Cancel" --window-icon=/usr/local/share/pixmaps/geeqie.png 2> /dev/null
 then
 	exit 0
 fi
@@ -99,13 +99,13 @@ dest_files_sorted=$(mktemp "${TMPDIR:-/tmp}/geeqie.XXXXXXXXXX")
 
 (
 	gphoto2 --port "$port" --list-files 2> /tmp/geeqie-camera-import.log | awk -W posix 'BEGIN {LINT = "fatal"} /#/ {print $2}' | sort > "$src_files_sorted"
-) | zenity --progress --auto-close --auto-kill --title="Geeqie camera import" --text="Searching for files to download..." --pulsate --window-icon=/usr/local/share/pixmaps/geeqie.png --width=250
+) | zenity --progress --auto-close --auto-kill --title="Geeqie camera import" --text="Searching for files to download..." --pulsate --window-icon=/usr/local/share/pixmaps/geeqie.png
 
 error=$(grep -i error /tmp/geeqie-camera-import.log)
 
 if [ -n "$error" ]
 then
-	zenity --text-info --title="Geeqie camera import" - --window-icon=error --width=250 < /tmp/geeqie-camera-import.log 2> /dev/null
+	zenity --text-info --title="Geeqie camera import" - --window-icon=error < /tmp/geeqie-camera-import.log 2> /dev/null
 	exit 1
 fi
 
@@ -117,7 +117,7 @@ if [ -n "$repeated" ]
 then
 	repeated="Warning: The following source filenames are not unique.\nSome files may not be downloaded.\nSee the Help file.\n\n$repeated"
 
-	if zenity --question --text="$repeated" --title="Geeqie camera import" --cancel-label="OK" --ok-label="Cancel" --width=400 --window-icon=/usr/local/share/pixmaps/geeqie.png 2> /dev/null
+	if zenity --question --text="$repeated" --title="Geeqie camera import" --cancel-label="OK" --ok-label="Cancel" --window-icon=/usr/local/share/pixmaps/geeqie.png 2> /dev/null
 	then
 		exit 1
 	fi
@@ -131,7 +131,7 @@ rm "$dest_files_sorted"
 
 if [ "$files_to_load" -eq 0 ]
 then
-	zenity --info --title="Geeqie camera download" --text="No photos to download" --width=250 --window-icon=usr/local/share/pixmaps/geeqie.png 2> /dev/null
+	zenity --info --title="Geeqie camera download" --text="No photos to download" --window-icon=usr/local/share/pixmaps/geeqie.png 2> /dev/null
 	exit 0
 fi
 
@@ -148,7 +148,7 @@ gphoto2 --port "$port" --hook-script "$script_dir/"geeqie-camera-import-hook-scr
 
 gphoto2_pid=$!
 
-(tail -f "$zen_pipe" 2> /dev/null) | zenity --progress --title="Geeqie camera import" --width=370 --text="Downloading: total: $files_to_load existing: $existing_file_count\n" --auto-close --auto-kill --percentage=0 window-icon=/usr/local/share/pixmaps/geeqie.png 2> /dev/null &
+(tail -f "$zen_pipe" 2> /dev/null) | zenity --progress --title="Geeqie camera import" --text="Downloading: total: $files_to_load existing: $existing_file_count\n" --auto-close --auto-kill --percentage=0 window-icon=/usr/local/share/pixmaps/geeqie.png 2> /dev/null &
 zen_pid=$!
 
 n=0
