@@ -24,21 +24,14 @@
 ##
 ## The lists will be displayed in the About - Credits dialog.
 ##
-## $1 Meson PRIVATE_DIR \n
-## $2 Meson current_build_dir \n
-## $3 running_from_git - "true" or "false" \n
-## $4 gresource.xml \n
+## $1 Meson current_build_dir \n
+## $2 running_from_git - "true" or "false" \n
 ##
 
-mkdir -p "$1"
-
-if [ "$3" = "true" ]
+if [ "$2" = "true" ]
 then
 	git log --pretty=format:"%aN <%aE>" | sed 's/<>//' | sort | uniq --count | sort --general-numeric-sort --reverse --stable --key 1,1 | cut  --characters 1-8 --complement > "$1"/authors
 	printf "\n\0" >> "$1"/authors
 else
 	printf "List of authors not available\n\0" > "$1"/authors
 fi
-
-glib-compile-resources --generate-header --sourcedir="$1" --target="$2"/authors.h "$4"
-glib-compile-resources --generate-source --sourcedir="$1" --target="$2"/authors.c "$4"
