@@ -22,7 +22,7 @@
 #include "bar-gps.h"
 
 #include <array>
-#include <cstring>
+#include <string>
 
 #include <cairo.h>
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
@@ -40,7 +40,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
 #include "bar.h"
 #include "compat.h"
-#include "debug.h"
 #include "dnd.h"
 #include "filedata.h"
 #include "intl.h"
@@ -151,9 +150,6 @@ void bar_pane_gps_dnd_receive(GtkWidget *pane, GdkDragContext *,
 	gint geocoded_count;
 	gdouble latitude;
 	gdouble longitude;
-	GString *message;
-	gchar *location;
-	gchar **latlong;
 
 	pgd = static_cast<PaneGPSData *>(g_object_get_data(G_OBJECT(pane), "pane_data"));
 	if (!pgd) return;
@@ -192,7 +188,7 @@ void bar_pane_gps_dnd_receive(GtkWidget *pane, GdkDragContext *,
 
 			if(count)
 				{
-				message = g_string_new("");
+				g_autoptr(GString) message = g_string_new("");
 				if (count == 1)
 					{
 					fd_found = static_cast<FileData *>(g_list_first(pgd->geocode_list)->data);
@@ -233,23 +229,20 @@ void bar_pane_gps_dnd_receive(GtkWidget *pane, GdkDragContext *,
 												bar_pane_gps_close_save_cb, TRUE);
 
 				gtk_widget_show(gd->dialog);
-				g_string_free(message, TRUE);
 				}
 			}
 		}
 
 	if (info == TARGET_TEXT_PLAIN)
 		{
-		location = decode_geo_parameters(reinterpret_cast<const gchar *>(gtk_selection_data_get_data(selection_data)));
+		g_autofree gchar *location = decode_geo_parameters(reinterpret_cast<const gchar *>(gtk_selection_data_get_data(selection_data)));
 		if (!(g_strstr_len(location,-1,"Error")))
 			{
-			latlong = g_strsplit(location, " ", 2);
+			g_auto(GStrv) latlong = g_strsplit(location, " ", 2);
 			champlain_view_center_on(CHAMPLAIN_VIEW(pgd->gps_view),
-							g_ascii_strtod(latlong[0],nullptr),
-							g_ascii_strtod(latlong[1],nullptr));
-			g_strfreev(latlong);
+			                         g_ascii_strtod(latlong[0], nullptr),
+			                         g_ascii_strtod(latlong[1], nullptr));
 			}
-		g_free(location);
 		}
 }
 
@@ -309,15 +302,12 @@ gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButtonEvent *
 	ClutterColor marker_colour = { MARKER_COLOUR };
 	ClutterColor text_colour = { TEXT_COLOUR };
 	ClutterColor thumb_colour = { THUMB_COLOUR };
-	gchar *current_text;
 	ClutterActor *actor;
 	ClutterActor *direction;
 	ClutterActor *current_image;
-	GString *text;
 	gint height;
 	gint width;
 	GdkPixbufRotation rotate;
-	gchar *altitude = nullptr;
 	ThumbLoader *tl;
 
 	if (bevent->button == MOUSE_BUTTON_LEFT)
@@ -334,7 +324,7 @@ gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButtonEvent *
 		 	champlain_label_set_image(CHAMPLAIN_LABEL(label_marker), nullptr);
 			}
 
-		current_text = g_strdup(champlain_label_get_text(CHAMPLAIN_LABEL(label_marker)));
+		g_autofree gchar *current_text = g_strdup(champlain_label_get_text(CHAMPLAIN_LABEL(label_marker)));
 
 		/* If the marker is showing only the text character, replace it with a
 		 * thumbnail and date and altitude
@@ -387,11 +377,12 @@ gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButtonEvent *
 				thumb_loader_start(tl, fd);
 				}
 
-			text = g_string_new(fd->name);
+			g_autoptr(GString) text = g_string_new(fd->name);
 			g_string_append(text, "\n");
 			g_string_append(text, text_from_time(fd->date));
 			g_string_append(text, "\n");
-			altitude = metadata_read_string(fd, "formatted.GPSAltitude", METADATA_FORMATTED);
+
+			g_autofree gchar *altitude = metadata_read_string(fd, "formatted.GPSAltitude", METADATA_FORMATTED);
 			if (altitude != nullptr)
 				{
 				g_string_append(text, altitude);
@@ -402,9 +393,6 @@ gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButtonEvent *
 			champlain_marker_set_selection_color(&thumb_colour);
 			champlain_marker_set_selection_text_color(&text_colour);
 
-			g_free(altitude);
-			g_string_free(text, TRUE);
-
 			parent_marker = clutter_actor_get_parent(label_marker);
 			if (clutter_actor_get_n_children(parent_marker ) > 1 )
 				{
@@ -429,8 +417,6 @@ gboolean bar_pane_gps_marker_keypress_cb(GtkWidget *widget, ClutterButtonEvent *
 				}
 			}
 
-		g_free(current_text);
-
 		return TRUE;
 		}
 	return TRUE;
@@ -448,18 +434,14 @@ gboolean bar_pane_gps_create_markers_cb(gpointer data)
 	ClutterActor *direction;
 	ClutterColor marker_colour = { MARKER_COLOUR };
 	ClutterColor thumb_colour = { THUMB_COLOUR };
-	GString *message;
 	ClutterContent *canvas;
 
+	const gint selection_added = pgd->selection_count - g_list_length(pgd->not_added);
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgd->progress),
-							static_cast<gdouble>(pgd->selection_count - g_list_length(pgd->not_added)) /
-							static_cast<gdouble>(pgd->selection_count));
+	                              static_cast<gdouble>(selection_added) / static_cast<gdouble>(pgd->selection_count));
 
-	message = g_string_new("");
-	g_string_printf(message, "%u/%i",
-	                pgd->selection_count - g_list_length(pgd->not_added), pgd->selection_count);
-	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pgd->progress), message->str);
-	g_string_free(message, TRUE);
+	g_autofree gchar *message = g_strdup_printf("%u/%i", selection_added, pgd->selection_count);
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pgd->progress), message);
 
 	if(pgd->not_added)
 		{
@@ -690,8 +672,8 @@ void bar_pane_gps_write_config(GtkWidget *pane, GString *outstr, gint indent)
 
 	WRITE_NL();
 	WRITE_STRING("<pane_gps ");
-	write_char_option(outstr, indent, "id", pgd->pane.id);
-	write_char_option(outstr, indent, "title", gtk_label_get_text(GTK_LABEL(pgd->pane.title)));
+	write_char_option(outstr, "id", pgd->pane.id);
+	write_char_option(outstr, "title", gtk_label_get_text(GTK_LABEL(pgd->pane.title)));
 	WRITE_BOOL(pgd->pane, expanded);
 
 	gint w;
@@ -701,12 +683,12 @@ void bar_pane_gps_write_config(GtkWidget *pane, GString *outstr, gint indent)
 
 	const gchar *map_id = bar_pane_gps_get_map_id(pgd);
 	WRITE_NL();
-	write_char_option(outstr, indent, "map-id", map_id);
+	write_char_option(outstr, "map-id", map_id);
 
 	gint zoom;
 	g_object_get(G_OBJECT(pgd->gps_view), "zoom-level", &zoom, NULL);
 	WRITE_NL();
-	write_int_option(outstr, indent, "zoom-level", zoom);
+	write_int_option(outstr, "zoom-level", zoom);
 
 	const auto write_lat_long_option = [pgd, outstr, indent](const gchar *option)
 	{
@@ -714,7 +696,7 @@ void bar_pane_gps_write_config(GtkWidget *pane, GString *outstr, gint indent)
 		g_object_get(G_OBJECT(pgd->gps_view), option, &position, NULL);
 		const gint int_position = position * 1000000;
 		WRITE_NL();
-		write_int_option(outstr, indent, option, int_position);
+		write_int_option(outstr, option, int_position);
 	};
 	write_lat_long_option("latitude");
 	write_lat_long_option("longitude");
@@ -729,26 +711,21 @@ void bar_pane_gps_slider_changed_cb(GtkScaleButton *slider,
                                     gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
-	GString *message;
 
-	message = g_string_new("");
-	g_string_printf(message, _("Zoom %i"), static_cast<gint>(zoom));
+	g_autofree gchar *message = g_strdup_printf(_("Zoom %i"), static_cast<gint>(zoom));
 
 	g_object_set(G_OBJECT(CHAMPLAIN_VIEW(pgd->gps_view)), "zoom-level", static_cast<gint>(zoom), NULL);
-	gtk_widget_set_tooltip_text(GTK_WIDGET(slider), message->str);
-	g_string_free(message, TRUE);
-
+	gtk_widget_set_tooltip_text(GTK_WIDGET(slider), message);
 }
+
 void bar_pane_gps_view_state_changed_cb(ChamplainView *view, GParamSpec *, gpointer data)
 {
 	auto pgd = static_cast<PaneGPSData *>(data);
  	ChamplainState status;
  	gint zoom;
-	GString *message;
 
 	g_object_get(G_OBJECT(view), "zoom-level", &zoom, NULL);
-	message = g_string_new("");
-	g_string_printf(message, _("Zoom level %i"), zoom);
+	g_autofree gchar *message = g_strdup_printf(_("Zoom level %i"), zoom);
 
 	g_object_get(G_OBJECT(view), "state", &status, NULL);
 	if (status == CHAMPLAIN_STATE_LOADING)
@@ -757,13 +734,11 @@ void bar_pane_gps_view_state_changed_cb(ChamplainView *view, GParamSpec *, gpoin
 		}
 	else
 		{
-		gtk_label_set_text(GTK_LABEL(pgd->state), message->str);
+		gtk_label_set_text(GTK_LABEL(pgd->state), message);
 		}
 
-	gtk_widget_set_tooltip_text(GTK_WIDGET(pgd->slider), message->str);
+	gtk_widget_set_tooltip_text(GTK_WIDGET(pgd->slider), message);
 	gtk_scale_button_set_value(GTK_SCALE_BUTTON(pgd->slider), static_cast<gdouble>(zoom));
-
-	g_string_free(message, TRUE);
 }
 
 void bar_pane_gps_notify_cb(FileData *fd, NotifyType type, gpointer data)
@@ -824,27 +799,25 @@ GtkWidget *bar_pane_gps_menu(PaneGPSData *pgd)
 void bar_pane_gps_map_centreing(PaneGPSData *pgd)
 {
 	GenericDialog *gd;
-	GString *message = g_string_new("");
 
+	const gchar *message;
 	if (pgd->centre_map_checked)
 		{
-		message = g_string_append(message, _("Move map centre to marker\n is disabled"));
-		pgd->centre_map_checked = FALSE;
+		message = _("Move map centre to marker\n is disabled");
 		}
 	else
 		{
-		message = g_string_append(message, _("Move map centre to marker\n is enabled"));
-		pgd->centre_map_checked = TRUE;
+		message = _("Move map centre to marker\n is enabled");
 		}
 
+	pgd->centre_map_checked = !pgd->centre_map_checked;
+
 	gd = generic_dialog_new(_("Map centering"),
 				"map_centering", nullptr, TRUE, nullptr, pgd);
-	generic_dialog_add_message(gd, GQ_ICON_DIALOG_INFO, _("Map Centering"), message->str, TRUE);
+	generic_dialog_add_message(gd, GQ_ICON_DIALOG_INFO, _("Map Centering"), message, TRUE);
 	generic_dialog_add_button(gd, GQ_ICON_OK, "OK", nullptr, TRUE);
 
 	gtk_widget_show(gd->dialog);
-
-	g_string_free(message, TRUE);
 }
 
 #if HAVE_GTK4
@@ -859,7 +832,6 @@ gboolean bar_pane_gps_map_keypress_cb(GtkWidget *, GdkEventButton *bevent, gpoin
 	auto pgd = static_cast<PaneGPSData *>(data);
 	GtkWidget *menu;
 	GtkClipboard *clipboard;
-	gchar *geo_coords;
 
 	if (bevent->button == MOUSE_BUTTON_RIGHT)
 		{
@@ -877,13 +849,11 @@ gboolean bar_pane_gps_map_keypress_cb(GtkWidget *, GdkEventButton *bevent, gpoin
 	if (bevent->button == MOUSE_BUTTON_LEFT)
 		{
 		clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
-		geo_coords = g_strdup_printf("%f %f",
-		                             champlain_view_y_to_latitude(CHAMPLAIN_VIEW(pgd->gps_view),bevent->y),
-		                             champlain_view_x_to_longitude(CHAMPLAIN_VIEW(pgd->gps_view),bevent->x));
+		g_autofree gchar *geo_coords = g_strdup_printf("%f %f",
+		                                               champlain_view_y_to_latitude(CHAMPLAIN_VIEW(pgd->gps_view),bevent->y),
+		                                               champlain_view_x_to_longitude(CHAMPLAIN_VIEW(pgd->gps_view),bevent->x));
 		gtk_clipboard_set_text(clipboard, geo_coords, -1);
 
-		g_free(geo_coords);
-
 		return TRUE;
 		}
 
@@ -926,7 +896,6 @@ GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *ma
 	ChamplainMarkerLayer *layer;
 	ChamplainView *view;
 	const gchar *slider_list[] = {GQ_ICON_ZOOM_IN, GQ_ICON_ZOOM_OUT, nullptr};
-	const gchar **slider_icons = slider_list;
 
 	pgd = g_new0(PaneGPSData, 1);
 
@@ -952,9 +921,10 @@ GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *ma
 
 	status = gtk_box_new(GTK_ORIENTATION_HORIZONTAL,0);
 #if HAVE_GTK4
+	const gchar **slider_icons = slider_list;
 	slider = gtk_scale_button_new(1, 17, 1, slider_icons);
 #else
-	slider = gtk_scale_button_new(GTK_ICON_SIZE_SMALL_TOOLBAR, 1, 17, 1, slider_icons);
+	slider = gtk_scale_button_new(GTK_ICON_SIZE_SMALL_TOOLBAR, 1, 17, 1, slider_list);
 #endif
 	gtk_widget_set_tooltip_text(slider, _("Zoom"));
 	gtk_scale_button_set_value(GTK_SCALE_BUTTON(slider), static_cast<gdouble>(zoom));
@@ -1019,8 +989,8 @@ GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *ma
 
 GtkWidget *bar_pane_gps_new_from_config(const gchar **attribute_names, const gchar **attribute_values)
 {
-	gchar *title = g_strdup(_("GPS Map"));
-	gchar *map_id = nullptr;
+	g_autofree gchar *title = g_strdup(_("GPS Map"));
+	g_autofree gchar *map_id = nullptr;
 	gboolean expanded = TRUE;
 	gint height = 350;
 	gint zoom = 7;
@@ -1031,8 +1001,7 @@ GtkWidget *bar_pane_gps_new_from_config(const gchar **attribute_names, const gch
 	 */
 	gint int_latitude = 54000000;
 	gint int_longitude = -4000000;
-	gchar *id = g_strdup("gps");
-	GtkWidget *ret;
+	g_autofree gchar *id = g_strdup("gps");
 
 	while (*attribute_names)
 		{
@@ -1056,17 +1025,14 @@ GtkWidget *bar_pane_gps_new_from_config(const gchar **attribute_names, const gch
 		if (READ_CHAR_FULL("id", id))
 			continue;
 
-		log_printf("unknown attribute %s = %s\n", option, value);
+		config_file_error((std::string("Unknown attribute: ") + option + " = " + value).c_str());
 		}
 
 	bar_pane_translate_title(PANE_COMMENT, id, &title);
 	latitude = static_cast<gdouble>(int_latitude) / 1000000;
 	longitude = static_cast<gdouble>(int_longitude) / 1000000;
-	ret = bar_pane_gps_new(id, title, map_id, zoom, latitude, longitude, expanded, height);
-	g_free(title);
-	g_free(map_id);
-	g_free(id);
-	return ret;
+
+	return bar_pane_gps_new(id, title, map_id, zoom, latitude, longitude, expanded, height);
 }
 
 void bar_pane_gps_update_from_config(GtkWidget *pane, const gchar **attribute_names,
@@ -1083,7 +1049,7 @@ void bar_pane_gps_update_from_config(GtkWidget *pane, const gchar **attribute_na
 	if (!pgd)
 		return;
 
-	gchar *title = nullptr;
+	g_autofree gchar *title = nullptr;
 
 	while (*attribute_names)
 	{
@@ -1117,14 +1083,14 @@ void bar_pane_gps_update_from_config(GtkWidget *pane, const gchar **attribute_na
 			g_object_set(G_OBJECT(CHAMPLAIN_VIEW(pgd->gps_view)), "latitude", latitude, NULL);
 			continue;
 			}
-		log_printf("unknown attribute %s = %s\n", option, value);
+
+		config_file_error((std::string("Unknown attribute: ") + option + " = " + value).c_str());
 	}
 
 	if (title)
 		{
 		bar_pane_translate_title(PANE_COMMENT, pgd->pane.id, &title);
 		gtk_label_set_text(GTK_LABEL(pgd->pane.title), title);
-		g_free(title);
 		}
 
 	gtk_widget_set_size_request(pgd->widget, -1, pgd->height);
