@@ -22,6 +22,9 @@
 #ifndef BAR_GPS_H
 #define BAR_GPS_H
 
+#include <glib.h>
+#include <gtk/gtk.h>
+
 GtkWidget *bar_pane_gps_new(const gchar *id, const gchar *title, const gchar *map_id, gint zoom,
 			gdouble latitude, gdouble longitude, gboolean expanded, gint height);
 GtkWidget *bar_pane_gps_new_from_config(const gchar **attribute_names, const gchar **attribute_values);
