@@ -25,12 +25,12 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdk.h>
 #include <glib-object.h>
+#include <string>
 
 #include <config.h>
 
 #include "bar.h"
 #include "compat.h"
-#include "debug.h"
 #include "filedata.h"
 #include "histogram.h"
 #include "intl.h"
@@ -47,14 +47,12 @@ struct HistMap;
  *-------------------------------------------------------------------
  */
 
-
-
 struct PaneHistogramData
 {
 	PaneData pane;
 	GtkWidget *widget;
 	GtkWidget *drawing_area;
-	Histogram *histogram;
+	Histogram histogram;
 	gint histogram_width;
 	gint histogram_height;
 	GdkPixbuf *pixbuf;
@@ -71,7 +69,7 @@ static void bar_pane_histogram_update(PaneHistogramData *phd)
 	if (phd->pixbuf) g_object_unref(phd->pixbuf);
 	phd->pixbuf = nullptr;
 
-	gtk_label_set_text(GTK_LABEL(phd->pane.title), histogram_label(phd->histogram));
+	gtk_label_set_text(GTK_LABEL(phd->pane.title), phd->histogram.label());
 
 	if (!phd->histogram_width || !phd->histogram_height || !phd->fd) return;
 
@@ -93,7 +91,6 @@ static void bar_pane_histogram_update(PaneHistogramData *phd)
 
 static gboolean bar_pane_histogram_update_cb(gpointer data)
 {
-	const HistMap *histmap;
 	auto phd = static_cast<PaneHistogramData *>(data);
 
 	phd->idle_id = 0;
@@ -101,18 +98,21 @@ static gboolean bar_pane_histogram_update_cb(gpointer data)
 
 	gq_gtk_widget_queue_draw_area(GTK_WIDGET(phd->drawing_area), 0, 0, phd->histogram_width, phd->histogram_height);
 
-	if (phd->fd == nullptr) return G_SOURCE_REMOVE;
-	histmap = histmap_get(phd->fd);
-
-	if (!histmap)
+	if (phd->fd != nullptr)
 		{
-		histmap_start_idle(phd->fd);
-		return G_SOURCE_REMOVE;
-		}
+		const HistMap *histmap = histmap_get(phd->fd);
 
-	phd->pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, phd->histogram_width, phd->histogram_height);
-	gdk_pixbuf_fill(phd->pixbuf, 0xffffffff);
-	histogram_draw(phd->histogram, histmap, phd->pixbuf, 0, 0, phd->histogram_width, phd->histogram_height);
+		if (histmap)
+			{
+			phd->pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, phd->histogram_width, phd->histogram_height);
+			gdk_pixbuf_fill(phd->pixbuf, 0xffffffff);
+			phd->histogram.draw(histmap, phd->pixbuf, 0, 0, phd->histogram_width, phd->histogram_height);
+			}
+		else
+			{
+			histmap_start_idle(phd->fd);
+			}
+		}
 
 	return G_SOURCE_REMOVE;
 }
@@ -139,11 +139,11 @@ static void bar_pane_histogram_write_config(GtkWidget *pane, GString *outstr, gi
 	if (!phd) return;
 
 	WRITE_NL(); WRITE_STRING("<pane_histogram ");
-	write_char_option(outstr, indent, "id", phd->pane.id);
-	write_char_option(outstr, indent, "title", gtk_label_get_text(GTK_LABEL(phd->pane.title)));
+	write_char_option(outstr, "id", phd->pane.id);
+	write_char_option(outstr, "title", gtk_label_get_text(GTK_LABEL(phd->pane.title)));
 	WRITE_BOOL(phd->pane, expanded);
-	WRITE_INT(*phd->histogram, histogram_channel);
-	WRITE_INT(*phd->histogram, histogram_mode);
+	WRITE_INT(phd->histogram, histogram_channel);
+	WRITE_INT(phd->histogram, histogram_mode);
 	WRITE_STRING("/>");
 }
 
@@ -192,7 +192,6 @@ static void bar_pane_histogram_destroy(gpointer data)
 	file_data_unregister_notify_func(bar_pane_histogram_notify_cb, phd);
 
 	file_data_unref(phd->fd);
-	histogram_free(phd->histogram);
 	if (phd->pixbuf) g_object_unref(phd->pixbuf);
 	g_free(phd->pane.id);
 
@@ -202,54 +201,50 @@ static void bar_pane_histogram_destroy(gpointer data)
 static void bar_pane_histogram_popup_channels_cb(GtkWidget *widget, gpointer data)
 {
 	auto phd = static_cast<PaneHistogramData *>(data);
-	gint channel;
+	if (!phd) return;
 
 	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) return;
 
-	if (!phd) return;
+	gint channel = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "menu_item_radio_data"));
+	if (channel == phd->histogram.get_channel()) return;
 
-	channel = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "menu_item_radio_data"));
-	if (channel == histogram_get_channel(phd->histogram)) return;
-
-	histogram_set_channel(phd->histogram, channel);
+	phd->histogram.set_channel(channel);
 	bar_pane_histogram_update(phd);
 }
 
 static void bar_pane_histogram_popup_mode_cb(GtkWidget *widget, gpointer data)
 {
 	auto phd = static_cast<PaneHistogramData *>(data);
-	gint logmode;
+	if (!phd) return;
 
 	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) return;
 
-	if (!phd) return;
+	gint mode = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "menu_item_radio_data"));
+	if (mode == phd->histogram.get_mode()) return;
 
-	logmode = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "menu_item_radio_data"));
-	if (logmode == histogram_get_mode(phd->histogram)) return;
-
-	histogram_set_mode(phd->histogram, logmode);
+	phd->histogram.set_mode(mode);
 	bar_pane_histogram_update(phd);
 }
 
 static GtkWidget *bar_pane_histogram_menu(PaneHistogramData *phd)
 {
 	GtkWidget *menu;
-	gint channel = histogram_get_channel(phd->histogram);
-	gint mode = histogram_get_mode(phd->histogram);
+	gint channel = phd->histogram.get_channel();
+	gint mode = phd->histogram.get_mode();
 
 	menu = popup_menu_short_lived();
 
 	/* use the same strings as in layout-util.cc */
-	menu_item_add_radio(menu, _("Histogram on _Red"),   GINT_TO_POINTER(HCHAN_R), (channel == HCHAN_R), G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
-	menu_item_add_radio(menu, _("Histogram on _Green"), GINT_TO_POINTER(HCHAN_G), (channel == HCHAN_G), G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
-	menu_item_add_radio(menu, _("Histogram on _Blue"),  GINT_TO_POINTER(HCHAN_B), (channel == HCHAN_B), G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
-	menu_item_add_radio(menu, _("_Histogram on RGB"),   GINT_TO_POINTER(HCHAN_RGB), (channel == HCHAN_RGB), G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
-	menu_item_add_radio(menu, _("Histogram on _Value"), GINT_TO_POINTER(HCHAN_MAX), (channel == HCHAN_MAX), G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
+	menu_item_add_radio(menu, _("Histogram on _Red"),   GINT_TO_POINTER(HCHAN_R), channel == HCHAN_R, G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
+	menu_item_add_radio(menu, _("Histogram on _Green"), GINT_TO_POINTER(HCHAN_G), channel == HCHAN_G, G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
+	menu_item_add_radio(menu, _("Histogram on _Blue"),  GINT_TO_POINTER(HCHAN_B), channel == HCHAN_B, G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
+	menu_item_add_radio(menu, _("_Histogram on RGB"),   GINT_TO_POINTER(HCHAN_RGB), channel == HCHAN_RGB, G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
+	menu_item_add_radio(menu, _("Histogram on _Value"), GINT_TO_POINTER(HCHAN_MAX), channel == HCHAN_MAX, G_CALLBACK(bar_pane_histogram_popup_channels_cb), phd);
 
 	menu_item_add_divider(menu);
 
-	menu_item_add_radio(menu, _("Li_near Histogram"), GINT_TO_POINTER(0), (mode == 0), G_CALLBACK(bar_pane_histogram_popup_mode_cb), phd);
-	menu_item_add_radio(menu, _("L_og Histogram"),    GINT_TO_POINTER(1), (mode == 1), G_CALLBACK(bar_pane_histogram_popup_mode_cb), phd);
+	menu_item_add_radio(menu, _("Li_near Histogram"), GINT_TO_POINTER(HMODE_LINEAR), mode == HMODE_LINEAR, G_CALLBACK(bar_pane_histogram_popup_mode_cb), phd);
+	menu_item_add_radio(menu, _("L_og Histogram"),    GINT_TO_POINTER(HMODE_LOG), mode == HMODE_LOG, G_CALLBACK(bar_pane_histogram_popup_mode_cb), phd);
 
 	return menu;
 }
@@ -281,10 +276,9 @@ static GtkWidget *bar_pane_histogram_new(const gchar *id, const gchar *title, gi
 
 	phd->pane.expanded = expanded;
 
-	phd->histogram = histogram_new();
-
-	histogram_set_channel(phd->histogram, histogram_channel);
-	histogram_set_mode(phd->histogram, histogram_mode);
+	phd->histogram = Histogram();
+	phd->histogram.set_channel(histogram_channel);
+	phd->histogram.set_mode(histogram_mode);
 
 	phd->widget = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, PREF_PAD_GAP);
 	g_object_set_data_full(G_OBJECT(phd->widget), "pane_data", phd, bar_pane_histogram_destroy);
@@ -320,13 +314,12 @@ static GtkWidget *bar_pane_histogram_new(const gchar *id, const gchar *title, gi
 
 GtkWidget *bar_pane_histogram_new_from_config(const gchar **attribute_names, const gchar **attribute_values)
 {
-	gchar *title = nullptr;
-	gchar *id = g_strdup("histogram");
+	g_autofree gchar *id = g_strdup("histogram");
+	g_autofree gchar *title = nullptr;
 	gboolean expanded = TRUE;
-	gint height = 80;
+	constexpr gint height = 80;
 	gint histogram_channel = HCHAN_RGB;
-	gint histogram_mode = 0;
-	GtkWidget *ret;
+	gint histogram_mode = HMODE_LINEAR;
 
 	while (*attribute_names)
 		{
@@ -339,25 +332,21 @@ GtkWidget *bar_pane_histogram_new_from_config(const gchar **attribute_names, con
 		if (READ_INT_FULL("histogram_channel", histogram_channel)) continue;
 		if (READ_INT_FULL("histogram_mode", histogram_mode)) continue;
 
-		log_printf("unknown attribute %s = %s\n", option, value);
+		config_file_error((std::string("Unknown attribute: ") + option + " = " + value).c_str());
 		}
 
 	bar_pane_translate_title(PANE_HISTOGRAM, id, &title);
-	ret = bar_pane_histogram_new(id, title, height, expanded, histogram_channel, histogram_mode);
-	g_free(title);
-	g_free(id);
-	return ret;
+
+	return bar_pane_histogram_new(id, title, height, expanded, histogram_channel, histogram_mode);
 }
 
 void bar_pane_histogram_update_from_config(GtkWidget *pane, const gchar **attribute_names, const gchar **attribute_values)
 {
-	PaneHistogramData *phd;
-
-	phd = static_cast<PaneHistogramData *>(g_object_get_data(G_OBJECT(pane), "pane_data"));
+	auto *phd = static_cast<PaneHistogramData *>(g_object_get_data(G_OBJECT(pane), "pane_data"));
 	if (!phd) return;
 
-	gint histogram_channel = phd->histogram->histogram_channel;
-	gint histogram_mode = phd->histogram->histogram_mode;
+	gint histogram_channel = phd->histogram.get_channel();
+	gint histogram_mode = phd->histogram.get_mode();
 
 	while (*attribute_names)
 		{
@@ -369,16 +358,14 @@ void bar_pane_histogram_update_from_config(GtkWidget *pane, const gchar **attrib
 		if (READ_INT_FULL("histogram_channel", histogram_channel)) continue;
 		if (READ_INT_FULL("histogram_mode", histogram_mode)) continue;
 
-
-		log_printf("unknown attribute %s = %s\n", option, value);
+		config_file_error((std::string("Unknown attribute: ") + option + " = " + value).c_str());
 		}
 
-	histogram_set_channel(phd->histogram, histogram_channel);
-	histogram_set_mode(phd->histogram, histogram_mode);
+	phd->histogram.set_channel(histogram_channel);
+	phd->histogram.set_mode(histogram_mode);
 
 	bar_update_expander(pane);
 	bar_pane_histogram_update(phd);
 }
 
-
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
