@@ -21,7 +21,7 @@
 
 #include "bar-sort.h"
 
-#include <cstring>
+#include <string>
 
 #include <gdk/gdk.h>
 #include <glib-object.h>
@@ -29,7 +29,6 @@
 #include "collect-io.h"
 #include "collect.h"
 #include "compat.h"
-#include "debug.h"
 #include "editors.h"
 #include "filedata.h"
 #include "history-list.h"
@@ -61,7 +60,9 @@ struct SortData
 	GtkWidget *bookmarks;
 	LayoutWindow *lw;
 
-	FileDialog *dialog;
+	gchar *name;
+	GtkPopover *name_popover;
+
 	GtkWidget *dialog_name_entry;
 
 	SortModeType mode;
@@ -88,8 +89,6 @@ struct SortData
 
 
 static void bar_sort_undo_set(SortData *sd, GList *src_list, const gchar *dest);
-static void bar_sort_add_close(SortData *sd);
-
 
 static void bar_sort_collection_list_build(GtkWidget *bookmarks)
 {
@@ -110,7 +109,7 @@ static void bar_sort_collection_list_build(GtkWidget *bookmarks)
 	while (work)
 		{
 		FileData *fd;
-		gchar *name;
+		g_autofree gchar *name = nullptr;
 
 		fd = static_cast<FileData *>(work->data);
 		work = work->next;
@@ -124,7 +123,6 @@ static void bar_sort_collection_list_build(GtkWidget *bookmarks)
 			name = g_strdup(fd->name);
 			}
 		bookmark_list_add(bookmarks, name, fd->path);
-		g_free(name);
 		}
 
 	filelist_free(list);
@@ -156,8 +154,6 @@ static void bar_sort_mode_sync(SortData *sd, SortModeType mode)
 		bar_sort_collection_list_build(sd->bookmarks);
 		}
 
-	bar_sort_add_close(sd);
-
 	bar_sort_undo_set(sd, nullptr, nullptr);
 }
 
@@ -167,10 +163,12 @@ static void bar_sort_mode_cb(GtkWidget *combo, gpointer data)
 
 	if (gtk_combo_box_get_active(GTK_COMBO_BOX(combo)) == BAR_SORT_MODE_FOLDER)
 		{
+		gtk_widget_set_tooltip_text(sd->add_button, _("Add Bookmark"));
 		bar_sort_mode_sync(sd, BAR_SORT_MODE_FOLDER);
 		}
 	else
 		{
+		gtk_widget_set_tooltip_text(sd->add_button, _("Create new Collection file"));
 		bar_sort_mode_sync(sd, BAR_SORT_MODE_COLLECTION);
 		}
 }
@@ -469,131 +467,180 @@ static void bar_sort_set_selection_selected_cb(GtkWidget *button, gpointer data)
 	bar_sort_set_selection(sd, BAR_SORT_SELECTION_SELECTED);
 }
 
-static void bar_sort_add_close(SortData *sd)
-{
-	if (sd->dialog) file_dialog_close(sd->dialog);
-	sd->dialog_name_entry = nullptr;
-	sd->dialog = nullptr;
-}
-
-static void bar_sort_add_ok_cb(FileDialog *fd, gpointer data)
+static void bar_sort_add_response_cb(GtkFileChooser *chooser, gint response_id, gpointer data)
 {
 	auto sd = static_cast<SortData *>(data);
-	const gchar *name = gq_gtk_entry_get_text(GTK_ENTRY(sd->dialog_name_entry));
-	gboolean empty_name = (name[0] == '\0');
 
-	name = gq_gtk_entry_get_text(GTK_ENTRY(sd->dialog_name_entry));
-	if (sd->mode == BAR_SORT_MODE_FOLDER)
+	if (response_id == GTK_RESPONSE_ACCEPT)
 		{
-		if (empty_name)
+		if (sd->mode == BAR_SORT_MODE_FOLDER)
 			{
-			name = filename_from_path(fd->dest_path);
+			gboolean empty_name = (sd->name == nullptr);
+
+			g_autoptr(GFile) file = gtk_file_chooser_get_file(chooser);
+			g_autofree gchar *selected_dir = g_file_get_path(file);
+
+			bookmark_list_add(sd->bookmarks, empty_name ? filename_from_path(selected_dir) : sd->name, selected_dir);
+
+			g_free(sd->name);
+			sd->name = nullptr;
 			}
-
-		bookmark_list_add(sd->bookmarks, name, fd->dest_path);
-		}
-	else
-		{
-		gchar *path;
-		gboolean has_extension;
-		auto filename = const_cast<gchar *>(name);
-
-		if (empty_name) return;
-
-		has_extension = file_extension_match(name, GQ_COLLECTION_EXT);
-		if (!has_extension)
-			{
-			filename = g_strconcat(name, GQ_COLLECTION_EXT, NULL);
-			}
-
-		path = g_build_filename(get_collections_dir(), filename, NULL);
-		if (isfile(path))
-			{
-			gchar *text = g_strdup_printf(_("The collection:\n%s\nalready exists."), filename);
-			file_util_warning_dialog(_("Collection exists"), text, GQ_ICON_DIALOG_INFO, nullptr);
-			g_free(text);
-			}
-		else
-			{
-			CollectionData *cd;
-
-			cd = collection_new(path);
-			if (collection_save(cd, path))
-				{
-				bar_sort_collection_list_build(sd->bookmarks);
-				}
-			else
-				{
-				gchar *text = g_strdup_printf(_("Failed to save the collection:\n%s"), path);
-				file_util_warning_dialog(_("Save Failed"), text,
-							 GQ_ICON_DIALOG_ERROR, GENERIC_DIALOG(fd)->dialog);
-				g_free(text);
-				}
-			collection_unref(cd);
-			}
-
-		if (!has_extension) g_free(filename);
-		g_free(path);
 		}
 
-	bar_sort_add_close(sd);
-}
-
-static void bar_sort_add_cancel_cb(FileDialog *, gpointer data)
-{
-	auto sd = static_cast<SortData *>(data);
-
-	bar_sort_add_close(sd);
-}
-
-static void bar_sort_add_cb(GtkWidget *button, gpointer data)
-{
-	auto sd = static_cast<SortData *>(data);
-	GtkWidget *hbox;
-	const gchar *title;
-
-	if (sd->dialog)
+	if (response_id == GQ_RESPONSE_NAME_CLICKED)
 		{
-		gtk_window_present(GTK_WINDOW(GENERIC_DIALOG(sd->dialog)->dialog));
+		gtk_popover_popup(GTK_POPOVER(sd->name_popover));
+		gq_gtk_widget_show_all(GTK_WIDGET(sd->name_popover));
+		gtk_widget_grab_focus(GTK_WIDGET(sd->name_popover));
+
 		return;
 		}
 
-	if (sd->mode == BAR_SORT_MODE_FOLDER)
+	gq_gtk_widget_destroy(GTK_WIDGET(chooser));
+}
+
+static void name_entry_activate_cb(GtkEntry *entry, gpointer data)
+{
+	auto sd = static_cast<SortData *>(data);
+
+	g_free(sd->name);
+	sd->name = g_strdup(gtk_entry_get_text(entry));
+
+	gtk_popover_popdown(GTK_POPOVER(sd->name_popover));
+}
+
+static void new_collection_file_save_failed_cb(GtkDialog *dialog, gint, gpointer)
+{
+	gq_gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+
+static gboolean save_new_collection(GtkFileChooser *chooser, gpointer data)
+{
+	auto sd = static_cast<SortData *>(data);
+	CollectionData *cd;
+	gboolean ret;
+
+	g_autoptr(GFile) file = gtk_file_chooser_get_file(chooser);
+	g_autofree gchar *path = g_file_get_path(file);
+
+	cd = collection_new(path);
+	if (collection_save(cd, path))
 		{
-		title = _("Add Bookmark");
+		bar_sort_collection_list_build(sd->bookmarks);
+		ret = TRUE;
 		}
 	else
 		{
-		title = _("Add Collection");
+		GtkWidget *new_collection_file_save_failed = gtk_message_dialog_new_with_markup(GTK_WINDOW(chooser), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, _("<b>File save failed.</b>\n\nFile \"%s\" was not saved."), path);
+		gtk_window_set_modal(GTK_WINDOW(new_collection_file_save_failed), TRUE);
+
+		g_signal_connect(new_collection_file_save_failed, "response", G_CALLBACK(new_collection_file_save_failed_cb), nullptr);
+
+		gtk_widget_show(new_collection_file_save_failed);
+
+		ret = FALSE;
 		}
 
-	sd->dialog = file_util_file_dlg(title,
-				       "add_bookmark", button,
-				       bar_sort_add_cancel_cb, sd);
-	file_dialog_add_button(sd->dialog, GQ_ICON_OK, "OK", bar_sort_add_ok_cb, TRUE);
+	collection_unref(cd);
 
-	generic_dialog_add_message(GENERIC_DIALOG(sd->dialog), nullptr, title, nullptr, FALSE);
+	return ret;
+}
+
+static void collection_exists_response_cb(GtkDialog *dialog, gint, gpointer)
+{
+	gq_gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+
+static void new_collection_file_response_cb(GtkFileChooser *chooser, gint response_id, gpointer data)
+{
+	auto sd = static_cast<SortData *>(data);
+
+	if (response_id == GTK_RESPONSE_ACCEPT)
+		{
+		g_autofree gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
+
+		if (g_file_test(filename, G_FILE_TEST_EXISTS))
+			{
+			GtkWidget *collection_exists = gtk_message_dialog_new(GTK_WINDOW(chooser), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, _("File \"%s\" already exists."), filename);
+			gtk_window_set_modal(GTK_WINDOW(collection_exists), TRUE);
+
+			g_signal_connect(collection_exists, "response", G_CALLBACK(collection_exists_response_cb), nullptr);
+
+			gtk_widget_show(collection_exists);
+			}
+		else
+			{
+			if (save_new_collection(chooser, sd))
+				{
+				gq_gtk_widget_destroy(GTK_WIDGET(chooser));
+				}
+			}
+		}
+	else
+		{
+		gq_gtk_widget_destroy(GTK_WIDGET(chooser));
+		}
+}
+
+static void bar_sort_add_cb(GtkWidget *, gpointer data)
+{
+	auto sd = static_cast<SortData *>(data);
 
 	if (sd->mode == BAR_SORT_MODE_FOLDER)
 		{
-		file_dialog_add_path_widgets(sd->dialog, nullptr, nullptr, "add_bookmark", nullptr, nullptr);
+		GtkWidget *dialog;
+		GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+
+		dialog = gtk_file_chooser_dialog_new(_("Add Bookmark - Geeqie"), nullptr, action, _("_Cancel"), GTK_RESPONSE_CANCEL, _("Add"), GTK_RESPONSE_ACCEPT, _("Name"), GQ_RESPONSE_NAME_CLICKED, nullptr);
+
+		gtk_widget_set_tooltip_text(gtk_dialog_get_widget_for_response(GTK_DIALOG(dialog), GQ_RESPONSE_NAME_CLICKED), _("Optional alias name for the shortcut.\nThis may be amended or added from the Sort Manager pane.\nIf none given, the basename of the folder is used"));
+
+		GtkWidget *entry = gtk_entry_new();
+		GtkWidget *name_popover = gtk_popover_new(GTK_WIDGET(gtk_dialog_get_widget_for_response(GTK_DIALOG(dialog), GQ_RESPONSE_NAME_CLICKED)));
+		sd->name_popover = GTK_POPOVER(name_popover);
+
+		g_signal_connect(GTK_ENTRY(entry), "activate", G_CALLBACK(name_entry_activate_cb), sd);
+
+		gtk_popover_set_position(GTK_POPOVER(name_popover), GTK_POS_BOTTOM);
+		gq_gtk_container_add(GTK_WIDGET(name_popover), entry);
+		gtk_container_set_border_width(GTK_CONTAINER(name_popover), 6);
+
+		gtk_file_chooser_set_create_folders(GTK_FILE_CHOOSER(dialog), TRUE);
+
+		g_signal_connect(dialog, "response", G_CALLBACK(bar_sort_add_response_cb), sd);
+
+		gq_gtk_widget_show_all(GTK_WIDGET(dialog));
 		}
-
-	hbox = pref_box_new(GENERIC_DIALOG(sd->dialog)->vbox, FALSE, GTK_ORIENTATION_HORIZONTAL, PREF_PAD_GAP);
-
-	pref_label_new(hbox, _("Name:"));
-
-	sd->dialog_name_entry = gtk_entry_new();
-	gq_gtk_box_pack_start(GTK_BOX(hbox), sd->dialog_name_entry, TRUE, TRUE, 0);
-	generic_dialog_attach_default(GENERIC_DIALOG(sd->dialog), sd->dialog_name_entry);
-	gtk_widget_show(sd->dialog_name_entry);
-
-	if (sd->mode == BAR_SORT_MODE_COLLECTION)
+	else
 		{
-		gtk_widget_grab_focus(sd->dialog_name_entry);
-		}
+		GtkWidget *dialog = gtk_file_chooser_dialog_new(_("Create empty Collection file"), GTK_WINDOW(sd->lw->window), GTK_FILE_CHOOSER_ACTION_SAVE, _("_Cancel"), GTK_RESPONSE_CANCEL, _("_Save"), GTK_RESPONSE_ACCEPT, nullptr);
 
-	gtk_widget_show(GENERIC_DIALOG(sd->dialog)->dialog);
+		GtkFileFilter *all_filter = gtk_file_filter_new();
+		gtk_file_filter_set_name(all_filter, _("All files"));
+		gtk_file_filter_add_pattern(all_filter, "*");
+		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), all_filter);
+
+		GtkFileFilter *collections_filter = gtk_file_filter_new();
+		gtk_file_filter_set_name(collections_filter, _("Collections files"));
+		gtk_file_filter_add_pattern(collections_filter, "*.gqv");
+		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), collections_filter);
+
+		gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), collections_filter);
+
+#if HAVE_GTK4
+		g_autoptr(GFile) path = g_file_new_for_path(get_collections_dir(), nullptr);
+		gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(dialog), path, nullptr);
+#else
+		gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(dialog), get_collections_dir(), nullptr);
+#endif
+
+		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), _("Untitled.gqv"));
+		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), get_collections_dir());
+
+		g_signal_connect(dialog, "response", G_CALLBACK(new_collection_file_response_cb), sd);
+
+		gtk_window_present(GTK_WINDOW(dialog));
+		}
 }
 
 void bar_sort_close(GtkWidget *bar)
@@ -610,8 +657,6 @@ static void bar_sort_destroy(gpointer data)
 {
 	auto sd = static_cast<SortData *>(data);
 
-	bar_sort_add_close(sd);
-
 	g_free(sd->filter_key);
 	g_list_free_full(sd->undo_src_list, g_free);
 	g_list_free_full(sd->undo_dest_list, g_free);
@@ -628,8 +673,6 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 	GtkWidget *label;
 	GtkWidget *tbar;
 	GtkWidget *combo;
-	GList *editors_list;
-	GList *work;
 	gboolean have_filter;
 	GtkWidget *button;
 
@@ -685,35 +728,27 @@ static GtkWidget *bar_sort_new(LayoutWindow *lw, SortActionType action,
 
 
 	have_filter = FALSE;
-	editors_list = editor_list_get();
-	work = editors_list;
-	while (work)
+	EditorsList editors_list = editor_list_get();
+	for (const EditorDescription *editor : editors_list)
 		{
-		GtkWidget *button;
-		auto editor = static_cast<EditorDescription *>(work->data);
-		gchar *key;
-		gboolean select = FALSE;
-
-		work = work->next;
-
 		if (!editor_is_filter(editor->key)) continue;
 
-		key = g_strdup(editor->key);
-		if (sd->action == BAR_SORT_FILTER && strcmp(key, filter_key) == 0)
+		gchar *key = g_strdup(editor->key);
+
+		gboolean select = (sd->action == BAR_SORT_FILTER && strcmp(key, filter_key) == 0);
+		if (select)
 			{
 			bar_sort_set_action(sd, sd->action, key);
-			select = TRUE;
 			have_filter = TRUE;
 			}
 
-		button = pref_radiobutton_new(sd->folder_group, buttongrp,
-					      editor->name, select,
-					      G_CALLBACK(bar_sort_set_filter_cb), sd);
+		GtkWidget *button = pref_radiobutton_new(sd->folder_group, buttongrp,
+		                                         editor->name, select,
+		                                         G_CALLBACK(bar_sort_set_filter_cb), sd);
 		g_signal_connect(G_OBJECT(button), "button_press_event", G_CALLBACK(bar_filter_message_cb), NULL);
 
 		g_object_set_data_full(G_OBJECT(button), "filter_key", key, g_free);
 		}
-	g_list_free(editors_list);
 
 	if (sd->action == BAR_SORT_FILTER && !have_filter) sd->action = BAR_SORT_COPY;
 
@@ -792,7 +827,7 @@ void bar_sort_cold_start(LayoutWindow *lw, const gchar **attribute_names, const
 		if (READ_INT_CLAMP_FULL("selection", selection, 0, BAR_SORT_SELECTION_COUNT - 1)) continue;
 		if (READ_CHAR_FULL("filter_key", filter_key)) continue;
 
-		log_printf("unknown attribute %s = %s\n", option, value);
+		config_file_error((std::string("Unknown attribute: ") + option + " = " + value).c_str());
 		}
 
 	lw->options.action = static_cast<SortActionType>(action);
@@ -816,7 +851,7 @@ void bar_sort_write_config(GtkWidget *bar, GString *outstr, gint indent)
 	if (!sd) return;
 
 	WRITE_NL(); WRITE_STRING("<bar_sort ");
-	write_bool_option(outstr, indent, "enabled", gtk_widget_get_visible(bar));
+	write_bool_option(outstr, "enabled", gtk_widget_get_visible(bar));
 	WRITE_INT(*sd, mode);
 	WRITE_INT(*sd, action);
 	WRITE_INT(*sd, selection);
