@@ -29,7 +29,7 @@
 #include <gdk/gdk.h>
 #include <glib-object.h>
 
-#if defined(__linux__)
+#if defined(__GLIBC__)
 #include <mntent.h>
 #else
 #include <sys/mount.h>
@@ -38,7 +38,6 @@
 #include <config.h>
 
 #include "collect.h"
-#include "debug.h"
 #include "filedata.h"
 #include "intl.h"
 #include "layout-util.h"
@@ -49,31 +48,73 @@
 #include "ui-fileops.h"
 #include "ui-utildlg.h"
 
-#define GQ_COLLECTION_MARKER "#" GQ_APPNAME
-
-enum {
-	GQ_COLLECTION_FAIL_MIN =     300,
-	GQ_COLLECTION_FAIL_PERCENT = 98,
-	GQ_COLLECTION_READ_BUFSIZE = 4096
-};
-
-struct CollectManagerEntry;
-
-static void collection_load_thumb_step(CollectionData *cd);
-static gboolean collection_save_private(CollectionData *cd, const gchar *path);
-
-static CollectManagerEntry *collect_manager_get_entry(const gchar *path);
-static void collect_manager_entry_reset(CollectManagerEntry *entry);
-static gint collect_manager_process_action(CollectManagerEntry *entry, gchar **path_ptr);
+#ifdef __NetBSD__
+#define statfs statvfs
+#endif
 
 namespace
 {
 
+#if defined(__GLIBC__)
+using MFILE = FILE;
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(MFILE, endmntent)
+#endif
+
+constexpr guint GQ_COLLECTION_FAIL_MIN = 300;
+constexpr guint GQ_COLLECTION_FAIL_PERCENT = 98;
+constexpr size_t GQ_COLLECTION_READ_BUFSIZE = 4096;
+
+#define GQ_COLLECTION_MARKER "#" GQ_APPNAME
 const size_t GQ_COLLECTION_MARKER_LEN = strlen(GQ_COLLECTION_MARKER);
 
 constexpr gchar gqview_collection_marker[] = "#GQview collection";
 const size_t gqview_collection_marker_len = strlen(gqview_collection_marker);
 
+constexpr gint COLLECT_MANAGER_ACTIONS_PER_IDLE = 1000;
+constexpr guint COLLECT_MANAGER_FLUSH_DELAY = 10000;
+
+struct CollectManagerEntry
+{
+	gchar *path;
+	GList *add_list;
+	GHashTable *oldpath_hash;
+	GHashTable *newpath_hash;
+	gboolean empty;
+};
+
+enum CollectManagerType {
+	COLLECTION_MANAGER_UPDATE,
+	COLLECTION_MANAGER_ADD,
+	COLLECTION_MANAGER_REMOVE
+};
+
+struct CollectManagerAction
+{
+	gchar *oldpath;
+	gchar *newpath;
+
+	CollectManagerType type;
+
+	gint ref;
+};
+
+
+GList *collection_manager_entry_list = nullptr;
+GList *collection_manager_action_list = nullptr;
+GList *collection_manager_action_tail = nullptr;
+
+CollectManagerEntry *collect_manager_get_entry(const gchar *path)
+{
+	const auto collect_manager_entry_compare_path = [](gconstpointer data, gconstpointer user_data)
+	{
+		return strcmp(static_cast<const CollectManagerEntry *>(data)->path, static_cast<const gchar *>(user_data));
+	};
+
+	GList *work = g_list_find_custom(collection_manager_entry_list, path, collect_manager_entry_compare_path);
+
+	return work ? static_cast<CollectManagerEntry *>(work->data) : nullptr;
+}
+
 gboolean scan_geometry(gchar *buffer, GdkRectangle &window)
 {
 	gint nx;
@@ -91,13 +132,61 @@ gboolean scan_geometry(gchar *buffer, GdkRectangle &window)
 	return TRUE;
 }
 
+bool is_file_on_mounted_drive(const gchar *filename)
+{
+	const auto is_file_in_dir = [filename](const gchar *dirname)
+	{
+		return (g_strcmp0(dirname, G_DIR_SEPARATOR_S) != 0) &&
+		       g_str_has_prefix(filename, dirname);
+	};
+
+#if defined(__GLIBC__)
+	g_autoptr(MFILE) mount_entries = setmntent("/proc/mounts", "r");
+	if (mount_entries == nullptr)
+		{
+		/* It is assumed this will never fail */
+		perror("setmntent");
+		exit(EXIT_FAILURE);
+		}
+
+	struct mntent *mount_entry;
+	while (nullptr != (mount_entry = getmntent(mount_entries)))
+		{
+		if (is_file_in_dir(mount_entry->mnt_dir))
+			return true;
+		}
+#else
+	struct statfs* mounts;
+	int num_mounts = getmntinfo(&mounts, MNT_NOWAIT);
+
+	if (num_mounts < 0)
+		{
+		/* It is assumed this will never fail */
+		perror("getmntinfo");
+		exit(EXIT_FAILURE);
+		}
+
+	for (int i = 0; i < num_mounts; i++)
+		{
+		if (is_file_in_dir(mounts[i].f_mntonname))
+			return true;
+		}
+#endif
+	return false;
+}
+
 } // namespace
 
+static void collection_load_thumb_step(CollectionData *cd);
+static gboolean collection_save_private(CollectionData *cd, const gchar *path);
+
+static void collect_manager_entry_reset(CollectManagerEntry *entry);
+static gint collect_manager_process_action(CollectManagerEntry *entry, gchar **path_ptr);
+
 static gboolean collection_load_private(CollectionData *cd, const gchar *path, CollectionLoadFlags flags)
 {
 	gchar s_buf[GQ_COLLECTION_READ_BUFSIZE];
 	FILE *f;
-	gchar *pathl;
 	gboolean limit_failures = TRUE;
 	gboolean success = TRUE;
 	gboolean has_official_header = FALSE;
@@ -111,8 +200,6 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 	guint flush = !!(flags & COLLECTION_LOAD_FLUSH);
 	guint append = !!(flags & COLLECTION_LOAD_APPEND);
 	guint only_geometry = !!(flags & COLLECTION_LOAD_GEOMETRY);
-	gboolean reading_extended_filename = FALSE;
-	gchar *buffer2;
 
 	if (!only_geometry)
 		{
@@ -134,26 +221,25 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 
 	if (!path) path = cd->path;
 
-	pathl = path_from_utf8(path);
+	g_autofree gchar *pathl = path_from_utf8(path);
 
 	DEBUG_1("collection load: append=%d flush=%d only_geometry=%d path=%s", append, flush, only_geometry, pathl);
 
 	/* load it */
 	f = fopen(pathl, "r");
-	g_free(pathl);
 	if (!f)
 		{
 		log_printf("Failed to open collection file: \"%s\"\n", path);
 		return FALSE;
 		}
 
-	GString *extended_filename_buffer = g_string_new(nullptr);
+	g_autoptr(GString) extended_filename_buffer = nullptr;
 	while (fgets(s_buf, sizeof(s_buf), f))
 		{
 		gchar *buf;
 		gchar *p = s_buf;
 
-		if (!reading_extended_filename)
+		if (!extended_filename_buffer)
 			{
 			/* Skip whitespaces and empty lines */
 			while (*p && g_ascii_isspace(*p)) p++;
@@ -196,7 +282,7 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 		/* Read filenames */
 		/** @todo This is not safe! */
 		/* Updated: anything within double quotes is considered a filename */
-		if (!reading_extended_filename)
+		if (!extended_filename_buffer)
 			{
 			/* not yet reading filename */
 			while (*p && *p != '"') p++;
@@ -207,8 +293,7 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 			if (p[0] != '"')
 				{
 				/* first part of extended filename */
-				g_string_append(extended_filename_buffer, buf);
-				reading_extended_filename = TRUE;
+				extended_filename_buffer = g_string_new(buf);
 				continue;
 				}
 			}
@@ -225,122 +310,62 @@ static gboolean collection_load_private(CollectionData *cd, const gchar *path, C
 
 			/* end of extended filename found */
 			g_string_append_len(extended_filename_buffer, buf, p - buf);
-			reading_extended_filename = FALSE;
 			}
 
-		if (extended_filename_buffer->len > 0)
+		g_autofree gchar *filename = nullptr;
+		if (extended_filename_buffer)
 			{
-			buffer2 = g_strdup(extended_filename_buffer->str);
-			g_string_erase(extended_filename_buffer, 0, -1);
+			filename = g_string_free(extended_filename_buffer, FALSE);
+			extended_filename_buffer = nullptr;
 			}
 		else
 			{
 			*p = 0;
-			buffer2 = g_strdup(buf);
+			filename = g_strdup(buf);
 			}
 
-		if (*buffer2)
+		if (!*filename) continue;
+
+		total++;
+
+		if (!flush)
+			changed |= collect_manager_process_action(entry, &filename);
+
+		if (filename[0] == G_DIR_SEPARATOR && collection_add_check(cd, file_data_new_simple(filename), FALSE, TRUE)) continue;
+
+		log_printf("Warning: Collection: %s Invalid file: %s", cd->name, filename);
+		DEBUG_1("collection invalid file: %s", filename);
+
+		/* If the file path has the prefix home, tmp or usr it was on the local file system and has been deleted. Ignore it. */
+		if (!g_str_has_prefix(filename, "/home") && !g_str_has_prefix(filename, "/tmp") && !g_str_has_prefix(filename, "/usr"))
 			{
-			gboolean valid;
-			gboolean found = FALSE;
-
-			if (!flush)
-				changed |= collect_manager_process_action(entry, &buffer2);
-
-			valid = (buffer2[0] == G_DIR_SEPARATOR && collection_add_check(cd, file_data_new_simple(buffer2), FALSE, TRUE));
-			if (!valid)
+			/* The file was on a mounted drive and either has been deleted or the drive is not mounted */
+			if (!is_file_on_mounted_drive(filename))
 				{
-				log_printf("Warning: Collection: %s Invalid file: %s", cd->name, buffer2);
-				DEBUG_1("collection invalid file: %s", buffer2);
+				log_printf("%s is a file on an unmounted filesystem: %s", filename, cd->path);
+				g_autofree gchar *text = g_strdup_printf(_("This Collection cannot be opened because it contains a link to a file on a drive which is not yet mounted.\n\nCollection: %s\nFile: %s\n"),
+				                                         cd->path, filename);
+				warning_dialog(_("Cannot open Collection"), text, GQ_ICON_DIALOG_WARNING, nullptr);
+
+				success = FALSE;
+				break;
 				}
 
-			total++;
-			if (!valid)
-				{
-				/* If the file path has the prefix home, tmp or usr it was on the local file system and has been deleted. Ignore it. */
-				if (!g_str_has_prefix(buffer2, "/home") && !g_str_has_prefix(buffer2, "/tmp") && !g_str_has_prefix(buffer2, "/usr"))
-					{
-					/* The file was on a mounted drive and either has been deleted or the drive is not mounted */
-#if defined(__linux__)
-					struct mntent *mount_entry;
-					FILE *mount_entries;
-
-					mount_entries = setmntent("/proc/mounts", "r");
-					if (mount_entries == nullptr)
-						{
-						/* It is assumed this will never fail */
-						perror("setmntent");
-						exit(EXIT_FAILURE);
-						}
-
-					while (nullptr != (mount_entry = getmntent(mount_entries)))
-						{
-						if (g_strcmp0(mount_entry->mnt_dir, G_DIR_SEPARATOR_S) != 0)
-							{
-							if (g_str_has_prefix(buffer2, mount_entry->mnt_dir))
-								{
-								log_printf("%s was a file on a mounted filesystem but has been deleted: %s", buffer2, cd->name);
-								found = TRUE;
-								break;
-								}
-							}
-						}
-					endmntent(mount_entries);
-#else
-					struct statfs* mounts;
-					int num_mounts = getmntinfo(&mounts, MNT_NOWAIT);
-
-					if (num_mounts < 0)
-						{
-						/* It is assumed this will never fail */
-						perror("setmntent");
-						exit(EXIT_FAILURE);
-						}
-
-					for (int i = 0; i < num_mounts; i++)
-						{
-						if (g_strcmp0(mounts[i].f_mntonname, G_DIR_SEPARATOR_S) != 0)
-							{
-							if (g_str_has_prefix(buffer2, mounts[i].f_mntonname))
-								{
-								log_printf("%s was a file on a mounted filesystem but has been deleted: %s", buffer2, cd->name);
-								found = TRUE;
-								break;
-								}
-							}
-						}
-#endif
-
-					if (!found)
-						{
-						log_printf("%s is a file on an unmounted filesystem: %s", buffer2, cd->path);
-						gchar *text = g_strdup_printf(_("This Collection cannot be opened because it contains a link to a file on a drive which is not yet mounted.\n\nCollection: %s\nFile: %s\n"), cd->path, buffer2);
-						warning_dialog(_("Cannot open Collection"), text, GQ_ICON_DIALOG_WARNING, nullptr);
-						g_free(text);
-
-						collection_window_close_by_collection(cd);
-						success = FALSE;
-						break;
-						}
-					}
-				else
-					{
-					log_printf("%s was a file on local filesystem but has been deleted: %s", buffer2, cd->name);
-					}
-
-				fail++;
-				if (limit_failures && fail > GQ_COLLECTION_FAIL_MIN && fail * 100 / total > GQ_COLLECTION_FAIL_PERCENT)
-					{
-					log_printf("%u invalid filenames in unofficial collection file, closing: %s\n", fail, path);
-					success = FALSE;
-					break;
-					}
-				}
+			log_printf("%s was a file on a mounted filesystem but has been deleted: %s", filename, cd->name);
+			}
+		else
+			{
+			log_printf("%s was a file on local filesystem but has been deleted: %s", filename, cd->name);
 			}
-		g_free(buffer2);
-		}
 
-	g_string_free(extended_filename_buffer, TRUE);
+		fail++;
+		if (limit_failures && fail > GQ_COLLECTION_FAIL_MIN && fail * 100 / total > GQ_COLLECTION_FAIL_PERCENT)
+			{
+			log_printf("%u invalid filenames in unofficial collection file, closing: %s\n", fail, path);
+			success = FALSE;
+			break;
+			}
+		}
 
 	DEBUG_1("collection files: total = %u fail = %u official=%d gqview=%d geometry=%d", total, fail, has_official_header, has_gqview_header, has_geometry_header);
 
@@ -489,8 +514,6 @@ void collection_load_stop(CollectionData *cd)
 static gboolean collection_save_private(CollectionData *cd, const gchar *path)
 {
 	SecureSaveInfo *ssi;
-	GList *work;
-	gchar *pathl;
 
 	if (!path && !cd->path) return FALSE;
 
@@ -499,10 +522,8 @@ static gboolean collection_save_private(CollectionData *cd, const gchar *path)
 		path = cd->path;
 		}
 
-
-	pathl = path_from_utf8(path);
+	g_autofree gchar *pathl = path_from_utf8(path);
 	ssi = secure_open(pathl);
-	g_free(pathl);
 	if (!ssi)
 		{
 		log_printf(_("failed to open collection (write) \"%s\"\n"), path);
@@ -518,29 +539,26 @@ static gboolean collection_save_private(CollectionData *cd, const gchar *path)
 		secure_fprintf(ssi, "#geometry: %d %d %d %d\n", cd->window.x, cd->window.y, cd->window.width, cd->window.height);
 		}
 
-	work = cd->list;
-	while (work && secsave_errno == SS_ERR_NONE)
+	for (GList *work = cd->list; work && secsave_succeed(); work = work->next)
 		{
 		auto ci = static_cast<CollectInfo *>(work->data);
 		secure_fprintf(ssi, "\"%s\"\n", ci->fd->path);
-		work = work->next;
 		}
 
 	secure_fprintf(ssi, "#end\n");
 
 	if (secure_close(ssi))
 		{
-		log_printf(_("error saving collection file: %s\nerror: %s\n"), path,
-			    secsave_strerror(secsave_errno));
+		log_printf(_("error saving collection file: %s\nerror: %s\n"),
+		           path, secsave_strerror());
 		return FALSE;
 		}
 
 	if (!cd->path || strcmp(path, cd->path) != 0)
 		{
-		gchar *buf = cd->path;
+		g_autofree gchar *buf = cd->path;
 		cd->path = g_strdup(path);
 		path = cd->path;
-		g_free(buf);
 
 		g_free(cd->name);
 		cd->name = g_strdup(filename_from_path(cd->path));
@@ -576,43 +594,6 @@ gboolean collection_load_only_geometry(CollectionData *cd, const gchar *path)
  *-------------------------------------------------------------------
  */
 
-enum {
-	COLLECT_MANAGER_ACTIONS_PER_IDLE = 1000,
-	COLLECT_MANAGER_FLUSH_DELAY =      10000
-};
-
-struct CollectManagerEntry
-{
-	gchar *path;
-	GList *add_list;
-	GHashTable *oldpath_hash;
-	GHashTable *newpath_hash;
-	gboolean empty;
-};
-
-enum CollectManagerType {
-	COLLECTION_MANAGER_UPDATE,
-	COLLECTION_MANAGER_ADD,
-	COLLECTION_MANAGER_REMOVE
-};
-
-struct CollectManagerAction
-{
-	gchar *oldpath;
-	gchar *newpath;
-
-	CollectManagerType type;
-
-	gint ref;
-};
-
-
-static GList *collection_manager_entry_list = nullptr;
-static GList *collection_manager_action_list = nullptr;
-static GList *collection_manager_action_tail = nullptr;
-static guint collection_manager_timer_id = 0; /* event source id */
-
-
 static CollectManagerAction *collect_manager_action_new(const gchar *oldpath, const gchar *newpath,
 							CollectManagerType type)
 {
@@ -697,18 +678,6 @@ static void collect_manager_entry_reset(CollectManagerEntry *entry)
 	collect_manager_entry_init_data(entry);
 }
 
-static CollectManagerEntry *collect_manager_get_entry(const gchar *path)
-{
-	const auto collect_manager_entry_compare_path = [](gconstpointer data, gconstpointer user_data)
-	{
-		return strcmp(static_cast<const CollectManagerEntry *>(data)->path, static_cast<const gchar *>(user_data));
-	};
-
-	GList *work = g_list_find_custom(collection_manager_entry_list, path, collect_manager_entry_compare_path);
-
-	return work ? static_cast<CollectManagerEntry *>(work->data) : nullptr;
-}
-
 static void collect_manager_entry_add_action(CollectManagerEntry *entry, CollectManagerAction *action)
 {
 
@@ -975,18 +944,22 @@ static gboolean collect_manager_process_cb(gpointer)
 	return G_SOURCE_REMOVE;
 }
 
-static gboolean collect_manager_timer_cb(gpointer)
+static gboolean collect_manager_timer_cb(gpointer user_data)
 {
 	DEBUG_1("collection manager timer expired");
 
 	g_idle_add_full(G_PRIORITY_LOW, collect_manager_process_cb, nullptr, nullptr);
 
-	collection_manager_timer_id = 0;
-	return FALSE;
+	auto *collection_manager_timer_id = static_cast<guint *>(user_data);
+	*collection_manager_timer_id = 0;
+
+	return G_SOURCE_REMOVE;
 }
 
 static void collect_manager_timer_push(gint stop)
 {
+	static guint collection_manager_timer_id = 0; /* event source id */
+
 	if (collection_manager_timer_id)
 		{
 		if (!stop) return;
@@ -998,7 +971,7 @@ static void collect_manager_timer_push(gint stop)
 	if (!stop)
 		{
 		collection_manager_timer_id = g_timeout_add(COLLECT_MANAGER_FLUSH_DELAY,
-							    collect_manager_timer_cb, nullptr);
+		                                            collect_manager_timer_cb, &collection_manager_timer_id);
 		DEBUG_1("collection manager timer started");
 		}
 }
