@@ -34,7 +34,6 @@
 #include "collect-io.h"
 #include "collect-table.h"
 #include "compat.h"
-#include "debug.h"
 #include "filedata.h"
 #include "img-view.h"
 #include "intl.h"
@@ -53,10 +52,11 @@
 #include "utilops.h"
 #include "window.h"
 
-enum {
-	COLLECT_DEF_WIDTH = 440,
-	COLLECT_DEF_HEIGHT = 450
-};
+namespace
+{
+
+constexpr gint COLLECT_DEF_WIDTH = 440;
+constexpr gint COLLECT_DEF_HEIGHT = 450;
 
 /**
  *  list of paths to collections */
@@ -66,14 +66,16 @@ enum {
  *
  * Type ::_CollectionData
  */
-static GList *collection_list = nullptr;
+GList *collection_list = nullptr;
 
 /**
  * @brief  List of currently open Collection windows.
  *
  * Type ::_CollectWindow
  */
-static GList *collection_window_list = nullptr;
+GList *collection_window_list = nullptr;
+
+} // namespace
 
 static void collection_window_get_geometry(CollectWindow *cw);
 static void collection_window_refresh(CollectWindow *cw);
@@ -330,7 +332,6 @@ CollectWindow *collection_window_find_by_path(const gchar *path)
 gchar *collection_path(const gchar *param)
 {
 	gchar *path = nullptr;
-	gchar *full_name = nullptr;
 
 	if (file_extension_match(param, GQ_COLLECTION_EXT))
 		{
@@ -338,7 +339,7 @@ gchar *collection_path(const gchar *param)
 		}
 	else if (file_extension_match(param, nullptr))
 		{
-		full_name = g_strconcat(param, GQ_COLLECTION_EXT, NULL);
+		g_autofree gchar *full_name = g_strconcat(param, GQ_COLLECTION_EXT, NULL);
 		path = g_build_filename(get_collections_dir(), full_name, NULL);
 		}
 
@@ -348,7 +349,6 @@ gchar *collection_path(const gchar *param)
 		path = nullptr;
 		}
 
-	g_free(full_name);
 	return path;
 }
 
@@ -361,15 +361,9 @@ gchar *collection_path(const gchar *param)
  */
 gboolean is_collection(const gchar *param)
 {
-	gchar *name = nullptr;
+	g_autofree gchar *name = collection_path(param);
 
-	name = collection_path(param);
-	if (name)
-		{
-		g_free(name);
-		return TRUE;
-		}
-	return FALSE;
+	return name != nullptr;
 }
 
 /**
@@ -379,32 +373,25 @@ gboolean is_collection(const gchar *param)
  *
  *
  */
-void collection_contents(const gchar *name, GString **contents)
+GString *collection_contents(const gchar *name, GString *contents)
 {
-	gchar *path;
-	CollectionData *cd;
-	CollectInfo *ci;
-	GList *work;
-	FileData *fd;
+	if (!is_collection(name)) return contents;
 
-	if (is_collection(name))
+	CollectionData *cd = collection_new("");
+	g_autofree gchar *path = collection_path(name);
+	collection_load(cd, path, COLLECTION_LOAD_APPEND);
+
+	for (GList *work = cd->list; work; work = work->next)
 		{
-		path = collection_path(name);
-		cd = collection_new("");
-		collection_load(cd, path, COLLECTION_LOAD_APPEND);
-		work = cd->list;
-		while (work)
-			{
-			ci = static_cast<CollectInfo *>(work->data);
-			fd = ci->fd;
-			*contents = g_string_append(*contents, fd->path);
-			*contents = g_string_append(*contents, "\n");
+		auto *ci = static_cast<CollectInfo *>(work->data);
 
-			work = work->next;
-			}
-		g_free(path);
-		collection_free(cd);
+		contents = g_string_append(contents, ci->fd->path);
+		contents = g_string_append(contents, "\n");
 		}
+
+	collection_free(cd);
+
+	return contents;
 }
 
 /**
@@ -415,29 +402,27 @@ void collection_contents(const gchar *name, GString **contents)
  */
 GList *collection_contents_fd(const gchar *name)
 {
-	gchar *path;
 	CollectionData *cd;
 	CollectInfo *ci;
 	GList *work;
 	GList *list = nullptr;
 
-	if (is_collection(name))
-		{
-		path = collection_path(name);
-		cd = collection_new("");
-		collection_load(cd, path, COLLECTION_LOAD_APPEND);
-		work = cd->list;
-		while (work)
-			{
-			ci = static_cast<CollectInfo *>(work->data);
-			list = g_list_append(list, ci->fd);
+	if (!is_collection(name)) return nullptr;
 
-			work = work->next;
-			}
-		g_free(path);
-		collection_free(cd);
+	g_autofree gchar *path = collection_path(name);
+	cd = collection_new("");
+	collection_load(cd, path, COLLECTION_LOAD_APPEND);
+	work = cd->list;
+	while (work)
+		{
+		ci = static_cast<CollectInfo *>(work->data);
+		list = g_list_append(list, ci->fd);
+
+		work = work->next;
 		}
 
+	collection_free(cd);
+
 	return list;
 }
 
@@ -565,7 +550,7 @@ CollectionData *collection_from_number(gint n)
 
 	if (!list && !info_list) return cd;
 
-	GStrv numbers = g_strsplit(data, "\n", -1);
+	g_auto(GStrv) numbers = g_strsplit(data, "\n", -1);
 	for (gint i = 1; numbers[i] != nullptr; i++)
 		{
 		if (!numbers[i + 1]) break; // numbers[i] is data after last \n, skip it
@@ -578,7 +563,6 @@ CollectionData *collection_from_number(gint n)
 		if (info_list) *info_list = g_list_append(*info_list, info);
 		}
 
-	g_strfreev(numbers);
 	return cd;
 }
 
@@ -990,12 +974,7 @@ static gboolean collection_window_keypress(GtkWidget *, GdkEventKey *event, gpoi
 			case 'P': case 'p':
 				if (event->state & GDK_SHIFT_MASK)
 					{
-					CollectInfo *info;
-
-					info = collection_table_get_focus_info(cw->table);
-
-					print_window_new(info->fd, collection_table_selection_get_list(cw->table),
-							 collection_list_to_filelist(cw->cd->list), cw->window);
+					print_window_new(collection_table_selection_get_list(cw->table), cw->window);
 					}
 				else
 					{
@@ -1064,7 +1043,6 @@ static void collection_window_update_title(CollectWindow *cw)
 {
 	gboolean free_name = FALSE;
 	gchar *name;
-	gchar *buf;
 
 	if (!cw) return;
 
@@ -1078,10 +1056,9 @@ static void collection_window_update_title(CollectWindow *cw)
 		name = cw->cd->name;
 		}
 
-	buf = g_strdup_printf(_("%s - Collection - %s"), name, GQ_APPNAME);
+	g_autofree gchar *buf = g_strdup_printf(_("%s - Collection - %s"), name, GQ_APPNAME);
 	if (free_name) g_free(name);
 	gtk_window_set_title(GTK_WINDOW(cw->window), buf);
-	g_free(buf);
 }
 
 static void collection_window_update_info(CollectionData *, CollectInfo *ci, gpointer data)
@@ -1152,10 +1129,8 @@ static void collection_close_save_cb(GenericDialog *gd, gpointer data)
 
 	if (!collection_save(cw->cd, cw->cd->path))
 		{
-		gchar *buf;
-		buf = g_strdup_printf(_("Failed to save the collection:\n%s"), cw->cd->path);
+		g_autofree gchar *buf = g_strdup_printf(_("Failed to save the collection:\n%s"), cw->cd->path);
 		warning_dialog(_("Save Failed"), buf, GQ_ICON_DIALOG_ERROR, cw->window);
-		g_free(buf);
 		return;
 		}
 
@@ -1361,7 +1336,15 @@ CollectWindow *collection_window_new(const gchar *path)
 
 	collection_set_update_info_func(cw->cd, collection_window_update_info, cw);
 
-	if (path && *path == G_DIR_SEPARATOR) collection_load_begin(cw->cd, nullptr, COLLECTION_LOAD_NONE);
+	if (path && *path == G_DIR_SEPARATOR)
+		{
+		if (!collection_load_begin(cw->cd, nullptr, COLLECTION_LOAD_NONE))
+			{
+			collection_window_close(cw);
+
+			return nullptr;
+			}
+		}
 
 	return cw;
 }
