@@ -25,7 +25,6 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <glib.h>
 
-class FileData;
 struct ImageWindow;
 
 enum ColorManProfileType {
@@ -75,6 +74,7 @@ void color_man_correct_region(ColorMan *cm, GdkPixbuf *pixbuf, gint x, gint y, g
 
 gboolean color_man_get_status(ColorMan *cm, gchar **image_profile, gchar **screen_profile);
 
-guchar *heif_color_profile(FileData *fd, guint *profile_len);
+const gchar *get_profile_name(const guchar *profile_data, guint profile_len);
+
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
