@@ -20,8 +20,10 @@
 #include <config.h>
 
 #ifdef __linux__
+#ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE
 #endif
+#endif
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -33,17 +35,9 @@
 
 #include <glib.h>
 
-#if HAVE_LCMS
-/*** color support enabled ***/
-#  if HAVE_LCMS2
-#    include <lcms2.h>
-#  else
-#    include <lcms.h>
-#  endif
-#endif
-
 #include "cache.h"
-#include "debug.h"
+#include "color-man-heif.h"
+#include "exif-int.h" /* Required to prevent clang-tidy warnings */
 #include "exif.h"
 #include "filecache.h"
 #include "filedata.h"
@@ -55,6 +49,7 @@
 #include "third-party/zonedetect.h"
 #include "typedefs.h"
 #include "ui-fileops.h"
+
 struct ExifData;
 struct ExifItem;
 struct FileCacheData;
@@ -78,14 +73,6 @@ static gdouble exif_get_rational_as_double(ExifData *exif, const gchar *key)
 	return exif_rational_to_double(r, sign);
 }
 
-static GString *append_comma_text(GString *string, const gchar *text)
-{
-	string = g_string_append(string, ", ");
-	string = g_string_append(string, text);
-
-	return string;
-}
-
 static gchar *remove_common_prefix(gchar *s, gchar *t)
 {
 	gint i;
@@ -155,10 +142,9 @@ static gboolean remove_suffix(gchar *str, const gchar *suffix, gint suffix_len)
 
 static gchar *exif_build_formatted_Camera(ExifData *exif)
 {
-	gchar *text;
-	gchar *make = exif_get_data_as_text(exif, "Exif.Image.Make");
-	gchar *model = exif_get_data_as_text(exif, "Exif.Image.Model");
-	gchar *software = exif_get_data_as_text(exif, "Exif.Image.Software");
+	g_autofree gchar *make = exif_get_data_as_text(exif, "Exif.Image.Make");
+	g_autofree gchar *model = exif_get_data_as_text(exif, "Exif.Image.Model");
+	g_autofree gchar *software = exif_get_data_as_text(exif, "Exif.Image.Software");
 	gchar *model2;
 	gchar *software2;
 
@@ -194,118 +180,70 @@ static gchar *exif_build_formatted_Camera(ExifData *exif)
 	model2 = remove_common_prefix(make, model);
 	software2 = remove_common_prefix(model2, software);
 
-	text = g_strdup_printf("%s%s%s%s%s%s", (make) ? make : "", (make && model2) ? " " : "",
-					       (model2) ? model2 : "",
-					       (software2 && (make || model2)) ? " (" : "",
-					       (software2) ? software2 : "",
-					       (software2 && (make || model2)) ? ")" : "");
+	return g_strdup_printf("%s%s%s%s%s%s",
+	                       make ? make : "",
+	                       (make && model2) ? " " : "",
+	                       model2 ? model2 : "",
+	                       (software2 && (make || model2)) ? " (" : "",
+	                       software2 ? software2 : "",
+	                       (software2 && (make || model2)) ? ")" : "");
+}
+
+static gchar *exif_build_formatted_DateTime(ExifData *exif, const gchar *text_key, const gchar *subsec_key)
+{
+	g_autofree gchar *subsec = nullptr;
+
+	gchar *text = exif_get_data_as_text(exif, text_key);
+	if (text)
+		{
+		subsec = exif_get_data_as_text(exif, subsec_key);
+		}
+	else
+		{
+		text = exif_get_data_as_text(exif, "Exif.Image.DateTime");
+		if (!text) return nullptr;
+
+		subsec = exif_get_data_as_text(exif, "Exif.Photo.SubSecTime");
+		}
+
+	/* Convert the stuff into a tm struct */
+	std::tm tm{}; /* Uh, strptime could let garbage in tm! */
+	if (strptime(text, "%Y:%m:%d %H:%M:%S", &tm))
+		{
+		gchar buf[128];
+		const gint buflen = strftime(buf, sizeof(buf), "%x %X", &tm);
+		if (buflen > 0)
+			{
+			g_autoptr(GError) error = nullptr;
+			g_autofree gchar *tmp = g_locale_to_utf8(buf, buflen, nullptr, nullptr, &error);
+			if (error)
+				{
+				log_printf("Error converting locale strftime to UTF-8: %s\n", error->message);
+				}
+			else
+				{
+				std::swap(text, tmp);
+				}
+			}
+		}
+
+	if (subsec)
+		{
+		g_autofree gchar *tmp = text;
+		text = g_strconcat(tmp, ".", subsec, NULL);
+		}
 
-	g_free(make);
-	g_free(model);
-	g_free(software);
 	return text;
 }
 
 static gchar *exif_build_formatted_DateTime(ExifData *exif)
 {
-	gchar *text = exif_get_data_as_text(exif, "Exif.Photo.DateTimeOriginal");
-	gchar *subsec = nullptr;
-	gchar buf[128];
-	gchar *tmp;
-	gint buflen;
-	struct tm tm;
-	GError *error = nullptr;
-
-	if (text)
-		{
-		subsec = exif_get_data_as_text(exif, "Exif.Photo.SubSecTimeOriginal");
-		}
-	else
-		{
-		text = exif_get_data_as_text(exif, "Exif.Image.DateTime");
-		if (text) subsec = exif_get_data_as_text(exif, "Exif.Photo.SubSecTime");
-		}
-
-	/* Convert the stuff into a tm struct */
-	memset(&tm, 0, sizeof(tm)); /* Uh, strptime could let garbage in tm! */
-	if (text && strptime(text, "%Y:%m:%d %H:%M:%S", &tm))
-		{
-		buflen = strftime(buf, sizeof(buf), "%x %X", &tm);
-		if (buflen > 0)
-			{
-			tmp = g_locale_to_utf8(buf, buflen, nullptr, nullptr, &error);
-			if (error)
-				{
-				log_printf("Error converting locale strftime to UTF-8: %s\n", error->message);
-				g_error_free(error);
-				}
-			else
-				{
-				g_free(text);
-				text = g_strdup(tmp);
-				}
-			}
-		}
-
-	if (subsec)
-		{
-		tmp = text;
-		text = g_strconcat(tmp, ".", subsec, NULL);
-		g_free(tmp);
-		g_free(subsec);
-		}
-	return text;
+	return exif_build_formatted_DateTime(exif, "Exif.Photo.DateTimeOriginal", "Exif.Photo.SubSecTimeOriginal");
 }
 
 static gchar *exif_build_formatted_DateTimeDigitized(ExifData *exif)
 {
-	gchar *text = exif_get_data_as_text(exif, "Exif.Photo.DateTimeDigitized");
-	gchar *subsec = nullptr;
-	gchar buf[128];
-	gchar *tmp;
-	gint buflen;
-	struct tm tm;
-	GError *error = nullptr;
-
-	if (text)
-		{
-		subsec = exif_get_data_as_text(exif, "Exif.Photo.SubSecTimeDigitized");
-		}
-	else
-		{
-		text = exif_get_data_as_text(exif, "Exif.Image.DateTime");
-		if (text) subsec = exif_get_data_as_text(exif, "Exif.Photo.SubSecTime");
-		}
-
-	/* Convert the stuff into a tm struct */
-	memset(&tm, 0, sizeof(tm)); /* Uh, strptime could let garbage in tm! */
-	if (text && strptime(text, "%Y:%m:%d %H:%M:%S", &tm))
-		{
-		buflen = strftime(buf, sizeof(buf), "%x %X", &tm);
-		if (buflen > 0)
-			{
-			tmp = g_locale_to_utf8(buf, buflen, nullptr, nullptr, &error);
-			if (error)
-				{
-				log_printf("Error converting locale strftime to UTF-8: %s\n", error->message);
-				g_error_free(error);
-				}
-			else
-				{
-				g_free(text);
-				text = g_strdup(tmp);
-				}
-			}
-		}
-
-	if (subsec)
-		{
-		tmp = text;
-		text = g_strconcat(tmp, ".", subsec, NULL);
-		g_free(tmp);
-		g_free(subsec);
-		}
-	return text;
+	return exif_build_formatted_DateTime(exif, "Exif.Photo.DateTimeDigitized", "Exif.Photo.SubSecTimeDigitized");
 }
 
 static gchar *exif_build_formatted_ShutterSpeed(ExifData *exif)
@@ -435,7 +373,7 @@ static gchar *exif_build_formatted_Flash(ExifData *exif)
 
 	/* flash mode (bits 3, 4) */
 	v = (n >> 3) & 0x03;
-	if (v) string = append_comma_text(string, _("mode:"));
+	if (v) g_string_append_printf(string, ", %s", _("mode:"));
 	switch (v)
 		{
 		case 1:
@@ -453,13 +391,13 @@ static gchar *exif_build_formatted_Flash(ExifData *exif)
 
 	/* return light (bits 1, 2) */
 	v = (n >> 1) & 0x03;
-	if (v == 2) string = append_comma_text(string, _("not detected by strobe"));
-	if (v == 3) string = append_comma_text(string, _("detected by strobe"));
+	if (v == 2) g_string_append_printf(string, ", %s", _("not detected by strobe"));
+	if (v == 3) g_string_append_printf(string, ", %s", _("detected by strobe"));
 
 	/* we ignore flash function (bit 5) */
 
 	/* red-eye (bit 6) */
-	if ((n >> 5) & 0x01) string = append_comma_text(string, _("red-eye reduction"));
+	if ((n >> 5) & 0x01) g_string_append_printf(string, ", %s", _("red-eye reduction"));
 
 	return g_string_free(string, FALSE);
 }
@@ -468,41 +406,34 @@ static gchar *exif_build_formatted_Resolution(ExifData *exif)
 {
 	ExifRational *rx;
 	ExifRational *ry;
-	gchar *units;
-	gchar *text;
 
 	rx = exif_get_rational(exif, "Exif.Image.XResolution", nullptr);
 	ry = exif_get_rational(exif, "Exif.Image.YResolution", nullptr);
 	if (!rx || !ry) return nullptr;
 
-	units = exif_get_data_as_text(exif, "Exif.Image.ResolutionUnit");
-	text = g_strdup_printf("%0.f x %0.f (%s/%s)", rx->den ? static_cast<gdouble>(rx->num) / rx->den : 1.0,
-						      ry->den ? static_cast<gdouble>(ry->num) / ry->den : 1.0,
-						      _("dot"), (units) ? units : _("unknown"));
-
-	g_free(units);
-	return text;
+	g_autofree gchar *units = exif_get_data_as_text(exif, "Exif.Image.ResolutionUnit");
+	return g_strdup_printf("%0.f x %0.f (%s/%s)",
+	                       rx->den ? static_cast<gdouble>(rx->num) / rx->den : 1.0,
+	                       ry->den ? static_cast<gdouble>(ry->num) / ry->den : 1.0,
+	                       _("dot"),
+	                       units ? units : _("unknown"));
 }
 
 static gchar *exif_build_formatted_ColorProfile(ExifData *exif)
 {
-#if HAVE_LCMS2
-	cmsUInt8Number profileID[17];
-#endif
 	const gchar *name = "";
 	const gchar *source = "";
-	guchar *profile_data;
 	guint profile_len;
 
-	profile_data = exif_get_color_profile(exif, &profile_len);
+	g_autofree guchar *profile_data = exif_get_color_profile(exif, &profile_len);
 	if (!profile_data)
 		{
 		gint cs;
-		gchar *interop_index;
 
 		/* ColorSpace == 1 specifies sRGB per EXIF 2.2 */
 		if (!exif_get_integer(exif, "Exif.Photo.ColorSpace", &cs)) cs = 0;
-		interop_index = exif_get_data_as_text(exif, "Exif.Iop.InteroperabilityIndex");
+
+		g_autofree gchar *interop_index = exif_get_data_as_text(exif, "Exif.Iop.InteroperabilityIndex");
 
 		if (cs == 1)
 			{
@@ -514,89 +445,54 @@ static gchar *exif_build_formatted_ColorProfile(ExifData *exif)
 			name = _("AdobeRGB");
 			source = (cs == 2) ? "ColorSpace" : "Iop";
 			}
-
-		g_free(interop_index);
 		}
 	else
 		{
+		const gchar *profile_name = get_profile_name(profile_data, profile_len);
+		if (profile_name) name = profile_name;
+
 		source = _("embedded");
-#if HAVE_LCMS
-
-			{
-			cmsHPROFILE profile;
-
-			profile = cmsOpenProfileFromMem(profile_data, profile_len);
-			if (profile)
-				{
-#if HAVE_LCMS2
-				profileID[16] = '\0';
-				cmsGetHeaderProfileID(profile, profileID);
-				name = reinterpret_cast<gchar *>(profileID);
-#else
-				name = (gchar *) cmsTakeProductName(profile);
-#endif
-				cmsCloseProfile(profile);
-				}
-			g_free(profile_data);
-			}
-#endif
 		}
+
 	if (name[0] == 0 && source[0] == 0) return nullptr;
 	return g_strdup_printf("%s (%s)", name, source);
 }
 
 static gchar *exif_build_formatted_GPSPosition(ExifData *exif)
 {
-	GString *string;
-	gchar *ref;
-	ExifRational *value;
-	ExifItem *item;
-	guint i;
-	gdouble p;
-	gdouble p3;
-	gulong p1;
-	gulong p2;
+	GString *string = g_string_new("");
 
-	string = g_string_new("");
+	const auto build = [exif, string](const gchar *item_key, const gchar *ref_key)
+	{
+		ExifItem *item = exif_get_item(exif, item_key);
+		if (!item) return;
 
-	item = exif_get_item(exif, "Exif.GPSInfo.GPSLatitude");
-	ref = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLatitudeRef");
-	if (item && ref)
-		{
-		p = 0;
-		for (i = 0; i < exif_item_get_elements(item); i++)
+		g_autofree gchar *ref = exif_get_data_as_text(exif, ref_key);
+		if (!ref) return;
+
+		gdouble p = 0;
+		for (guint i = 0, elements = exif_item_get_elements(item); i < elements; i++)
 			{
-			value = exif_item_get_rational(item, nullptr, i);
+			ExifRational *value = exif_item_get_rational(item, nullptr, i);
 			if (value && value->num && value->den)
 				p += static_cast<gdouble>(value->num) / static_cast<gdouble>(value->den) / pow(60.0, static_cast<gdouble>(i));
 			}
-		p1 = static_cast<gint>(p);
-		p2 = static_cast<gint>((p - p1)*60);
-		p3 = ((p - p1)*60 - p2)*60;
+
+		const gulong p1 = static_cast<gint>(p);
+		const gulong p2 = static_cast<gint>((p - p1)*60);
+		const gdouble p3 = ((p - p1)*60 - p2)*60;
+
+		if (string->len > 0)
+			g_string_append(string, ", ");
 
 		g_string_append_printf(string, "%0lu° %0lu' %0.2f\" %.1s", p1, p2, p3, ref);
-		} // if (item && ref)
+	};
 
-	item = exif_get_item(exif, "Exif.GPSInfo.GPSLongitude");
-	ref = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLongitudeRef");
-	if (item && ref)
-		{
-		p = 0;
-		for (i = 0; i < exif_item_get_elements(item); i++)
-			{
-			value = exif_item_get_rational(item, nullptr, i);
-			if (value && value->num && value->den)
-			p += static_cast<gdouble>(value->num) / static_cast<gdouble>(value->den) / pow(60.0, static_cast<gdouble>(i));
-			}
-		p1 = static_cast<gint>(p);
-		p2 = static_cast<gint>((p - p1)*60);
-		p3 = ((p - p1)*60 - p2)*60;
-
-		g_string_append_printf(string, ", %0lu° %0lu' %0.2f\" %.1s", p1, p2, p3, ref);
-		} // if (item && ref)
+	build("Exif.GPSInfo.GPSLatitude", "Exif.GPSInfo.GPSLatitudeRef");
+	build("Exif.GPSInfo.GPSLongitude", "Exif.GPSInfo.GPSLongitudeRef");
 
 	return g_string_free(string, FALSE);
-} // static gchar *exif_build_forma...
+}
 
 static gchar *exif_build_formatted_GPSAltitude(ExifData *exif)
 {
@@ -628,8 +524,8 @@ static gchar *exif_build_formatted_GPSAltitude(ExifData *exif)
  */
 static void zd_tz(ZoneDetectResult *results, gchar **timezone, gchar **countryname, gchar **countryalpha2)
 {
-	gchar *timezone_pre = nullptr;
-	gchar *timezone_id = nullptr;
+	g_autofree gchar *timezone_pre = nullptr;
+	g_autofree gchar *timezone_id = nullptr;
 	unsigned int index = 0;
 
 	while(results[index].lookupResult != ZD_LOOKUP_END)
@@ -660,8 +556,6 @@ static void zd_tz(ZoneDetectResult *results, gchar **timezone, gchar **countryna
 		}
 
 	*timezone = g_strconcat(timezone_pre, timezone_id, NULL);
-	g_free(timezone_pre);
-	g_free(timezone_id);
 }
 
 static void ZoneDetect_onError(int errZD, int errNative)
@@ -684,27 +578,18 @@ static gboolean exif_build_tz_data(ExifData *exif, gchar **exif_date_time, gchar
 {
 	gfloat latitude;
 	gfloat longitude;
-	gchar *text_latitude;
-	gchar *text_longitude;
-	gchar *text_latitude_ref;
-	gchar *text_longitude_ref;
-	gchar *text_date;
-	gchar *text_time;
 	gchar *lat_deg;
 	gchar *lat_min;
 	gchar *lon_deg;
 	gchar *lon_min;
-	gchar *timezone_path;
 	ZoneDetect *cd;
 	ZoneDetectResult *results;
 	gboolean ret = FALSE;
 
-	text_latitude = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLatitude");
-	text_longitude = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLongitude");
-	text_latitude_ref = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLatitudeRef");
-	text_longitude_ref = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLongitudeRef");
-	text_date = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSDateStamp");
-	text_time = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSTimeStamp");
+	g_autofree gchar *text_latitude = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLatitude");
+	g_autofree gchar *text_longitude = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLongitude");
+	g_autofree gchar *text_latitude_ref = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLatitudeRef");
+	g_autofree gchar *text_longitude_ref = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSLongitudeRef");
 
 	if (text_latitude && text_longitude && text_latitude_ref && text_longitude_ref)
 		{
@@ -731,7 +616,7 @@ static gboolean exif_build_tz_data(ExifData *exif, gchar **exif_date_time, gchar
 			longitude = -longitude;
 			}
 
-		timezone_path = g_build_filename(get_rc_dir(), TIMEZONE_DATABASE_FILE, NULL);
+		g_autofree gchar *timezone_path = g_build_filename(get_rc_dir(), TIMEZONE_DATABASE_FILE, NULL);
 		if (g_file_test(timezone_path, G_FILE_TEST_EXISTS))
 			{
 			ZDSetErrorHandler(ZoneDetect_onError);
@@ -751,17 +636,23 @@ static gboolean exif_build_tz_data(ExifData *exif, gchar **exif_date_time, gchar
 				}
 			ZDCloseDatabase(cd);
 			}
-		g_free(timezone_path);
 		}
 
-	if (ret && text_date && text_time)
+	if (ret)
 		{
-		*exif_date_time = g_strconcat(text_date, ":", text_time, NULL);
-		}
-	else
-		{
-		ret = FALSE;
+		g_autofree gchar *text_date = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSDateStamp");
+		g_autofree gchar *text_time = exif_get_data_as_text(exif, "Exif.GPSInfo.GPSTimeStamp");
+
+		if (text_date && text_time)
+			{
+			*exif_date_time = g_strconcat(text_date, ":", text_time, NULL);
+			}
+		else
+			{
+			ret = FALSE;
+			}
 		}
+
 	return ret;
 }
 
@@ -779,23 +670,19 @@ static gboolean exif_build_tz_data(ExifData *exif, gchar **exif_date_time, gchar
 static gchar *exif_build_formatted_localtime(ExifData *exif)
 {
 	gchar buf[128];
-	gchar *tmp;
 	gint buflen;
-	GError *error = nullptr;
-	gchar *time_zone_image;
-	gchar *time_zone_org;
 	struct tm *tm_local;
 	struct tm tm_utc;
 	time_t stamp;
 	gchar *exif_date_time = nullptr;
-	gchar *timezone = nullptr;
-	gchar *countryname = nullptr;
-	gchar *countryalpha2 = nullptr;
+	g_autofree gchar *timezone = nullptr;
+	g_autofree gchar *countryname = nullptr;
+	g_autofree gchar *countryalpha2 = nullptr;
 
 	if (exif_build_tz_data(exif, &exif_date_time, &timezone, &countryname, &countryalpha2))
 		{
-		time_zone_image = g_strconcat("TZ=", timezone, NULL);
-		time_zone_org = g_strconcat("TZ=", getenv("TZ"), NULL);
+		g_autofree gchar *time_zone_image = g_strconcat("TZ=", timezone, NULL);
+		g_autofree gchar *time_zone_org = g_strconcat("TZ=", getenv("TZ"), NULL);
 		setenv("TZ", "UTC", TRUE);
 
 		memset(&tm_utc, 0, sizeof(tm_utc));
@@ -810,29 +697,22 @@ static gchar *exif_build_formatted_localtime(ExifData *exif)
 			buflen = strftime(buf, sizeof(buf), "%x %X", tm_local);
 			if (buflen > 0)
 				{
-				tmp = g_locale_to_utf8(buf, buflen, nullptr, nullptr, &error);
+				g_autoptr(GError) error = nullptr;
+				g_autofree gchar *tmp = g_locale_to_utf8(buf, buflen, nullptr, nullptr, &error);
 				if (error)
 					{
 					log_printf("Error converting locale strftime to UTF-8: %s\n", error->message);
-					g_error_free(error);
 					}
 				else
 					{
-					g_free(exif_date_time);
-					exif_date_time = tmp;
+					std::swap(exif_date_time, tmp);
 					}
 				}
 			}
+
 		putenv(time_zone_org);
-
-		g_free(time_zone_image);
-		g_free(time_zone_org);
 		}
 
-	g_free(timezone);
-	g_free(countryname);
-	g_free(countryalpha2);
-
 	return exif_date_time;
 }
 
@@ -845,17 +725,13 @@ static gchar *exif_build_formatted_localtime(ExifData *exif)
  */
 static gchar *exif_build_formatted_timezone(ExifData *exif)
 {
-	gchar *exif_date_time = nullptr;
+	g_autofree gchar *exif_date_time = nullptr;
 	gchar *timezone = nullptr;
-	gchar *countryname = nullptr;
-	gchar *countryalpha2 = nullptr;
+	g_autofree gchar *countryname = nullptr;
+	g_autofree gchar *countryalpha2 = nullptr;
 
 	exif_build_tz_data(exif, &exif_date_time, &timezone, &countryname, &countryalpha2);
 
-	g_free(exif_date_time);
-	g_free(countryname);
-	g_free(countryalpha2);
-
 	return timezone;
 }
 
@@ -868,17 +744,13 @@ static gchar *exif_build_formatted_timezone(ExifData *exif)
  */
 static gchar *exif_build_formatted_countryname(ExifData *exif)
 {
-	gchar *exif_date_time = nullptr;
-	gchar *timezone = nullptr;
+	g_autofree gchar *exif_date_time = nullptr;
+	g_autofree gchar *timezone = nullptr;
 	gchar *countryname = nullptr;
-	gchar *countryalpha2 = nullptr;
+	g_autofree gchar *countryalpha2 = nullptr;
 
 	exif_build_tz_data(exif, &exif_date_time, &timezone, &countryname, &countryalpha2);
 
-	g_free(exif_date_time);
-	g_free(timezone);
-	g_free(countryalpha2);
-
 	return countryname;
 }
 
@@ -891,17 +763,13 @@ static gchar *exif_build_formatted_countryname(ExifData *exif)
  */
 static gchar *exif_build_formatted_countrycode(ExifData *exif)
 {
-	gchar *exif_date_time = nullptr;
-	gchar *timezone = nullptr;
-	gchar *countryname = nullptr;
+	g_autofree gchar *exif_date_time = nullptr;
+	g_autofree gchar *timezone = nullptr;
+	g_autofree gchar *countryname = nullptr;
 	gchar *countryalpha2 = nullptr;
 
 	exif_build_tz_data(exif, &exif_date_time, &timezone, &countryname, &countryalpha2);
 
-	g_free(exif_date_time);
-	g_free(timezone);
-	g_free(countryname);
-
 	return countryalpha2;
 }
 
@@ -1035,8 +903,6 @@ static void exif_init_cache()
 
 ExifData *exif_read_fd(FileData *fd)
 {
-	gchar *sidecar_path;
-
 	if (!exif_cache) exif_init_cache();
 
 	if (!fd) return nullptr;
@@ -1046,7 +912,7 @@ ExifData *exif_read_fd(FileData *fd)
 
 	/* CACHE_TYPE_XMP_METADATA file should exist only if the metadata are
 	 * not writable directly, thus it should contain the most up-to-date version */
-	sidecar_path = nullptr;
+	g_autofree gchar *sidecar_path = nullptr;
 
 #if HAVE_EXIV2
 	/* we are not able to handle XMP sidecars without exiv2 */
@@ -1057,7 +923,6 @@ ExifData *exif_read_fd(FileData *fd)
 
 	fd->exif = exif_read(fd->path, sidecar_path, fd->modified_xmp);
 
-	g_free(sidecar_path);
 	file_cache_put(exif_cache, fd, 1);
 	return fd->exif;
 }
@@ -1069,6 +934,73 @@ void exif_free_fd(FileData *fd, ExifData *exif)
 	g_assert(fd->exif == exif);
 }
 
+guchar *exif_get_color_profile(FileData *fd, guint &profile_len, ColorManProfileType &color_profile_from_image)
+{
+	color_profile_from_image = COLOR_PROFILE_NONE;
+
+	ExifData *exif = exif_read_fd(fd);
+	if (!exif) return nullptr;
+
+	guchar *profile = nullptr;
+
+	if (g_strcmp0(fd->format_name, "heif") == 0)
+		{
+		profile = heif_color_profile(fd->path, profile_len);
+		}
+
+	if (!profile)
+		{
+		profile = exif_get_color_profile(exif, &profile_len);
+		}
+
+	if (profile)
+		{
+		DEBUG_1("Found embedded color profile");
+		color_profile_from_image = COLOR_PROFILE_MEM;
+		}
+	else
+		{
+		g_autofree gchar *interop_index = exif_get_data_as_text(exif, "Exif.Iop.InteroperabilityIndex");
+
+		if (interop_index)
+			{
+			/* Exif 2.21 specification */
+			if (!strcmp(interop_index, "R98"))
+				{
+				color_profile_from_image = COLOR_PROFILE_SRGB;
+				DEBUG_1("Found EXIF 2.21 ColorSpace of sRGB");
+				}
+			else if (!strcmp(interop_index, "R03"))
+				{
+				color_profile_from_image = COLOR_PROFILE_ADOBERGB;
+				DEBUG_1("Found EXIF 2.21 ColorSpace of AdobeRGB");
+				}
+			}
+		else
+			{
+			gint cs;
+
+			/* ColorSpace == 1 specifies sRGB per EXIF 2.2 */
+			if (!exif_get_integer(exif, "Exif.Photo.ColorSpace", &cs)) cs = 0;
+			if (cs == 1)
+				{
+				color_profile_from_image = COLOR_PROFILE_SRGB;
+				DEBUG_1("Found EXIF 2.2 ColorSpace of sRGB");
+				}
+			else if (cs == 2)
+				{
+				/* non-standard way of specifying AdobeRGB (used by some software) */
+				color_profile_from_image = COLOR_PROFILE_ADOBERGB;
+				DEBUG_1("Found EXIF 2.2 ColorSpace of AdobeRGB");
+				}
+			}
+		}
+
+	exif_free_fd(fd, exif);
+
+	return profile;
+}
+
 /* embedded icc in jpeg */
 
 gboolean exif_jpeg_parse_color(ExifData *exif, guchar *data, guint size)
@@ -1086,10 +1018,10 @@ gboolean exif_jpeg_parse_color(ExifData *exif, guchar *data, guint size)
 	 */
 
 	while (jpeg_segment_find(data + seg_offset + seg_length,
-				      size - seg_offset - seg_length,
-				      JPEG_MARKER_APP2,
-				      "ICC_PROFILE\x00", 12,
-				      &seg_offset, &seg_length))
+	                         size - seg_offset - seg_length,
+	                         JPEG_MARKER_APP2,
+	                         "ICC_PROFILE\x00", 12,
+	                         seg_offset, seg_length))
 		{
 		guchar chunk_num;
 		guchar chunk_tot;
@@ -1120,25 +1052,24 @@ gboolean exif_jpeg_parse_color(ExifData *exif, guchar *data, guint size)
 
 	if (chunk_count > 0)
 		{
-		guchar *cp_data;
 		guint cp_length = 0;
 		guint i;
 
 		for (i = 0; i < chunk_count; i++) cp_length += chunk_length[i];
-		cp_data = static_cast<guchar *>(g_malloc(cp_length));
+
+		g_autofree auto *cp_data = static_cast<guchar *>(g_malloc(cp_length));
 
 		for (i = 0; i < chunk_count; i++)
 			{
 			if (chunk_offset[i] == 0)
 				{
 				/* error, we never saw this chunk */
-				g_free(cp_data);
 				return FALSE;
 				}
 			memcpy(cp_data, data + chunk_offset[i], chunk_length[i]);
 			}
 		DEBUG_1("Found embedded icc profile in jpeg");
-		exif_add_jpeg_color_profile(exif, cp_data, cp_length);
+		exif_add_jpeg_color_profile(exif, g_steal_pointer(&cp_data), cp_length);
 
 		return TRUE;
 		}
@@ -1248,25 +1179,13 @@ gchar *metadata_file_info(FileData *fd, const gchar *key, MetadataFormat)
 #if HAVE_LUA
 gchar *metadata_lua_info(FileData *fd, const gchar *key, MetadataFormat)
 {
-	gchar *script_name;
-	gchar *script_name_utf8;
-	gchar *data;
-	gchar *raw_data;
-	gchar *valid_data;
+	g_autofree gchar *script_name_utf8 = g_strdup(key + 4);
+	g_autofree gchar *script_name = path_from_utf8(script_name_utf8);
 
-	script_name_utf8 = g_strdup(key + 4);
-	script_name = path_from_utf8(script_name_utf8);
+	g_autofree gchar *raw_data = lua_callvalue(fd, script_name, nullptr);
+	g_autofree gchar *valid_data = g_utf8_make_valid(raw_data, -1);
 
-	raw_data = lua_callvalue(fd, script_name, nullptr);
-	valid_data = g_utf8_make_valid(raw_data, -1);
-	data = g_utf8_substring(valid_data, 0, 150);
-
-	g_free(script_name);
-	g_free(script_name_utf8);
-	g_free(raw_data);
-	g_free(valid_data);
-
-	return data;
+	return g_utf8_substring(valid_data, 0, 150);
 }
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
