@@ -47,6 +47,7 @@
 
 #include "exif.h"
 
+#include <algorithm>
 #include <fcntl.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
@@ -58,7 +59,6 @@
 #include <glib.h>
 #include <glib/gprintf.h>
 
-#include "debug.h"
 #include "exif-int.h"
 #include "format-raw.h"
 #include "intl.h"
@@ -92,9 +92,11 @@ ExifFormatAttrib ExifFormatList[] = {
 };
 
 /* tags that are special, or need special treatment */
-#define TAG_EXIFOFFSET          0x8769
-#define TAG_EXIFMAKERNOTE	0x927c
-#define TAG_GPSOFFSET		0x8825
+enum {
+TAG_EXIFOFFSET = 0x8769,
+TAG_EXIFMAKERNOTE = 0x927c,
+TAG_GPSOFFSET = 0x8825
+};
 
 
 /*
@@ -315,7 +317,8 @@ Tag names should match to exiv2 keys, https://www.exiv2.org/metadata.html
 Tags that don't match are not supported by exiv2 and should not be used anywhere in the code
 */
 
-ExifMarker ExifKnownMarkersList[] = {
+/**< the known exif tags list */
+static ExifMarker ExifKnownMarkersList[] = {
 { 0x0100, EXIF_FORMAT_LONG_UNSIGNED, 1,		"Exif.Image.ImageWidth",	N_("Image Width"), nullptr },
 { 0x0101, EXIF_FORMAT_LONG_UNSIGNED, 1,		"Exif.Image.ImageLength",	N_("Image Height"), nullptr },
 { 0x0102, EXIF_FORMAT_SHORT_UNSIGNED, 1,	"Exif.Image.BitsPerSample",	N_("Bits per Sample/Pixel"), nullptr },
@@ -429,7 +432,7 @@ ExifMarker ExifKnownMarkersList[] = {
 EXIF_MARKER_LIST_END
 };
 
-ExifMarker ExifKnownGPSInfoMarkersList[] = {
+static ExifMarker ExifKnownGPSInfoMarkersList[] = {
         /* The following do not work at the moment as the tag value 0x0000 has a
          * special meaning. */
         /* { 0x0000, EXIF_FORMAT_BYTE, -1, "Exif.GPSInfo.GPSVersionID", NULL, NULL }, */
@@ -467,7 +470,8 @@ ExifMarker ExifKnownGPSInfoMarkersList[] = {
         EXIF_MARKER_LIST_END
 };
 
-ExifMarker ExifUnknownMarkersList[] = {
+/**< the unknown tags utilize this generic list */
+static ExifMarker ExifUnknownMarkersList[] = {
 { 0x0000, EXIF_FORMAT_UNKNOWN, 0,		"unknown",	nullptr, nullptr },
 { 0x0000, EXIF_FORMAT_BYTE_UNSIGNED, -1,	"unknown",	nullptr, nullptr },
 { 0x0000, EXIF_FORMAT_STRING, -1,		"unknown",	nullptr, nullptr },
@@ -675,9 +679,11 @@ guint16 exif_byte_get_int16(guchar *f, ExifByteOrder bo)
 	memcpy(&align_buf, f, sizeof(guint16));
 
 	if (bo == EXIF_BYTE_ORDER_INTEL)
+		{
 		return GUINT16_FROM_LE(align_buf);
-	else
-		return GUINT16_FROM_BE(align_buf);
+		}
+
+	return GUINT16_FROM_BE(align_buf);
 }
 
 guint32 exif_byte_get_int32(guchar *f, ExifByteOrder bo)
@@ -687,9 +693,11 @@ guint32 exif_byte_get_int32(guchar *f, ExifByteOrder bo)
 	memcpy(&align_buf, f, sizeof(guint32));
 
 	if (bo == EXIF_BYTE_ORDER_INTEL)
+		{
 		return GUINT32_FROM_LE(align_buf);
-	else
-		return GUINT32_FROM_BE(align_buf);
+		}
+
+	return GUINT32_FROM_BE(align_buf); // NOLINT(readability-isolate-declaration)
 }
 
 void exif_byte_put_int16(guchar *f, guint16 n, ExifByteOrder bo)
@@ -718,7 +726,7 @@ void exif_byte_put_int32(guchar *f, guint32 n, ExifByteOrder bo)
 		}
 	else
 		{
-		align_buf = GUINT32_TO_BE(n);
+		align_buf = GUINT32_TO_BE(n); // NOLINT(readability-isolate-declaration)
 		}
 
 	memcpy(f, &align_buf, sizeof(guint32));
@@ -769,9 +777,8 @@ void exif_item_copy_data(ExifItem *item, gpointer src, guint len,
 	if (!dest ||
 	    ExifFormatList[src_format].size * ne > len)
 		{
-		gchar *tag = exif_item_get_tag_name(item);
+		g_autofree gchar *tag = exif_item_get_tag_name(item);
 		log_printf("exif tag %s data size mismatch\n", tag);
-		g_free(tag);
 		return;
 		}
 
@@ -793,7 +800,7 @@ void exif_item_copy_data(ExifItem *item, gpointer src, guint len,
 		case EXIF_FORMAT_SHORT:
 			for (i = 0; i < ne; i++)
 				{
-				(static_cast<guint16 *>(dest))[i] = exif_byte_get_int16(static_cast<guchar *>(src) + i * bs, bo);
+				(static_cast<guint16 *>(dest))[i] = exif_byte_get_int16(static_cast<guchar *>(src) + (i * bs), bo);
 				}
 			break;
 		case EXIF_FORMAT_LONG_UNSIGNED:
@@ -808,7 +815,7 @@ void exif_item_copy_data(ExifItem *item, gpointer src, guint len,
 				for (i = 0; i < ne; i++)
 					{
 					(static_cast<gint32 *>(dest))[i] =
-						static_cast<gint32>(exif_byte_get_int16(static_cast<guchar *>(src) + i * ss, bo));
+						static_cast<gint32>(exif_byte_get_int16(static_cast<guchar *>(src) + (i * ss), bo));
 					}
 				}
 			else
@@ -816,7 +823,7 @@ void exif_item_copy_data(ExifItem *item, gpointer src, guint len,
 				for (i = 0; i < ne; i++)
 					{
 					(static_cast<gint32 *>(dest))[i] =
-						exif_byte_get_int32(static_cast<guchar *>(src) + i * bs, bo);
+						exif_byte_get_int32(static_cast<guchar *>(src) + (i * bs), bo);
 					}
 				}
 			break;
@@ -824,13 +831,13 @@ void exif_item_copy_data(ExifItem *item, gpointer src, guint len,
 		case EXIF_FORMAT_RATIONAL:
 			for (i = 0; i < ne; i++)
 				{
-				rational_from_data(&(static_cast<ExifRational *>(dest))[i], static_cast<guchar *>(src) + i * bs, bo);
+				rational_from_data(&(static_cast<ExifRational *>(dest))[i], static_cast<guchar *>(src) + (i * bs), bo);
 				}
 			break;
 		case EXIF_FORMAT_FLOAT:
 			for (i = 0; i < ne; i++)
 				{
-				(static_cast<float *>(dest))[i] = exif_byte_get_int32(static_cast<guchar *>(src) + i * bs, bo);
+				(static_cast<float *>(dest))[i] = exif_byte_get_int32(static_cast<guchar *>(src) + (i * bs), bo);
 				}
 			break;
 		case EXIF_FORMAT_DOUBLE:
@@ -838,7 +845,7 @@ void exif_item_copy_data(ExifItem *item, gpointer src, guint len,
 				{
 				ExifRational r;
 
-				rational_from_data(&r, static_cast<guchar *>(src) + i * bs, bo);
+				rational_from_data(&r, static_cast<guchar *>(src) + (i * bs), bo);
 				if (r.den) (static_cast<gdouble *>(dest))[i] = static_cast<gdouble>(r.num) / r.den;
 				}
 			break;
@@ -949,6 +956,8 @@ static gint exif_parse_IFD_entry(ExifData *exif, guchar *tiff, guint offset,
 			case TAG_EXIFMAKERNOTE:
 				format_exif_makernote_parse(exif, tiff, data_val, size, bo);
 				break;
+			default:
+				break;
 			}
 		}
 
@@ -978,7 +987,7 @@ gint exif_parse_IFD_table(ExifData *exif,
 
 	for (i = 0; i < count; i++)
 		{
-		exif_parse_IFD_entry(exif, tiff, offset + i * EXIF_TIFD_SIZE, size, bo, level, list);
+		exif_parse_IFD_entry(exif, tiff, offset + (i * EXIF_TIFD_SIZE), size, bo, level, list);
 		}
 
 	return 0;
@@ -1064,8 +1073,6 @@ static gint exif_jpeg_parse(ExifData *exif,
 			    guchar *data, guint size,
 			    ExifMarker *list)
 {
-	guint seg_offset = 0;
-	guint seg_length = 0;
 	gint res = -1;
 
 	if (size < 4 ||
@@ -1074,9 +1081,11 @@ static gint exif_jpeg_parse(ExifData *exif,
 		return -2;
 		}
 
+	guint seg_offset = 0;
+	guint seg_length = 0;
 	if (jpeg_segment_find(data, size, JPEG_MARKER_APP1,
-				   "Exif\x00\x00", 6,
-				   &seg_offset, &seg_length))
+	                      "Exif\x00\x00", 6,
+	                      seg_offset, seg_length))
 		{
 		res = exif_tiff_parse(exif, data + seg_offset + 6, seg_length - 6, list);
 		}
@@ -1189,9 +1198,9 @@ static gint unmap_file(gpointer mapping, gint size)
  * this function gives access to the original data from the image.
  * original data are part of the processed data and should not be freed separately
  */
-ExifData *exif_get_original(ExifData *processed)
+ExifData *exif_get_original(ExifData *exif)
 {
-	return processed;
+	return exif;
 }
 
 void exif_free(ExifData *exif)
@@ -1207,18 +1216,13 @@ ExifData *exif_read(gchar *path, gchar *, GHashTable *)
 {
 	ExifData *exif;
 	gpointer f;
-	gint size, res;
-	gchar *pathl;
+	gint res;
+	gint size;
 
 	if (!path) return nullptr;
 
-	pathl = path_from_utf8(path);
-	if (map_file(pathl, &f, &size) == -1)
-		{
-		g_free(pathl);
-		return nullptr;
-		}
-	g_free(pathl);
+	g_autofree gchar *pathl = path_from_utf8(path);
+	if (map_file(pathl, &f, &size) == -1) return nullptr;
 
 	exif = g_new0(ExifData, 1);
 	exif->path = g_strdup(path);
@@ -1291,8 +1295,7 @@ ExifItem *exif_get_item(ExifData *exif, const gchar *key)
 	return work ? static_cast<ExifItem*>(work->data) : nullptr;
 }
 
-#define EXIF_DATA_AS_TEXT_MAX_COUNT 16
-
+constexpr guint EXIF_DATA_AS_TEXT_MAX_COUNT = 16;
 
 static gchar *exif_item_get_data_as_text_full(ExifItem *item, MetadataFormat format)
 {
@@ -1308,8 +1311,7 @@ static gchar *exif_item_get_data_as_text_full(ExifItem *item, MetadataFormat for
 	if (!marker) return nullptr;
 
 	data = item->data;
-	ne = item->elements;
-	if (ne > EXIF_DATA_AS_TEXT_MAX_COUNT) ne = EXIF_DATA_AS_TEXT_MAX_COUNT;
+	ne = std::min(item->elements, EXIF_DATA_AS_TEXT_MAX_COUNT);
 	string = g_string_new("");
 	switch (item->format)
 		{
@@ -1320,7 +1322,6 @@ static gchar *exif_item_get_data_as_text_full(ExifItem *item, MetadataFormat for
 		case EXIF_FORMAT_UNDEFINED:
 			if (ne == 1 && marker->list && format == METADATA_FORMATTED)
 				{
-				gchar *result;
 				guchar val;
 
 				if (item->format == EXIF_FORMAT_BYTE_UNSIGNED ||
@@ -1333,9 +1334,8 @@ static gchar *exif_item_get_data_as_text_full(ExifItem *item, MetadataFormat for
 					val = static_cast<guchar>((static_cast<gchar *>(data))[0]);
 					}
 
-				result = exif_text_list_find_value(marker->list, static_cast<guint>(val));
+				g_autofree gchar *result = exif_text_list_find_value(marker->list, static_cast<guint>(val));
 				string = g_string_append(string, result);
-				g_free(result);
 				}
 			else
 				{
@@ -1348,11 +1348,8 @@ static gchar *exif_item_get_data_as_text_full(ExifItem *item, MetadataFormat for
 		case EXIF_FORMAT_SHORT_UNSIGNED:
 			if (ne == 1 && marker->list && format == METADATA_FORMATTED)
 				{
-				gchar *result;
-
-				result = exif_text_list_find_value(marker->list, (static_cast<guint16 *>(data))[0]);
+				g_autofree gchar *result = exif_text_list_find_value(marker->list, (static_cast<guint16 *>(data))[0]);
 				string = g_string_append(string, result);
-				g_free(result);
 				}
 			else for (i = 0; i < ne; i++)
 				{
@@ -1506,17 +1503,12 @@ gchar *exif_get_tag_description_by_key(const gchar *key)
 
 static void exif_write_item(FILE *f, ExifItem *item, ExifData *exif)
 {
-	gchar *text;
+	g_autofree gchar *text = exif_item_get_data_as_text(item, exif);
+	if (!text) return;
 
-	text = exif_item_get_data_as_text(item, exif);
-	if (text)
-		{
-		gchar *tag = exif_item_get_tag_name(item);
-		g_fprintf(f, "%4x %9s %30s %s\n", item->tag, ExifFormatList[item->format].short_name,
-			tag, text);
-		g_free(tag);
-		}
-	g_free(text);
+	g_autofree gchar *tag = exif_item_get_tag_name(item);
+	g_fprintf(f, "%4x %9s %30s %s\n",
+	          item->tag, ExifFormatList[item->format].short_name, tag, text);
 }
 
 /**
