@@ -24,7 +24,6 @@
 #include <cstdint>
 #include <cstring>
 #include <exception>
-#include <list>
 #include <memory>
 #include <string>
 #include <utility>
@@ -38,7 +37,6 @@
 #  include <libintl.h>
 #endif
 
-#include "debug.h"
 #include "filedata.h"
 #include "filefilter.h"
 #include "misc.h"
@@ -94,17 +92,20 @@ static constexpr AltKey alt_keys[] = {
 	{"Xmp.photoshop.CaptionWriter",		nullptr,				"Iptc.Application2.Writer"		},
 	};
 
+#ifdef DEBUG
 static void _debug_exception(const char* file,
                              int line,
                              const char* func,
                              Exiv2::AnyError& e)
 {
-	gchar *str = g_locale_from_utf8(e.what(), -1, nullptr, nullptr, nullptr);
+	g_autofree gchar *str = g_locale_from_utf8(e.what(), -1, nullptr, nullptr, nullptr);
 	DEBUG_1("%s:%d:%s:Exiv2: %s", file, line, func, str);
-	g_free(str);
 }
 
-#define debug_exception(e) _debug_exception(__FILE__, __LINE__, __func__, e)
+#  define debug_exception(e) _debug_exception(__FILE__, __LINE__, __func__, e)
+#else
+#  define debug_exception(e)
+#endif
 
 struct ExifData
 {
@@ -151,31 +152,23 @@ protected:
 	unsigned char *cp_data_;
 	guint cp_length_;
 	gboolean valid_;
-	gchar *pathl_;
 
 	Exiv2::ExifData emptyExifData_;
 	Exiv2::IptcData emptyIptcData_;
 	Exiv2::XmpData emptyXmpData_;
 
 public:
-	ExifDataOriginal(Exiv2::Image::AutoPtr image)
-	{
-		cp_data_ = nullptr;
-		cp_length_ = 0;
-        	image_ = std::move(image);
-		valid_ = TRUE;
-	}
-
-	ExifDataOriginal(gchar *path)
+	ExifDataOriginal(const gchar *path)
 	{
 		cp_data_ = nullptr;
 		cp_length_ = 0;
 		valid_ = TRUE;
 
-		pathl_ = path_from_utf8(path);
 		try
 			{
-			image_ = Exiv2::ImageFactory::open(pathl_);
+			g_autofree gchar *pathl = path_from_utf8(path);
+
+			image_ = Exiv2::ImageFactory::open(pathl);
 			image_->readMetadata();
 
 			if (image_->mimeType() == "application/rdf+xml")
@@ -209,7 +202,6 @@ public:
 	~ExifDataOriginal() override
 	{
 		g_free(cp_data_);
-		g_free(pathl_);
 	}
 
 	Exiv2::Image *image() override
@@ -347,12 +339,10 @@ public:
 			}
 		else
 			{
-			gchar *pathl = path_from_utf8(path);;
+			g_autofree gchar *pathl = path_from_utf8(path);;
 
 			auto sidecar = Exiv2::ImageFactory::create(Exiv2::ImageType::xmp, pathl);
 
-			g_free(pathl);
-
 			sidecar->setXmpData(xmpData_);
 			sidecar->writeMetadata();
 			}
@@ -673,7 +663,7 @@ guint exif_item_get_format_id(ExifItem *item)
 	try {
 		if (!item) return EXIF_FORMAT_UNKNOWN;
 		guint id = (reinterpret_cast<Exiv2::Metadatum *>(item))->typeId();
-		if (id >= (sizeof(format_id_trans_tbl) / sizeof(format_id_trans_tbl[0])) ) return EXIF_FORMAT_UNKNOWN;
+		if (id >= G_N_ELEMENTS(format_id_trans_tbl)) return EXIF_FORMAT_UNKNOWN;
 		return format_id_trans_tbl[id];
 	}
 	catch (Exiv2::AnyError& e) {
