@@ -22,6 +22,9 @@
 #ifndef FILEFILTER_H
 #define FILEFILTER_H
 
+#include <glib.h>
+
+#include "typedefs.h"
 
 struct FilterEntry {
 	gchar *key;
