@@ -19,36 +19,56 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "main.h"
-#include "image-load.h"
 #include "image-load-gdk.h"
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <glib-object.h>
+#include <glib.h>
+
 #include "filedata.h"
+#include "image-load.h"
 
+namespace
+{
 
-static gchar* image_loader_gdk_get_format_name(gpointer loader)
+struct ImageLoaderGdk : public ImageLoaderBackend
+{
+public:
+	~ImageLoaderGdk() override;
+
+	void init(AreaUpdatedCb area_updated_cb, SizePreparedCb size_prepared_cb, AreaPreparedCb area_prepared_cb, gpointer data) override;
+	void set_size(int width, int height) override;
+	gboolean write(const guchar *buf, gsize &chunk_size, gsize count, GError **error) override;
+	GdkPixbuf *get_pixbuf() override;
+	gboolean close(GError **error) override;
+	gchar *get_format_name() override;
+	gchar **get_format_mime_types() override;
+
+private:
+	GdkPixbufLoader *loader;
+};
+
+gchar *ImageLoaderGdk::get_format_name()
 {
 	GdkPixbufFormat *format;
 
-	format = gdk_pixbuf_loader_get_format(GDK_PIXBUF_LOADER(loader));
+	format = gdk_pixbuf_loader_get_format(loader);
 	if (format)
 		{
 		return gdk_pixbuf_format_get_name(format);
 		}
-	else
-		{
-		return nullptr;
-		}
-}
-static gchar** image_loader_gdk_get_format_mime_types(gpointer loader)
-{
-	return gdk_pixbuf_format_get_mime_types(gdk_pixbuf_loader_get_format(GDK_PIXBUF_LOADER(loader)));
+
+	return nullptr;
 }
 
-static gpointer image_loader_gdk_new(ImageLoaderBackendCbAreaUpdated area_updated_cb, ImageLoaderBackendCbSize size_cb, ImageLoaderBackendCbAreaPrepared area_prepared_cb, gpointer data)
+gchar **ImageLoaderGdk::get_format_mime_types()
+{
+	return gdk_pixbuf_format_get_mime_types(gdk_pixbuf_loader_get_format(loader));
+}
+
+void ImageLoaderGdk::init(AreaUpdatedCb area_updated_cb, SizePreparedCb size_prepared_cb, AreaPreparedCb area_prepared_cb, gpointer data)
 {
 	auto il = static_cast<ImageLoader *>(data);
-	GdkPixbufLoader *loader;
 
 	/** @FIXME gdk-pixbuf-loader does not recognize .xbm files unless
 	 * the mime type is given. There should be a general case */
@@ -62,34 +82,40 @@ static gpointer image_loader_gdk_new(ImageLoaderBackendCbAreaUpdated area_update
 		}
 
 	g_signal_connect(G_OBJECT(loader), "area_updated", G_CALLBACK(area_updated_cb), data);
-	g_signal_connect(G_OBJECT(loader), "size_prepared", G_CALLBACK(size_cb), data);
+	g_signal_connect(G_OBJECT(loader), "size_prepared", G_CALLBACK(size_prepared_cb), data);
 	g_signal_connect(G_OBJECT(loader), "area_prepared", G_CALLBACK(area_prepared_cb), data);
-	return loader;
 }
 
-static void image_loader_gdk_abort(gpointer)
+void ImageLoaderGdk::set_size(int width, int height)
 {
+	gdk_pixbuf_loader_set_size(loader, width, height);
 }
 
-static void image_loader_gdk_free(gpointer loader)
+gboolean ImageLoaderGdk::write(const guchar *buf, gsize &chunk_size, gsize, GError **error)
+{
+	return gdk_pixbuf_loader_write(loader, buf, chunk_size, error);
+}
+
+GdkPixbuf *ImageLoaderGdk::get_pixbuf()
+{
+	return gdk_pixbuf_loader_get_pixbuf(loader);
+}
+
+gboolean ImageLoaderGdk::close(GError **error)
+{
+	return gdk_pixbuf_loader_close(loader, error);
+}
+
+ImageLoaderGdk::~ImageLoaderGdk()
 {
 	g_object_unref(G_OBJECT(loader));
 }
 
-void image_loader_backend_set_default(ImageLoaderBackend *funcs)
-{
-	funcs->loader_new = image_loader_gdk_new;
-	funcs->set_size = reinterpret_cast<ImageLoaderBackendFuncSetSize>(gdk_pixbuf_loader_set_size);
-	funcs->load = nullptr;
-	funcs->write = reinterpret_cast<ImageLoaderBackendFuncWrite>(gdk_pixbuf_loader_write);
-	funcs->get_pixbuf = reinterpret_cast<ImageLoaderBackendFuncGetPixbuf>(gdk_pixbuf_loader_get_pixbuf);
-	funcs->close = reinterpret_cast<ImageLoaderBackendFuncClose>(gdk_pixbuf_loader_close);
-	funcs->abort = image_loader_gdk_abort;
-	funcs->free = image_loader_gdk_free;
+} // namespace
 
-	funcs->get_format_name = image_loader_gdk_get_format_name;
-	funcs->get_format_mime_types = image_loader_gdk_get_format_mime_types;
+std::unique_ptr<ImageLoaderBackend> get_image_loader_backend_default()
+{
+	return std::make_unique<ImageLoaderGdk>();
 }
 
-
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
