@@ -30,8 +30,8 @@
 #include "collect-table.h"
 #include "collect.h"
 #include "color-man.h"
+#include "compat-deprecated.h"
 #include "compat.h"
-#include "debug.h"
 #include "exif.h"
 #include "filecache.h"
 #include "filedata.h"
@@ -50,19 +50,77 @@
 struct ExifData;
 struct FileCacheData;
 
+namespace
+{
+
+constexpr gdouble aspect_ratios[5] {0.0, gdouble(1.0), gdouble(4.0) / 3, gdouble(3) / 2, gdouble(16) / 9};
+
+/*
+ * SelectionRectangle
+ */
+
+struct SelectionRectangle
+{
+	gint x = 0;
+	gint y = 0;
+	gint width = 0;
+	gint height = 0;
+	gdouble aspect_ratio = 0.0;
+
+	SelectionRectangle(gint origin_x = 0, gint origin_y = 0,
+	                   RectangleDrawAspectRatio rectangle_draw_aspect_ratio = RECTANGLE_DRAW_ASPECT_RATIO_NONE);
+	void set_cursor(gint cursor_x, gint cursor_y);
+
+private:
+	gint origin_x;
+	gint origin_y;
+	RectangleDrawAspectRatio rectangle_draw_aspect_ratio = RECTANGLE_DRAW_ASPECT_RATIO_NONE;
+};
+
+SelectionRectangle::SelectionRectangle(gint origin_x, gint origin_y, RectangleDrawAspectRatio rectangle_draw_aspect_ratio)
+    : aspect_ratio(aspect_ratios[static_cast<gint>(rectangle_draw_aspect_ratio)])
+    , origin_x(origin_x)
+    , origin_y(origin_y)
+    , rectangle_draw_aspect_ratio(rectangle_draw_aspect_ratio)
+{}
+
+void SelectionRectangle::set_cursor(gint cursor_x, gint cursor_y)
+{
+	x = std::min(origin_x, cursor_x);
+	y = std::min(origin_y, cursor_y);
+	width = std::abs(cursor_x - origin_x);
+	height = std::abs(cursor_y - origin_y);
+
+	if (rectangle_draw_aspect_ratio == RECTANGLE_DRAW_ASPECT_RATIO_NONE) return;
+
+	if (width < height)
+		width = height / aspect_ratio;
+	else
+		height = width / aspect_ratio;
+
+	// left side of the origin: move x to respect that origin
+	if (cursor_x < origin_x)
+		x = origin_x - width;
+
+	// above the origin: move y
+	if (cursor_y < origin_y)
+		y = origin_y - height;
+}
+
+// For draw rectangle function
+gint image_start_x;
+gint image_start_y;
+gint rect_x1, rect_x2, rect_y1, rect_y2;
+gint rect_id = 0;
+SelectionRectangle selection_rectangle;
+
+} // namespace
+
 static GList *image_list = nullptr;
 
 static void image_read_ahead_start(ImageWindow *imd);
 static void image_cache_set(ImageWindow *imd, FileData *fd);
 
-// For draw rectangle function
-static gint pixbuf_start_x;
-static gint pixbuf_start_y;
-static gint image_start_x;
-static gint image_start_y;
-static gint rect_x1, rect_x2, rect_y1, rect_y2;
-static gint rect_id = 0;
-
 /*
  *-------------------------------------------------------------------
  * 'signals'
@@ -159,29 +217,10 @@ static void image_press_cb(PixbufRenderer *pr, GdkEventButton *event, gpointer d
 	if(options->draw_rectangle)
 		{
 		pixbuf_renderer_get_mouse_position(pr, &x_pixel, &y_pixel);
-
-		pixbuf_start_x = event->x;
-		pixbuf_start_y = event->y;
-
-		if (x_pixel == -1)
-			{
-			image_start_x = 0;
-			}
-		else
-			{
-			image_start_x = x_pixel;
-			}
-
-		if (y_pixel == -1)
-			{
-			image_start_y = 0;
-			}
-		else
-			{
-			image_start_y = y_pixel;
-			}
+		selection_rectangle = SelectionRectangle(std::max(0, gint(event->x)), std::max(0, gint(event->y)), options->rectangle_draw_aspect_ratio);
+		image_start_x = std::max(0, x_pixel);
+		image_start_y = std::max(0, y_pixel);
 		}
-
 	if (rect_id)
 		{
 		pixbuf_renderer_overlay_remove(PIXBUF_RENDERER(imd->pr), rect_id);
@@ -190,7 +229,7 @@ static void image_press_cb(PixbufRenderer *pr, GdkEventButton *event, gpointer d
 	lw = layout_find_by_image(imd);
 	if (!lw)
 		{
-		layout_valid(&lw);
+		lw = get_current_layout();
 		}
 
 	if (lw && event->button == MOUSE_BUTTON_LEFT && event->type == GDK_2BUTTON_PRESS
@@ -208,26 +247,25 @@ static void image_release_cb(PixbufRenderer *, GdkEventButton *event, gpointer d
 	lw = layout_find_by_image(imd);
 	if (!lw)
 		{
-		layout_valid(&lw);
+		lw = get_current_layout();
 		}
 
-	defined_mouse_buttons(nullptr, event, lw);
+	defined_mouse_buttons(event, lw);
 }
 
 static void image_drag_cb(PixbufRenderer *pr, GdkEventMotion *event, gpointer data)
 {
 	auto imd = static_cast<ImageWindow *>(data);
-	gdouble aspect_ratio = 0.0;
 	gint width;
 	gint height;
-	gint rect_width;
-	gint rect_height;
 	GdkPixbuf *rect_pixbuf;
 	gint x_pixel;
 	gint y_pixel;
 	gint image_x_pixel;
 	gint image_y_pixel;
 
+	selection_rectangle.set_cursor(event->x, event->y);
+
 	if (options->draw_rectangle)
 		{
 		pixbuf_renderer_get_image_size(pr, &width, &height);
@@ -253,34 +291,13 @@ static void image_drag_cb(PixbufRenderer *pr, GdkEventMotion *event, gpointer da
 
 		if (options->rectangle_draw_aspect_ratio != RECTANGLE_DRAW_ASPECT_RATIO_NONE)
 			{
-			switch (options->rectangle_draw_aspect_ratio)
+			if (gdouble(image_x_pixel - image_start_x) / (image_y_pixel - image_start_y) < selection_rectangle.aspect_ratio)
 				{
-				case RECTANGLE_DRAW_ASPECT_RATIO_ONE_ONE:
-					aspect_ratio = 1.0;
-					break;
-				case RECTANGLE_DRAW_ASPECT_RATIO_FOUR_THREE:
-					aspect_ratio = gdouble(4) / 3;
-					break;
-				case RECTANGLE_DRAW_ASPECT_RATIO_THREE_TWO:
-					aspect_ratio = gdouble(3) / 2;
-					break;
-				case RECTANGLE_DRAW_ASPECT_RATIO_SIXTEEN_NINE:
-					aspect_ratio = gdouble(16) / 9;
-					break;
-				default:
-					aspect_ratio = 1.0;
-				}
-			}
-
-		if (options->rectangle_draw_aspect_ratio != RECTANGLE_DRAW_ASPECT_RATIO_NONE)
-			{
-			if (gdouble(image_x_pixel - image_start_x) / (image_y_pixel - image_start_y) < aspect_ratio)
-				{
-				image_x_pixel = image_start_x + ((image_y_pixel - image_start_y) * aspect_ratio);
+				image_x_pixel = image_start_x + ((image_y_pixel - image_start_y) * selection_rectangle.aspect_ratio);
 				}
 			else
 				{
-				image_y_pixel = image_start_y + ((image_x_pixel - image_start_x) / aspect_ratio);
+				image_y_pixel = image_start_y + ((image_x_pixel - image_start_x) / selection_rectangle.aspect_ratio);
 				}
 			}
 
@@ -291,42 +308,25 @@ static void image_drag_cb(PixbufRenderer *pr, GdkEventMotion *event, gpointer da
 			pixbuf_renderer_overlay_remove(PIXBUF_RENDERER(imd->pr), rect_id);
 			}
 
-		rect_width = pr->drag_last_x - pixbuf_start_x;
-		if (rect_width <= 0)
-			{
-			rect_width = 1;
-			}
-		rect_height = pr->drag_last_y - pixbuf_start_y;
-		if (rect_height <= 0)
-			{
-			rect_height = 1;
-			}
+		if (selection_rectangle.height <= 0)
+			selection_rectangle.height = 1;
+		if (selection_rectangle.width <= 0)
+			selection_rectangle.width = 1;
 
-		if (options->rectangle_draw_aspect_ratio != RECTANGLE_DRAW_ASPECT_RATIO_NONE)
-			{
-			if (gdouble(rect_width) / rect_height < aspect_ratio)
-				{
-				rect_width = gdouble(rect_height) * aspect_ratio;
-				}
-			else
-				{
-				rect_height = rect_width / aspect_ratio;
-				}
-			}
+		// decorative border
+		rect_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, selection_rectangle.width, selection_rectangle.height);
+		pixbuf_set_rect_fill(rect_pixbuf, 0, 0, selection_rectangle.width, selection_rectangle.height, 255, 255, 255, 0);
+		pixbuf_set_rect(rect_pixbuf, 1, 1, selection_rectangle.width-2, selection_rectangle.height - 2, 0, 0, 0, 255, 1, 1, 1, 1);
+		pixbuf_set_rect(rect_pixbuf, 0, 0, selection_rectangle.width, selection_rectangle.height, 255, 255, 255, 255, 1, 1, 1, 1);
 
-		rect_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, rect_width, rect_height);
-		pixbuf_set_rect_fill(rect_pixbuf, 0, 0, rect_width, rect_height, 255, 255, 255, 0);
-		pixbuf_set_rect(rect_pixbuf, 1, 1, rect_width-2, rect_height - 2, 0, 0, 0, 255, 1, 1, 1, 1);
-		pixbuf_set_rect(rect_pixbuf, 0, 0, rect_width, rect_height, 255, 255, 255, 255, 1, 1, 1, 1);
-
-		rect_id = pixbuf_renderer_overlay_add(PIXBUF_RENDERER(imd->pr), rect_pixbuf, pixbuf_start_x, pixbuf_start_y, OVL_NORMAL);
+		rect_id = pixbuf_renderer_overlay_add(PIXBUF_RENDERER(imd->pr), rect_pixbuf, selection_rectangle.x, selection_rectangle.y, OVL_NORMAL);
 		}
 
 	pixbuf_renderer_get_scaled_size(pr, &width, &height);
 
 	if (imd->func_drag)
 		{
-		imd->func_drag(imd, event, static_cast<gfloat>(pr->drag_last_x - event->x) / width, static_cast<gfloat>(pr->drag_last_y - event->y) / height, imd->data_button);
+		imd->func_drag(imd, event, static_cast<gfloat>(selection_rectangle.x) / selection_rectangle.width, static_cast<gfloat>(selection_rectangle.y) / selection_rectangle.height, imd->data_button);
 		}
 }
 
@@ -339,8 +339,8 @@ static void image_scroll_notify_cb(PixbufRenderer *pr, gpointer data)
 		imd->func_scroll_notify(imd,
 					static_cast<gint>(static_cast<gdouble>(pr->x_scroll) / pr->scale),
 					static_cast<gint>(static_cast<gdouble>(pr->y_scroll) / pr->scale),
-					static_cast<gint>(static_cast<gdouble>(pr->image_width) - pr->vis_width / pr->scale),
-					static_cast<gint>(static_cast<gdouble>(pr->image_height) - pr->vis_height / pr->scale),
+					static_cast<gint>(static_cast<gdouble>(pr->image_width) - (pr->vis_width / pr->scale)),
+					static_cast<gint>(static_cast<gdouble>(pr->image_height) - (pr->vis_height / pr->scale)),
 					imd->data_scroll_notify);
 		}
 }
@@ -466,11 +466,7 @@ static gboolean image_post_process_color(ImageWindow *imd, gint start_row, gbool
 	ColorManProfileType screen_type;
 	const gchar *input_file = nullptr;
 	const gchar *screen_file = nullptr;
-	guchar *profile = nullptr;
-	guint profile_len;
-	guchar *screen_profile = nullptr;
 	gint screen_profile_len;
-	ExifData *exif;
 
 	if (imd->cm) return FALSE;
 
@@ -495,6 +491,7 @@ static gboolean image_post_process_color(ImageWindow *imd, gint start_row, gbool
 		return FALSE;
 		}
 
+	g_autofree guchar *screen_profile = nullptr;
 	if (options->color_profile.use_x11_screen_profile &&
 	    image_get_x11_screen_profile(imd, &screen_profile, &screen_profile_len))
 		{
@@ -516,85 +513,28 @@ static gboolean image_post_process_color(ImageWindow *imd, gint start_row, gbool
 
 	imd->color_profile_from_image = COLOR_PROFILE_NONE;
 
-	exif = exif_read_fd(imd->image_fd);
+	guint profile_len;
+	g_autofree guchar *profile = exif_get_color_profile(imd->image_fd, profile_len, imd->color_profile_from_image);
 
-	if (exif)
+	if (profile)
 		{
-		if (g_strcmp0(imd->image_fd->format_name, "heif") == 0)
+		if (!imd->color_profile_use_image)
 			{
-			profile = heif_color_profile(imd->image_fd, &profile_len);
+			g_free(profile);
+			profile = nullptr;
 			}
-
-		if (!profile)
-			{
-			profile = exif_get_color_profile(exif, &profile_len);
-			}
-
-		if (profile)
-			{
-			if (!imd->color_profile_use_image)
-				{
-				g_free(profile);
-				profile = nullptr;
-				}
-			DEBUG_1("Found embedded color profile");
-			imd->color_profile_from_image = COLOR_PROFILE_MEM;
-			}
-		else
-			{
-			gchar *interop_index = exif_get_data_as_text(exif, "Exif.Iop.InteroperabilityIndex");
-
-			if (interop_index)
-				{
-				/* Exif 2.21 specification */
-				if (!strcmp(interop_index, "R98"))
-					{
-					imd->color_profile_from_image = COLOR_PROFILE_SRGB;
-					DEBUG_1("Found EXIF 2.21 ColorSpace of sRGB");
-					}
-				else if (!strcmp(interop_index, "R03"))
-					{
-					imd->color_profile_from_image = COLOR_PROFILE_ADOBERGB;
-					DEBUG_1("Found EXIF 2.21 ColorSpace of AdobeRGB");
-					}
-				g_free(interop_index);
-				}
-			else
-				{
-				gint cs;
-
-				/* ColorSpace == 1 specifies sRGB per EXIF 2.2 */
-				if (!exif_get_integer(exif, "Exif.Photo.ColorSpace", &cs)) cs = 0;
-				if (cs == 1)
-					{
-					imd->color_profile_from_image = COLOR_PROFILE_SRGB;
-					DEBUG_1("Found EXIF 2.2 ColorSpace of sRGB");
-					}
-				else if (cs == 2)
-					{
-					/* non-standard way of specifying AdobeRGB (used by some software) */
-					imd->color_profile_from_image = COLOR_PROFILE_ADOBERGB;
-					DEBUG_1("Found EXIF 2.2 ColorSpace of AdobeRGB");
-					}
-				}
-
-			if (imd->color_profile_use_image && imd->color_profile_from_image != COLOR_PROFILE_NONE)
-                               {
-                               input_type = static_cast<ColorManProfileType>(imd->color_profile_from_image);
-                               input_file = nullptr;
-                               }
-			}
-
-		exif_free_fd(imd->image_fd, exif);
 		}
-
+	else if (imd->color_profile_use_image && imd->color_profile_from_image != COLOR_PROFILE_NONE)
+		{
+		input_type = imd->color_profile_from_image;
+		input_file = nullptr;
+		}
 
 	if (profile)
 		{
 		cm = color_man_new_embedded(run_in_bg ? imd : nullptr, nullptr,
 					    profile, profile_len,
 					    screen_type, screen_file, screen_profile, screen_profile_len);
-		g_free(profile);
 		}
 	else
 		{
@@ -616,12 +556,6 @@ static gboolean image_post_process_color(ImageWindow *imd, gint start_row, gbool
 
 	image_update_util(imd);
 
-	if (screen_profile)
-		{
-		g_free(screen_profile);
-		screen_profile = nullptr;
-		}
-
 	return !!cm;
 }
 
@@ -1224,11 +1158,11 @@ static gboolean image_scroll_cb(GtkWidget *, GdkEventScroll *event, gpointer dat
 	auto imd = static_cast<ImageWindow *>(data);
 	gboolean in_lw = FALSE;
 	gint i = 0;
-	LayoutWindow *lw = nullptr;
 
 	if (imd->func_scroll && event && event->type == GDK_SCROLL)
 		{
-		layout_valid(&lw);
+		LayoutWindow *lw = get_current_layout();
+
 		/* check if the image is in a layout window */
 		for (i = 0; i < MAX_SPLIT_IMAGES; i++)
 			{
@@ -1912,7 +1846,6 @@ void image_background_set_color_from_options(ImageWindow *imd, gboolean fullscre
 	GdkRGBA theme_color;
 	GdkRGBA bg_color;
 	GtkStyleContext *style_context;
-	LayoutWindow *lw = nullptr;
 
 	if ((options->image.use_custom_border_color && !fullscreen) ||
 	    (options->image.use_custom_border_color_in_fullscreen && fullscreen))
@@ -1922,10 +1855,11 @@ void image_background_set_color_from_options(ImageWindow *imd, gboolean fullscre
 
 	else
 		{
-		if (!layout_valid(&lw)) return;
+		LayoutWindow *lw = get_current_layout();
+		if (!lw) return;
 
 		style_context = gtk_widget_get_style_context(lw->window);
-		gtk_style_context_get_background_color(style_context, GTK_STATE_FLAG_NORMAL, &bg_color);
+		gq_gtk_style_context_get_background_color(style_context, GTK_STATE_FLAG_NORMAL, &bg_color);
 
 		theme_color.red = bg_color.red * 1;
 		theme_color.green = bg_color.green * 1;
@@ -2018,7 +1952,7 @@ void image_to_root_window(ImageWindow *, gboolean)
 {
 }
 
-void image_select(ImageWindow *imd, gboolean select)
+void image_select(ImageWindow *imd, bool select)
 {
 	if (!imd->has_frame) return;
 
