@@ -27,6 +27,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
+#include "color-man.h"
 #include "typedefs.h"
 
 struct CollectInfo;
@@ -34,15 +35,6 @@ struct CollectionData;
 class FileData;
 struct ImageLoader;
 
-enum RectangleDrawAspectRatio
-{
-	RECTANGLE_DRAW_ASPECT_RATIO_NONE = 0,
-	RECTANGLE_DRAW_ASPECT_RATIO_ONE_ONE,
-	RECTANGLE_DRAW_ASPECT_RATIO_FOUR_THREE,
-	RECTANGLE_DRAW_ASPECT_RATIO_THREE_TWO,
-	RECTANGLE_DRAW_ASPECT_RATIO_SIXTEEN_NINE
-};
-
 enum ImageState {
 	IMAGE_STATE_NONE	= 0,
 	IMAGE_STATE_IMAGE	= 1 << 0,
@@ -121,7 +113,7 @@ struct ImageWindow
 	gboolean color_profile_enable;
 	gint color_profile_input;
 	gboolean color_profile_use_image;
-	gint color_profile_from_image;
+	ColorManProfileType color_profile_from_image;
 	gpointer cm;
 
 	AlterType delay_alter_type;
@@ -170,7 +162,7 @@ void image_set_state_func(ImageWindow *imd,
 			  void (*func)(ImageWindow *imd, ImageState state, gpointer data),
 			  gpointer data);
 
-void image_select(ImageWindow *imd, gboolean select);
+void image_select(ImageWindow *imd, bool select);
 void image_set_selectable(ImageWindow *imd, gboolean selectable);
 
 void image_grab_focus(ImageWindow *imd);
