@@ -21,7 +21,9 @@
 
 #include "img-view.h"
 
+#include <algorithm>
 #include <array>
+#include <vector>
 
 #include <gdk/gdk.h>
 #include <glib-object.h>
@@ -30,8 +32,8 @@
 #include "archives.h"
 #include "collect-io.h"
 #include "collect.h"
+#include "compat-deprecated.h"
 #include "compat.h"
-#include "debug.h"
 #include "dnd.h"
 #include "editors.h"
 #include "filedata.h"
@@ -57,6 +59,9 @@
 #include "utilops.h"
 #include "window.h"
 
+namespace
+{
+
 struct ViewWindow
 {
 	GtkWidget *window;
@@ -69,8 +74,9 @@ struct ViewWindow
 };
 
 
-static GList *view_window_list = nullptr;
+std::vector<ViewWindow *> view_window_list;
 
+} // namespace
 
 static GtkWidget *view_popup_menu(ViewWindow *vw);
 static void view_fullscreen_toggle(ViewWindow *vw, gboolean force_off);
@@ -95,7 +101,7 @@ static void view_window_notify_cb(FileData *fd, NotifyType type, gpointer data);
  *
  * See also @link hard_coded_window_keys @endlink
  **/
-hard_coded_window_keys image_window_keys[] = {
+static hard_coded_window_keys image_window_keys[] = {
 	{GDK_CONTROL_MASK, 'C', N_("Copy")},
 	{GDK_CONTROL_MASK, 'M', N_("Move")},
 	{GDK_CONTROL_MASK, 'R', N_("Rename")},
@@ -266,24 +272,7 @@ static void view_list_step(ViewWindow *vw, gboolean next)
 		}
 	else
 		{
-		gboolean found = FALSE;
-
-		work = vw->list;
-		while (work && !found)
-			{
-			FileData *temp;
-
-			temp = static_cast<FileData *>(work->data);
-
-			if (fd == temp)
-				{
-				found = TRUE;
-				}
-			else
-				{
-				work = work->next;
-				}
-			}
+		work = g_list_find(vw->list, fd);
 		}
 	if (!work) return;
 
@@ -510,9 +499,8 @@ static gboolean view_window_key_press_cb(GtkWidget * (widget), GdkEventKey *even
 				view_fullscreen_toggle(vw, TRUE);
 				imd = view_window_active_image(vw);
 				fd = image_get_fd(imd);
-				print_window_new(fd,
-						 fd ? g_list_append(nullptr, file_data_ref(fd)) : nullptr,
-						 filelist_copy(vw->list), vw->window);
+				print_window_new(fd ? g_list_append(nullptr, file_data_ref(fd)) : nullptr,
+				                 vw->window);
 				}
 				break;
 			case GDK_KEY_Delete: case GDK_KEY_KP_Delete:
@@ -659,7 +647,6 @@ static void button_cb(ImageWindow *imd, GdkEventButton *event, gpointer data)
 {
 	auto vw = static_cast<ViewWindow *>(data);
 	GtkWidget *menu;
-	gchar *dest_dir;
 	LayoutWindow *lw_new;
 
 	switch (event->button)
@@ -667,12 +654,11 @@ static void button_cb(ImageWindow *imd, GdkEventButton *event, gpointer data)
 		case MOUSE_BUTTON_LEFT:
 			if (options->image_l_click_archive && imd->image_fd->format_class == FORMAT_CLASS_ARCHIVE)
 				{
-				dest_dir = open_archive(imd->image_fd);
+				g_autofree gchar *dest_dir = open_archive(imd->image_fd);
 				if (dest_dir)
 					{
 					lw_new = layout_new_from_default();
 					layout_set_path(lw_new, dest_dir);
-					g_free(dest_dir);
 					}
 				else
 					{
@@ -820,27 +806,15 @@ static void view_slideshow_prev(ViewWindow *vw)
 static void view_slideshow_stop_func(SlideShowData *, gpointer data)
 {
 	auto vw = static_cast<ViewWindow *>(data);
-	GList *work;
-	FileData *fd;
 
 	vw->ss = nullptr;
 
-	work = vw->list;
-	fd = image_get_fd(view_window_active_image(vw));
-	while (work)
-		{
-		FileData *temp;
+	FileData *fd = image_get_fd(view_window_active_image(vw));
 
-		temp = static_cast<FileData *>(work->data);
-		if (fd == temp)
-			{
-			vw->list_pointer = work;
-			work = nullptr;
-			}
-		else
-			{
-			work = work->next;
-			}
+	GList *work = g_list_find(vw->list, fd);
+	if (work)
+		{
+		vw->list_pointer = work;
 		}
 }
 
@@ -878,7 +852,8 @@ static void view_window_destroy_cb(GtkWidget *, gpointer data)
 {
 	auto vw = static_cast<ViewWindow *>(data);
 
-	view_window_list = g_list_remove(view_window_list, vw);
+	view_window_list.erase(std::remove(view_window_list.begin(), view_window_list.end(), vw),
+	                       view_window_list.end());
 
 	view_slideshow_stop(vw);
 	fullscreen_stop(vw->fs);
@@ -993,11 +968,11 @@ static ViewWindow *real_view_window_new(FileData *fd, GList *list, CollectionDat
 
 	if (options->image.limit_window_size)
 		{
-		gint mw = gdk_screen_width() * options->image.max_window_size / 100;
-		gint mh = gdk_screen_height() * options->image.max_window_size / 100;
+		gint mw = gq_gdk_screen_width() * options->image.max_window_size / 100;
+		gint mh = gq_gdk_screen_height() * options->image.max_window_size / 100;
 
-		if (w > mw) w = mw;
-		if (h > mh) h = mh;
+		w = std::min(w, mw);
+		h = std::min(h, mh);
 		}
 
 	gtk_window_set_default_size(GTK_WINDOW(vw->window), w, h);
@@ -1009,7 +984,7 @@ static ViewWindow *real_view_window_new(FileData *fd, GList *list, CollectionDat
 	gtk_window_set_focus_on_map(GTK_WINDOW(vw->window), FALSE);
 	gtk_widget_show(vw->window);
 
-	view_window_list = g_list_append(view_window_list, vw);
+	view_window_list.push_back(vw);
 
 	file_data_register_notify_func(view_window_notify_cb, vw, NOTIFY_PRIORITY_LOW);
 
@@ -1094,52 +1069,37 @@ void view_window_new_from_collection(CollectionData *cd, CollectInfo *info)
 
 void view_window_colors_update()
 {
-	GList *work;
-
-	work = view_window_list;
-	while (work)
+	for (ViewWindow *vw : view_window_list)
 		{
-		auto vw = static_cast<ViewWindow *>(work->data);
-		work = work->next;
-
 		image_background_set_color_from_options(vw->imd, !!vw->fs);
 		}
 }
 
-gboolean view_window_find_image(ImageWindow *imd, gint *index, gint *total)
+gboolean view_window_find_image(const ImageWindow *imd, gint &index, gint &total)
 {
-	GList *work;
+	const auto it = std::find_if(view_window_list.cbegin(), view_window_list.cend(),
+	                             [imd](const ViewWindow *vw) { return vw->imd == imd || (vw->fs && vw->fs->imd == imd); });
+	if (it == view_window_list.cend()) return FALSE;
 
-	work = view_window_list;
-	while (work)
+	const ViewWindow *vw = *it;
+
+	if (vw->ss)
 		{
-		auto vw = static_cast<ViewWindow *>(work->data);
-		work = work->next;
+		gint n = g_list_length(vw->ss->list_done);
+		gint t = n + g_list_length(vw->ss->list);
 
-		if (vw->imd == imd ||
-		    (vw->fs && vw->fs->imd == imd))
-			{
-			if (vw->ss)
-				{
-				gint n;
-				gint t;
+		if (n == 0) n = t;
 
-				n = g_list_length(vw->ss->list_done);
-				t = n + g_list_length(vw->ss->list);
-				if (n == 0) n = t;
-				if (index) *index = n - 1;
-				if (total) *total = t;
-				}
-			else
-				{
-				if (index) *index = g_list_position(vw->list, vw->list_pointer);
-				if (total) *total = g_list_length(vw->list);
-				}
-			return TRUE;
-			}
+		index = n - 1;
+		total = t;
+		}
+	else
+		{
+		index = g_list_position(vw->list, vw->list_pointer);
+		total = g_list_length(vw->list);
 		}
 
-	return FALSE;
+	return TRUE;
 }
 
 /*
