@@ -22,10 +22,11 @@
 #ifndef LAYOUT_H
 #define LAYOUT_H
 
+#include <functional>
+
 #include <glib.h>
 #include <gtk/gtk.h>
 
-#include "options.h"
 #include "typedefs.h"
 
 struct AnimationData;
@@ -36,12 +37,8 @@ struct SlideShowData;
 struct ViewDir;
 struct ViewFile;
 
-#define LAYOUT_ID_CURRENT "_current_"
 #define MAX_SPLIT_IMAGES 4
 
-extern GList *layout_window_list;
-
-
 enum LayoutLocation {
 	LAYOUT_HIDE   = 0,
 	LAYOUT_LEFT   = 1 << 0,
@@ -50,6 +47,119 @@ enum LayoutLocation {
 	LAYOUT_BOTTOM = 1 << 3
 };
 
+enum StartUpPath {
+	STARTUP_PATH_CURRENT	= 0,
+	STARTUP_PATH_LAST,
+	STARTUP_PATH_HOME,
+};
+
+enum SortActionType {
+	BAR_SORT_COPY = 0,
+	BAR_SORT_MOVE,
+	BAR_SORT_FILTER,
+	BAR_SORT_ACTION_COUNT
+};
+
+enum SortModeType {
+	BAR_SORT_MODE_FOLDER = 0,
+	BAR_SORT_MODE_COLLECTION,
+	BAR_SORT_MODE_COUNT
+};
+
+enum SortSelectionType {
+	BAR_SORT_SELECTION_IMAGE = 0,
+	BAR_SORT_SELECTION_SELECTED,
+	BAR_SORT_SELECTION_COUNT
+};
+
+struct LayoutOptions
+{
+	gchar *id;
+
+	gchar *order;
+	gint style;
+
+	DirViewType dir_view_type;
+	FileViewType file_view_type;
+
+	struct SortParams
+	{
+		SortType method;
+		gboolean ascend;
+		gboolean case_sensitive;
+	};
+	SortParams dir_view_list_sort;
+	SortParams file_view_list_sort;
+
+	gboolean show_thumbnails;
+	gboolean show_marks;
+	gboolean show_file_filter;
+	gboolean show_directory_date;
+	gboolean show_info_pixel;
+	gboolean split_pane_sync;
+	gboolean ignore_alpha;
+
+	struct {
+		GdkRectangle rect;
+		gboolean maximized;
+		gint hdivider_pos;
+		gint vdivider_pos;
+	} main_window;
+
+	struct {
+		GdkRectangle rect;
+		gint vdivider_pos;
+	} float_window;
+
+	struct {
+		gint vdivider_pos;
+	} folder_window;
+
+	struct {
+		guint state;
+		gint histogram_channel;
+		gint histogram_mode;
+	} image_overlay;
+
+	GdkRectangle log_window;
+
+	struct {
+		GdkRectangle rect;
+		gint page_number;
+	} preferences_window;
+
+	GdkRectangle search_window;
+
+	GdkRectangle dupe_window;
+
+	GdkRectangle advanced_exif_window;
+
+	gboolean tools_float;
+	gboolean tools_hidden;
+	gboolean selectable_toolbars_hidden;
+
+	struct {
+		gboolean info;
+		gboolean sort;
+		gboolean tools_float;
+		gboolean tools_hidden;
+		gboolean hidden;
+	} bars_state;
+
+	gchar *home_path;
+	gchar *last_path;
+
+	StartUpPath startup_path;
+
+	gboolean animate;
+	gint workspace;
+
+	SortActionType action;
+	SortModeType mode;
+	SortSelectionType selection;
+	gchar *filter_key;
+};
+
 struct LayoutWindow
 {
 	LayoutOptions options;
@@ -151,9 +261,6 @@ struct LayoutWindow
 	GtkWidget *log_window;
 };
 
-LayoutWindow *layout_new(FileData *dir_fd, LayoutOptions *lop);
-LayoutWindow *layout_new_with_geometry(FileData *dir_fd, LayoutOptions *lop,
-				       const gchar *geometry);
 LayoutWindow *layout_new_from_config(const gchar **attribute_names, const gchar **attribute_values, gboolean use_commandline);
 void layout_update_from_config(LayoutWindow *lw, const gchar **attribute_names, const gchar **attribute_values);
 LayoutWindow *layout_new_from_default();
@@ -161,22 +268,18 @@ LayoutWindow *layout_new_from_default();
 void layout_close(LayoutWindow *lw);
 void layout_free(LayoutWindow *lw);
 
+LayoutWindow *get_current_layout();
 gboolean layout_valid(LayoutWindow **lw);
 
 void layout_show_config_window(LayoutWindow *lw);
 
-void layout_apply_options(LayoutWindow *lw, LayoutOptions *lop);
-
 void layout_sync_options_with_current_state(LayoutWindow *lw);
-void layout_load_attributes(LayoutOptions *layout, const gchar **attribute_names, const gchar **attribute_values);
-void layout_write_attributes(LayoutOptions *layout, GString *outstr, gint indent);
 void layout_write_config(LayoutWindow *lw, GString *outstr, gint indent);
 
 
 LayoutWindow *layout_find_by_image(ImageWindow *imd);
 LayoutWindow *layout_find_by_image_fd(ImageWindow *imd);
 LayoutWindow *layout_find_by_layout_id(const gchar *id);
-gint layout_compare_options_id(const LayoutWindow *lw, const gchar *id);
 
 const gchar *layout_get_path(LayoutWindow *lw);
 gboolean layout_set_path(LayoutWindow *lw, const gchar *path);
@@ -192,7 +295,7 @@ guint layout_list_count(LayoutWindow *lw, gint64 *bytes);
 FileData *layout_list_get_fd(LayoutWindow *lw, gint index);
 gint layout_list_get_index(LayoutWindow *lw, FileData *fd);
 void layout_list_sync_fd(LayoutWindow *lw, FileData *fd);
-GString *layout_get_window_list();
+gchar *layout_get_window_list();
 
 GList *layout_selection_list(LayoutWindow *lw);
 /* return list of pointers to int for selection */
@@ -248,7 +351,15 @@ void layout_split_change(LayoutWindow *lw, ImageSplitMode mode);
 
 void save_layout(LayoutWindow *lw);
 gchar *layout_get_unique_id();
+
+
 guint layout_window_count();
+LayoutWindow *layout_window_first();
+
+using LayoutWindowCallback = std::function<void(LayoutWindow *)>;
+void layout_window_foreach(const LayoutWindowCallback &lw_cb);
+
+gboolean layout_window_is_displayed(const gchar *id);
 
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
