@@ -18,7 +18,9 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE
+#endif
 
 #include "glua.h"
 
@@ -31,7 +33,6 @@
 #include <glib.h>
 #include <lua.hpp>
 
-#include "debug.h"
 #include "exif.h"
 #include "filedata.h"
 #include "main.h"
@@ -233,10 +234,7 @@ static ExifData *lua_check_exif(lua_State *L, int index)
 static int lua_exif_get_datum(lua_State *L)
 {
 	const gchar *key;
-	gchar *value = nullptr;
 	ExifData *exif;
-	struct tm tm;
-	time_t datetime;
 
 	exif = lua_check_exif(L, 1);
 	key = luaL_checkstring(L, 2);
@@ -250,27 +248,16 @@ static int lua_exif_get_datum(lua_State *L)
 		lua_pushnil(L);
 		return 1;
 		}
-	value = exif_get_data_as_text(exif, key);
-	if (strcmp(key, "Exif.Photo.DateTimeOriginal") == 0)
+
+	g_autofree gchar *value = exif_get_data_as_text(exif, key);
+
+	if (strcmp(key, "Exif.Photo.DateTimeDigitized") == 0 ||
+	    strcmp(key, "Exif.Photo.DateTimeOriginal") == 0)
 		{
-		memset(&tm, 0, sizeof(tm));
+		std::tm tm{};
 		if (value && strptime(value, "%Y:%m:%d %H:%M:%S", &tm))
 			{
-			datetime = mktime(&tm);
-			lua_pushnumber(L, datetime);
-			return 1;
-			}
-
-		lua_pushnil(L);
-		return 1;
-		}
-
-	if (strcmp(key, "Exif.Photo.DateTimeDigitized") == 0)
-		{
-		memset(&tm, 0, sizeof(tm));
-		if (value && strptime(value, "%Y:%m:%d %H:%M:%S", &tm))
-			{
-			datetime = mktime(&tm);
+			const time_t datetime = mktime(&tm);
 			lua_pushnumber(L, datetime);
 			return 1;
 			}
@@ -395,18 +382,16 @@ gchar *lua_callvalue(FileData *fd, const gchar *file, const gchar *function)
 		}
 
 	gchar *data = g_strdup(lua_tostring(L, -1));
-	GError *error = nullptr;
-	gchar *tmp = g_locale_to_utf8(data, strlen(data), nullptr, nullptr, &error);
+	g_autoptr(GError) error = nullptr;
+	g_autofree gchar *tmp = g_locale_to_utf8(data, strlen(data), nullptr, nullptr, &error);
 	if (error)
 		{
 		log_printf("Error converting lua output from locale to UTF-8: %s\n", error->message);
-		g_error_free(error);
 		}
 	else
 		{
 		std::swap(data, tmp);
-		} // if (error) { ... } else
-	g_free(tmp);
+		}
 	return data;
 }
 
