@@ -923,6 +923,13 @@ void startup_cb(GtkApplication *app, gpointer)
 
 	startup_common(app, nullptr);
 
+	const gchar *gq_disable_clutter = g_getenv("GQ_DISABLE_CLUTTER");
+
+	if (gq_disable_clutter && (gq_disable_clutter[0] == 'y' || gq_disable_clutter[0] == 'Y'))
+		{
+		options->disable_gpu = TRUE;
+		}
+
 	/* restore session from the config file */
 
 	if (!load_options(options))
@@ -949,13 +956,6 @@ void startup_cb(GtkApplication *app, gpointer)
 		options->image_overlay_n.background_alpha[0] = options->image_overlay.background_alpha;
 		}
 
-	const gchar *gq_disable_clutter = g_getenv("GQ_DISABLE_CLUTTER");
-
-	if (gq_disable_clutter && (gq_disable_clutter[0] == 'y' || gq_disable_clutter[0] == 'Y'))
-		{
-		options->disable_gpu = TRUE;
-		}
-
 #if HAVE_CLUTTER
 	/** @FIXME For the background of this see:
 	 * https://github.com/BestImageViewer/geeqie/issues/397
@@ -1053,7 +1053,7 @@ gint main(gint argc, gchar *argv[])
 
 	if (!gq_disable_clutter || tolower(gq_disable_clutter[0]) != 'y')
 		{
-		if (clutter_init(nullptr, nullptr) != CLUTTER_INIT_SUCCESS)
+		if (gtk_clutter_init(nullptr, nullptr) != CLUTTER_INIT_SUCCESS)
 			{
 			fprintf(stderr,
 				_("Can't initialize clutter-gtk. \n \
