@@ -21,8 +21,6 @@
 
 #include "metadata.h"
 
-#include <unistd.h>
-
 #include <algorithm>
 #include <array>
 #include <clocale>
@@ -36,7 +34,6 @@
 #include <config.h>
 
 #include "cache.h"
-#include "debug.h"
 #include "exif.h"
 #include "filedata.h"
 #include "intl.h"
@@ -94,6 +91,8 @@ constexpr std::array<const gchar *, 22> group_keys{
 	"Xmp.xmp.Rating",
 };
 
+GtkTreeStore *keyword_tree;
+
 gint metadata_cache_entry_compare_key(const MetadataCacheEntry *entry, const gchar *key)
 {
 	return strcmp(entry->key, key);
@@ -214,7 +213,6 @@ void metadata_cache_free(FileData *fd)
  */
 
 static GList *metadata_write_queue = nullptr;
-static guint metadata_write_idle_id = 0; /* event source id */
 
 static void metadata_write_queue_add(FileData *fd)
 {
@@ -226,6 +224,8 @@ static void metadata_write_queue_add(FileData *fd)
 		layout_util_status_update_write_all();
 		}
 
+	static guint metadata_write_idle_id = 0; /* event source id */
+
 	if (metadata_write_idle_id)
 		{
 		g_source_remove(metadata_write_idle_id);
@@ -234,7 +234,7 @@ static void metadata_write_queue_add(FileData *fd)
 
 	if (options->metadata.confirm_after_timeout)
 		{
-		metadata_write_idle_id = g_timeout_add(options->metadata.confirm_timeout * 1000, metadata_write_queue_idle_cb, nullptr);
+		metadata_write_idle_id = g_timeout_add(options->metadata.confirm_timeout * 1000, metadata_write_queue_idle_cb, &metadata_write_idle_id);
 		}
 }
 
@@ -301,11 +301,14 @@ gboolean metadata_write_queue_confirm(gboolean force_dialog, FileUtilDoneFunc do
 	return (metadata_write_queue != nullptr);
 }
 
-static gboolean metadata_write_queue_idle_cb(gpointer)
+static gboolean metadata_write_queue_idle_cb(gpointer data)
 {
 	metadata_write_queue_confirm(FALSE, nullptr, nullptr);
-	metadata_write_idle_id = 0;
-	return FALSE;
+
+	auto *metadata_write_idle_id = static_cast<guint *>(data);
+	*metadata_write_idle_id = 0;
+
+	return G_SOURCE_REMOVE;
 }
 
 gboolean metadata_write_perform(FileData *fd)
@@ -441,17 +444,16 @@ static gboolean metadata_file_write(gchar *path, const GList *keywords, const gc
 	secure_fprintf(ssi, "#%s comment (%s)\n\n", GQ_APPNAME, VERSION);
 
 	secure_fprintf(ssi, "[keywords]\n");
-	while (keywords && secsave_errno == SS_ERR_NONE)
+	for (; keywords && secsave_succeed(); keywords = keywords->next)
 		{
 		auto word = static_cast<const gchar *>(keywords->data);
-		keywords = keywords->next;
 
 		secure_fprintf(ssi, "%s\n", word);
 		}
 	secure_fputc(ssi, '\n');
 
 	secure_fprintf(ssi, "[comment]\n");
-	secure_fprintf(ssi, "%s\n", (comment) ? comment : "");
+	secure_fprintf(ssi, "%s\n", comment ? comment : "");
 
 	secure_fprintf(ssi, "#end\n");
 
@@ -461,14 +463,12 @@ static gboolean metadata_file_write(gchar *path, const GList *keywords, const gc
 static gboolean metadata_legacy_write(FileData *fd)
 {
 	gboolean success = FALSE;
-	gchar *metadata_pathl;
 	gpointer keywords;
 	gpointer comment_l;
 	gboolean have_keywords;
 	gboolean have_comment;
 	const gchar *comment;
 	GList *orig_keywords = nullptr;
-	gchar *orig_comment = nullptr;
 
 	g_assert(fd->change && fd->change->dest);
 
@@ -476,20 +476,19 @@ static gboolean metadata_legacy_write(FileData *fd)
 
 	if (!fd->modified_xmp) return TRUE;
 
-	metadata_pathl = path_from_utf8(fd->change->dest);
+	g_autofree gchar *metadata_pathl = path_from_utf8(fd->change->dest);
 
 	have_keywords = g_hash_table_lookup_extended(fd->modified_xmp, KEYWORD_KEY, nullptr, &keywords);
 	have_comment = g_hash_table_lookup_extended(fd->modified_xmp, COMMENT_KEY, nullptr, &comment_l);
 	comment = static_cast<const gchar *>((have_comment && comment_l) ? (static_cast<GList *>(comment_l))->data : nullptr);
 
+	g_autofree gchar *orig_comment = nullptr;
 	if (!have_keywords || !have_comment) metadata_file_read(metadata_pathl, &orig_keywords, &orig_comment);
 
 	success = metadata_file_write(metadata_pathl,
 				      have_keywords ? static_cast<GList *>(keywords) : orig_keywords,
 				      have_comment ? comment : orig_comment);
 
-	g_free(metadata_pathl);
-	g_free(orig_comment);
 	g_list_free_full(orig_keywords, g_free);
 
 	return success;
@@ -501,11 +500,12 @@ static gboolean metadata_file_read(gchar *path, GList **keywords, gchar **commen
 	gchar s_buf[1024];
 	MetadataKey key = MK_NONE;
 	GList *list = nullptr;
-	GString *comment_build = nullptr;
 
 	f = fopen(path, "r");
 	if (!f) return FALSE;
 
+	g_autoptr(GString) comment_build = g_string_new(nullptr);
+
 	while (fgets(s_buf, sizeof(s_buf), f))
 		{
 		gchar *ptr = s_buf;
@@ -546,7 +546,6 @@ static gboolean metadata_file_read(gchar *path, GList **keywords, gchar **commen
 				}
 				break;
 			case MK_COMMENT:
-				if (!comment_build) comment_build = g_string_new("");
 				g_string_append(comment_build, s_buf);
 				break;
 			}
@@ -563,27 +562,22 @@ static gboolean metadata_file_read(gchar *path, GList **keywords, gchar **commen
 		g_list_free_full(list, g_free);
 		}
 
-	if (comment_build)
+	if (comment && comment_build->len > 0)
 		{
-		if (comment)
+		gint len;
+		gchar *ptr = comment_build->str;
+
+		/* strip leading and trailing newlines */
+		while (*ptr == '\n') ptr++;
+		len = strlen(ptr);
+		while (len > 0 && ptr[len - 1] == '\n') len--;
+		if (ptr[len] == '\n') len++; /* keep the last one */
+		if (len > 0)
 			{
-			gint len;
-			gchar *ptr = comment_build->str;
+			g_autofree gchar *text = g_strndup(ptr, len);
 
-			/* strip leading and trailing newlines */
-			while (*ptr == '\n') ptr++;
-			len = strlen(ptr);
-			while (len > 0 && ptr[len - 1] == '\n') len--;
-			if (ptr[len] == '\n') len++; /* keep the last one */
-			if (len > 0)
-				{
-				gchar *text = g_strndup(ptr, len);
-
-				*comment = utf8_validate_or_convert(text);
-				g_free(text);
-				}
+			*comment = utf8_validate_or_convert(text);
 			}
-		g_string_free(comment_build, TRUE);
 		}
 
 	return TRUE;
@@ -591,52 +585,35 @@ static gboolean metadata_file_read(gchar *path, GList **keywords, gchar **commen
 
 static void metadata_legacy_delete(FileData *fd, const gchar *except)
 {
-	gchar *metadata_path;
-	gchar *metadata_pathl;
 	if (!fd) return;
 
-	metadata_path = cache_find_location(CACHE_TYPE_METADATA, fd->path);
+	g_autofree gchar *metadata_path = cache_find_location(CACHE_TYPE_METADATA, fd->path);
 	if (metadata_path && (!except || strcmp(metadata_path, except) != 0))
 		{
-		metadata_pathl = path_from_utf8(metadata_path);
-		unlink(metadata_pathl);
-		g_free(metadata_pathl);
-		g_free(metadata_path);
+		unlink_file(metadata_path);
 		}
 
 #if HAVE_EXIV2
 	/* without exiv2: do not delete xmp metadata because we are not able to convert it,
 	   just ignore it */
-	metadata_path = cache_find_location(CACHE_TYPE_XMP_METADATA, fd->path);
-	if (metadata_path && (!except || strcmp(metadata_path, except) != 0))
+	g_autofree gchar *xmp_metadata_path = cache_find_location(CACHE_TYPE_XMP_METADATA, fd->path);
+	if (xmp_metadata_path && (!except || strcmp(xmp_metadata_path, except) != 0))
 		{
-		metadata_pathl = path_from_utf8(metadata_path);
-		unlink(metadata_pathl);
-		g_free(metadata_pathl);
-		g_free(metadata_path);
+		unlink_file(xmp_metadata_path);
 		}
 #endif
 }
 
 static gboolean metadata_legacy_read(FileData *fd, GList **keywords, gchar **comment)
 {
-	gchar *metadata_path;
-	gchar *metadata_pathl;
-	gboolean success = FALSE;
-
 	if (!fd) return FALSE;
 
-	metadata_path = cache_find_location(CACHE_TYPE_METADATA, fd->path);
+	g_autofree gchar *metadata_path = cache_find_location(CACHE_TYPE_METADATA, fd->path);
 	if (!metadata_path) return FALSE;
 
-	metadata_pathl = path_from_utf8(metadata_path);
+	g_autofree gchar *metadata_pathl = path_from_utf8(metadata_path);
 
-	success = metadata_file_read(metadata_pathl, keywords, comment);
-
-	g_free(metadata_pathl);
-	g_free(metadata_path);
-
-	return success;
+	return metadata_file_read(metadata_pathl, keywords, comment);
 }
 
 static GList *remove_duplicate_strings_from_list(GList *list)
@@ -748,12 +725,11 @@ guint64 metadata_read_int(FileData *fd, const gchar *key, guint64 fallback)
 {
 	guint64 ret;
 	gchar *endptr;
-	gchar *string = metadata_read_string(fd, key, METADATA_PLAIN);
+	g_autofree gchar *string = metadata_read_string(fd, key, METADATA_PLAIN);
 	if (!string) return fallback;
 
 	ret = g_ascii_strtoull(string, &endptr, 10);
 	if (string == endptr) ret = fallback;
-	g_free(string);
 	return ret;
 }
 
@@ -775,7 +751,7 @@ gdouble metadata_read_GPS_coord(FileData *fd, const gchar *key, gdouble fallback
 	gdouble min;
 	gdouble sec;
 	gboolean ok = FALSE;
-	gchar *string = metadata_read_string(fd, key, METADATA_PLAIN);
+	g_autofree gchar *string = metadata_read_string(fd, key, METADATA_PLAIN);
 	if (!string) return fallback;
 
 	deg = g_ascii_strtod(string, &endptr);
@@ -802,7 +778,6 @@ gdouble metadata_read_GPS_coord(FileData *fd, const gchar *key, gdouble fallback
 		log_printf("unable to parse GPS coordinate '%s'\n", string);
 		}
 
-	g_free(string);
 	return coord;
 }
 
@@ -811,7 +786,7 @@ gdouble metadata_read_GPS_direction(FileData *fd, const gchar *key, gdouble fall
 	gchar *endptr;
 	gdouble deg;
 	gboolean ok = FALSE;
-	gchar *string = metadata_read_string(fd, key, METADATA_PLAIN);
+	g_autofree gchar *string = metadata_read_string(fd, key, METADATA_PLAIN);
 	if (!string) return fallback;
 
 	DEBUG_3("GPS_direction: %s\n", string);
@@ -832,25 +807,19 @@ gdouble metadata_read_GPS_direction(FileData *fd, const gchar *key, gdouble fall
 		log_printf("unable to parse GPS direction '%s: %f'\n", string, deg);
 		}
 
-	g_free(string);
-
 	return deg;
 }
 
 gboolean metadata_append_string(FileData *fd, const gchar *key, const char *value)
 {
-	gchar *str = metadata_read_string(fd, key, METADATA_PLAIN);
-
+	g_autofree gchar *str = metadata_read_string(fd, key, METADATA_PLAIN);
 	if (!str)
 		{
 		return metadata_write_string(fd, key, value);
 		}
 
-	gchar *new_string = g_strconcat(str, value, NULL);
-	gboolean ret = metadata_write_string(fd, key, new_string);
-	g_free(str);
-	g_free(new_string);
-	return ret;
+	g_autofree gchar *new_string = g_strconcat(str, value, NULL);
+	return metadata_write_string(fd, key, new_string);
 }
 
 gboolean metadata_write_GPS_coord(FileData *fd, const gchar *key, gdouble value)
@@ -858,7 +827,6 @@ gboolean metadata_write_GPS_coord(FileData *fd, const gchar *key, gdouble value)
 	gint deg;
 	gdouble min;
 	gdouble param;
-	char *coordinate;
 	const char *ref;
 	gboolean ok = TRUE;
 	char *old_locale;
@@ -896,12 +864,11 @@ gboolean metadata_write_GPS_coord(FileData *fd, const gchar *key, gdouble value)
 			}
 		setlocale(LC_ALL, "C");
 
-		coordinate = g_strdup_printf("%i,%f,%s", deg, min, ref);
+		g_autofree gchar *coordinate = g_strdup_printf("%i,%f,%s", deg, min, ref);
 		metadata_write_string(fd, key, coordinate );
 
 		setlocale(LC_ALL, saved_locale);
 		free(saved_locale);
-		g_free(coordinate);
 		}
 
 	return ok;
@@ -925,77 +892,32 @@ gboolean metadata_append_list(FileData *fd, const gchar *key, const GList *value
 	return ret;
 }
 
-/**
- * @see find_string_in_list
- */
-static gchar *find_string_in_list_utf8nocase(GList *list, const gchar *string)
-{
-	gchar *string_casefold = g_utf8_casefold(string, -1);
-
-	while (list)
-		{
-		auto haystack = static_cast<gchar *>(list->data);
-
-		if (haystack)
-			{
-			gboolean equal;
-			gchar *haystack_casefold = g_utf8_casefold(haystack, -1);
-
-			equal = (strcmp(haystack_casefold, string_casefold) == 0);
-			g_free(haystack_casefold);
-
-			if (equal)
-				{
-				g_free(string_casefold);
-				return haystack;
-				}
-			}
-
-		list = list->next;
-		}
-
-	g_free(string_casefold);
-	return nullptr;
-}
-
-/**
- * @see find_string_in_list
- */
-static gchar *find_string_in_list_utf8case(GList *list, const gchar *string)
-{
-	while (list)
-		{
-		auto haystack = static_cast<gchar *>(list->data);
-
-		if (haystack && strcmp(haystack, string) == 0)
-			return haystack;
-
-		list = list->next;
-		} // while (list)
-
-	return nullptr;
-} // gchar *find_string_in_list_utf...
-
 /**
  * @brief Find a existent string in a list.
  *
- * This is a switch between find_string_in_list_utf8case and
- * find_string_in_list_utf8nocase to search with or without case for the
- * existence of a string.
+ * Search with or without case for the existence of a string.
  *
  * @param list The list to search in
  * @param string The string to search for
- * @return The string or NULL
- *
- * @see find_string_in_list_utf8case
- * @see find_string_in_list_utf8nocase
+ * @return The GList or NULL
  */
-static gchar *find_string_in_list(GList *list, const gchar *string)
+static GList *find_string_in_list(GList *list, const gchar *string)
 {
-	if (options->metadata.keywords_case_sensitive)
-		return find_string_in_list_utf8case(list, string);
+	if (!string) return nullptr;
 
-	return find_string_in_list_utf8nocase(list, string);
+	if (options->metadata.keywords_case_sensitive)
+		return g_list_find_custom(list, string, reinterpret_cast<GCompareFunc>(g_strcmp0));
+
+	g_autofree gchar *string_casefold = g_utf8_casefold(string, -1);
+	static const auto string_compare_utf8nocase = [](gconstpointer data, gconstpointer user_data)
+	{
+		if (!data) return 1;
+
+		g_autofree gchar *haystack = g_utf8_casefold(static_cast<const gchar *>(data), -1);
+		return strcmp(haystack, static_cast<const gchar *>(user_data));
+	};
+
+	return g_list_find_custom(list, string_casefold, string_compare_utf8nocase);
 }
 
 GList *string_to_keywords_list(const gchar *text)
@@ -1022,13 +944,13 @@ GList *string_to_keywords_list(const gchar *text)
 
 		if (l > 0)
 			{
-			gchar *keyword = g_strndup(begin, l);
+			g_autofree gchar *keyword = g_strndup(begin, l);
 
 			/* only add if not already in the list */
 			if (!find_string_in_list(list, keyword))
-				list = g_list_append(list, keyword);
-			else
-				g_free(keyword);
+				{
+				list = g_list_append(list, g_steal_pointer(&keyword));
+				}
 			}
 		}
 
@@ -1134,10 +1056,6 @@ void meta_data_connect_mark_with_keyword(GtkTreeModel *keyword_tree, GtkTreeIter
  *-------------------------------------------------------------------
  */
 
-
-
-GtkTreeStore *keyword_tree;
-
 gchar *keyword_get_name(GtkTreeModel *keyword_tree, GtkTreeIter *iter)
 {
 	gchar *name;
@@ -1169,12 +1087,11 @@ gboolean keyword_get_is_keyword(GtkTreeModel *keyword_tree, GtkTreeIter *iter)
 
 void keyword_set(GtkTreeStore *keyword_tree, GtkTreeIter *iter, const gchar *name, gboolean is_keyword)
 {
-	gchar *casefold = g_utf8_casefold(name, -1);
+	g_autofree gchar *casefold = g_utf8_casefold(name, -1);
 	gtk_tree_store_set(keyword_tree, iter, KEYWORD_COLUMN_MARK, "",
 						KEYWORD_COLUMN_NAME, name,
 						KEYWORD_COLUMN_CASEFOLD, casefold,
 						KEYWORD_COLUMN_IS_KEYWORD, is_keyword, -1);
-	g_free(casefold);
 }
 
 gboolean keyword_compare(GtkTreeModel *keyword_tree, GtkTreeIter *a, GtkTreeIter *b)
@@ -1209,7 +1126,6 @@ gboolean keyword_exists(GtkTreeModel *keyword_tree, GtkTreeIter *parent_ptr, Gtk
 	GtkTreeIter iter;
 	gboolean toplevel = FALSE;
 	gboolean ret;
-	gchar *casefold;
 
 	if (parent_ptr)
 		{
@@ -1226,7 +1142,7 @@ gboolean keyword_exists(GtkTreeModel *keyword_tree, GtkTreeIter *parent_ptr, Gtk
 
 	if (!gtk_tree_model_iter_children(GTK_TREE_MODEL(keyword_tree), &iter, toplevel ? nullptr : &parent)) return FALSE;
 
-	casefold = g_utf8_casefold(name, -1);
+	g_autofree gchar *casefold = g_utf8_casefold(name, -1);
 	ret = FALSE;
 
 	while (TRUE)
@@ -1235,15 +1151,13 @@ gboolean keyword_exists(GtkTreeModel *keyword_tree, GtkTreeIter *parent_ptr, Gtk
 			{
 			if (options->metadata.keywords_case_sensitive)
 				{
-				gchar *iter_name = keyword_get_name(keyword_tree, &iter);
+				g_autofree gchar *iter_name = keyword_get_name(keyword_tree, &iter);
 				ret = strcmp(name, iter_name) == 0;
-				g_free(iter_name);
 				}
 			else
 				{
-				gchar *iter_casefold = keyword_get_casefold(keyword_tree, &iter);
+				g_autofree gchar *iter_casefold = keyword_get_casefold(keyword_tree, &iter);
 				ret = strcmp(casefold, iter_casefold) == 0;
-				g_free(iter_casefold);
 				} // if (options->metadata.tags_cas...
 			}
 		if (ret)
@@ -1253,17 +1167,16 @@ gboolean keyword_exists(GtkTreeModel *keyword_tree, GtkTreeIter *parent_ptr, Gtk
 			}
 		if (!gtk_tree_model_iter_next(keyword_tree, &iter)) break;
 		}
-	g_free(casefold);
+
 	return ret;
 }
 
 
 void keyword_copy(GtkTreeStore *keyword_tree, GtkTreeIter *to, GtkTreeIter *from)
 {
-
-	gchar *mark;
-	gchar *name;
-	gchar *casefold;
+	g_autofree gchar *mark = nullptr;
+	g_autofree gchar *name = nullptr;
+	g_autofree gchar *casefold = nullptr;
 	gboolean is_keyword;
 
 	/* do not copy KEYWORD_COLUMN_HIDE_IN, it fully shows the new subtree */
@@ -1276,9 +1189,6 @@ void keyword_copy(GtkTreeStore *keyword_tree, GtkTreeIter *to, GtkTreeIter *from
 						KEYWORD_COLUMN_NAME, name,
 						KEYWORD_COLUMN_CASEFOLD, casefold,
 						KEYWORD_COLUMN_IS_KEYWORD, is_keyword, -1);
-	g_free(mark);
-	g_free(name);
-	g_free(casefold);
 }
 
 void keyword_copy_recursive(GtkTreeStore *keyword_tree, GtkTreeIter *to, GtkTreeIter *from)
@@ -1330,9 +1240,8 @@ gboolean keyword_tree_get_iter(GtkTreeModel *keyword_tree, GtkTreeIter *iter_ptr
 		GtkTreeIter children;
 		while (TRUE)
 			{
-			gchar *name = keyword_get_name(keyword_tree, &iter);
+			g_autofree gchar *name = keyword_get_name(keyword_tree, &iter);
 			if (strcmp(name, static_cast<const gchar *>(path->data)) == 0) break;
-			g_free(name);
 			if (!gtk_tree_model_iter_next(keyword_tree, &iter)) return FALSE;
 			}
 		path = path->next;
@@ -1372,9 +1281,8 @@ static gboolean keyword_tree_is_set_casefold(GtkTreeModel *keyword_tree, GtkTree
 
 		if (keyword_get_is_keyword(keyword_tree, &iter))
 			{
-			gchar *iter_casefold = keyword_get_casefold(keyword_tree, &iter);
+			g_autofree gchar *iter_casefold = keyword_get_casefold(keyword_tree, &iter);
 			GList *found = g_list_find_custom(casefold_list, iter_casefold, reinterpret_cast<GCompareFunc>(strcmp));
-			g_free(iter_casefold);
 			if (!found) return FALSE;
 			}
 
@@ -1407,9 +1315,8 @@ static gboolean keyword_tree_is_set_casefull(GtkTreeModel *keyword_tree, GtkTree
 
 		if (keyword_get_is_keyword(keyword_tree, &iter))
 			{
-			gchar *iter_name = keyword_get_name(keyword_tree, &iter);
+			g_autofree gchar *iter_name = keyword_get_name(keyword_tree, &iter);
 			GList *found = g_list_find_custom(kw_list, iter_name, reinterpret_cast<GCompareFunc>(strcmp));
-			g_free(iter_name);
 			if (!found) return FALSE;
 			}
 
@@ -1456,14 +1363,10 @@ void keyword_tree_set(GtkTreeModel *keyword_tree, GtkTreeIter *iter_ptr, GList *
 
 		if (keyword_get_is_keyword(keyword_tree, &iter))
 			{
-			gchar *name = keyword_get_name(keyword_tree, &iter);
+			g_autofree gchar *name = keyword_get_name(keyword_tree, &iter);
 			if (!find_string_in_list(*kw_list, name))
 				{
-				*kw_list = g_list_append(*kw_list, name);
-				}
-			else
-				{
-				g_free(name);
+				*kw_list = g_list_append(*kw_list, g_steal_pointer(&name));
 				}
 			}
 
@@ -1494,19 +1397,16 @@ GList *keyword_tree_get(GtkTreeModel *keyword_tree, GtkTreeIter *iter_ptr)
 
 static void keyword_tree_reset1(GtkTreeModel *keyword_tree, GtkTreeIter *iter, GList **kw_list)
 {
-	gchar *found;
-	gchar *name;
 	if (!keyword_get_is_keyword(keyword_tree, iter)) return;
 
-	name = keyword_get_name(keyword_tree, iter);
-	found = find_string_in_list(*kw_list, name);
+	g_autofree gchar *name = keyword_get_name(keyword_tree, iter);
 
+	GList *found = find_string_in_list(*kw_list, name);
 	if (found)
 		{
-		*kw_list = g_list_remove(*kw_list, found);
-		g_free(found);
+		g_free(found->data);
+		*kw_list = g_list_delete_link(*kw_list, found);
 		}
-	g_free(name);
 }
 
 static void keyword_tree_reset_recursive(GtkTreeModel *keyword_tree, GtkTreeIter *iter, GList **kw_list)
@@ -1681,11 +1581,12 @@ void keyword_show_set_in(GtkTreeStore *keyword_tree, gpointer id, GList *keyword
 }
 
 
-void keyword_tree_new()
+GtkTreeStore *keyword_tree_get_or_new()
 {
-	if (keyword_tree) return;
+	if (!keyword_tree)
+		keyword_tree = gtk_tree_store_new(KEYWORD_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_POINTER);
 
-	keyword_tree = gtk_tree_store_new(KEYWORD_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_POINTER);
+	return keyword_tree;
 }
 
 static GtkTreeIter keyword_tree_default_append(GtkTreeStore *keyword_tree, GtkTreeIter *parent, const gchar *name, gboolean is_keyword)
@@ -1696,13 +1597,13 @@ static GtkTreeIter keyword_tree_default_append(GtkTreeStore *keyword_tree, GtkTr
 	return iter;
 }
 
-void keyword_tree_new_default()
+void keyword_tree_set_default(GtkTreeStore *keyword_tree)
 {
+	if (!keyword_tree) return;
+
 	GtkTreeIter i1;
 	GtkTreeIter i2;
 
-	if (!keyword_tree) keyword_tree_new();
-
 	i1 = keyword_tree_default_append(keyword_tree, nullptr, _("People"), TRUE);
 		i2 = keyword_tree_default_append(keyword_tree, &i1, _("Family"), TRUE);
 		i2 = keyword_tree_default_append(keyword_tree, &i1, _("Free time"), TRUE);
@@ -1772,18 +1673,15 @@ static void keyword_tree_node_write_config(GtkTreeModel *keyword_tree, GtkTreeIt
 	while (TRUE)
 		{
 		GtkTreeIter children;
-		gchar *name;
-		gchar *mark_str;
 
 		WRITE_NL(); WRITE_STRING("<keyword ");
-		name = keyword_get_name(keyword_tree, &iter);
-		write_char_option(outstr, indent, "name", name);
-		g_free(name);
-		write_bool_option(outstr, indent, "kw", keyword_get_is_keyword(keyword_tree, &iter));
-		mark_str = keyword_get_mark(keyword_tree, &iter);
+		g_autofree gchar *name = keyword_get_name(keyword_tree, &iter);
+		write_char_option(outstr, "name", name);
+		write_bool_option(outstr, "kw", keyword_get_is_keyword(keyword_tree, &iter));
+		g_autofree gchar *mark_str = keyword_get_mark(keyword_tree, &iter);
 		if (mark_str && mark_str[0])
 			{
-			write_char_option(outstr, indent, "mark", mark_str);
+			write_char_option(outstr, "mark", mark_str);
 			}
 
 		if (gtk_tree_model_iter_children(keyword_tree, &children, &iter))
@@ -1860,7 +1758,7 @@ GtkTreeIter *keyword_add_from_config(GtkTreeStore *keyword_tree, GtkTreeIter *pa
 		if (READ_BOOL_FULL("kw", is_kw)) continue;
 		if (READ_CHAR_FULL("mark", mark_str)) continue;
 
-		log_printf("unknown attribute %s = %s\n", option, value);
+		config_file_error((std::string("Unknown attribute: ") + option + " = " + value).c_str());
 		}
 
 	if (name && name[0])
