@@ -23,10 +23,6 @@
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include "debug.h"
-#include "histogram.h" /* HCHAN_RGB */
-#include "image-overlay.h" /* OSD_SHOW_NOTHING */
-#include "image.h" /* RECTANGLE_DRAW_ASPECT_RATIO_NONE */
 #include "intl.h"
 #include "layout-image.h"
 #include "layout.h"
@@ -156,6 +152,22 @@ ConfOptions *init_options(ConfOptions *options)
 	options->image_overlay.background_blue = 240;
 	options->image_overlay.background_alpha = 210;
 
+	for (gint i = 0; i < OVERLAY_SCREEN_DISPLAY_PROFILE_COUNT; i++)
+		{
+		options->image_overlay_n.template_string[i] = nullptr;
+		options->image_overlay_n.x[i] = 10;
+		options->image_overlay_n.y[i] = -10;
+		options->image_overlay_n.font[i] = g_strdup("Sans 10");
+		options->image_overlay_n.text_red[i] = 0;
+		options->image_overlay_n.text_green[i] = 0;
+		options->image_overlay_n.text_blue[i] = 0;
+		options->image_overlay_n.text_alpha[i] = 255;
+		options->image_overlay_n.background_red[i] = 240;
+		options->image_overlay_n.background_green[i] = 240;
+		options->image_overlay_n.background_blue[i] = 240;
+		options->image_overlay_n.background_alpha[i] = 210;
+		}
+
 	options->lazy_image_sync = FALSE;
 	options->mousewheel_scrolls = FALSE;
 	options->image_lm_click_nav = TRUE;
@@ -256,16 +268,13 @@ ConfOptions *init_options(ConfOptions *options)
 
 void setup_default_options(ConfOptions *options)
 {
-	gchar *path;
 	gint i;
 
-	path = get_current_dir();
-	bookmark_add_default(".", path);
-	g_free(path);
+	g_autofree gchar *current_path = get_current_dir();
+	bookmark_add_default(".", current_path);
 	bookmark_add_default(_("Home"), homedir());
-	path = g_build_filename(homedir(), "Desktop", NULL);
-	bookmark_add_default(_("Desktop"), path);
-	g_free(path);
+	g_autofree gchar *desktop_path = g_build_filename(homedir(), "Desktop", NULL);
+	bookmark_add_default(_("Desktop"), desktop_path);
 	bookmark_add_default(_("Collections"), get_collections_dir());
 
 	g_free(options->file_ops.safe_delete_path);
@@ -277,7 +286,7 @@ void setup_default_options(ConfOptions *options)
 		options->color_profile.input_name[i] = nullptr;
 		}
 
-	set_default_image_overlay_template_string(&options->image_overlay.template_string);
+	set_default_image_overlay_template_string(options);
 	options->sidecar.ext = g_strdup(".jpg;%raw;.gqv;.xmp;%unknown");
 
 	options->shell.path = g_strdup(GQ_DEFAULT_SHELL_PATH);
@@ -291,76 +300,11 @@ void setup_default_options(ConfOptions *options)
 	options->help_search_engine = g_strdup(HELP_SEARCH_ENGINE);
 }
 
-void copy_layout_options(LayoutOptions *dest, const LayoutOptions *src)
-{
-	free_layout_options_content(dest);
-
-	*dest = *src;
-	dest->id = g_strdup(src->id);
-	dest->order = g_strdup(src->order);
-	dest->home_path = g_strdup(src->home_path);
-	dest->last_path = g_strdup(src->last_path);
-}
-
-void free_layout_options_content(LayoutOptions *dest)
-{
-	g_free(dest->id);
-	g_free(dest->order);
-	g_free(dest->home_path);
-	g_free(dest->last_path);
-}
-
-LayoutOptions *init_layout_options(LayoutOptions *options)
-{
-	memset(options, 0, sizeof(LayoutOptions));
-
-	options->dir_view_type = DIRVIEW_LIST;
-	options->dir_view_list_sort.ascend = TRUE;
-	options->dir_view_list_sort.case_sensitive = TRUE;
-	options->dir_view_list_sort.method = SORT_NAME;
-	options->file_view_list_sort.ascend = TRUE;
-	options->file_view_list_sort.case_sensitive = TRUE;
-	options->file_view_list_sort.method = SORT_NAME;
-	options->file_view_type = FILEVIEW_LIST;
-	options->float_window.rect = {0, 0, 260, 450};
-	options->float_window.vdivider_pos = -1;
-	options->home_path = nullptr;
-	options->id = g_strdup("null");
-	options->main_window.hdivider_pos = -1;
-	options->main_window.maximized = FALSE;
-	options->main_window.rect = {0, 0, 720, 540};
-	options->main_window.vdivider_pos = 200;
-	options->search_window = {100, 100, 700, 650};
-	options->dupe_window = {100, 100, 800, 400};
-	options->advanced_exif_window = {0, 0, 900, 600};
-	options->folder_window.vdivider_pos = 100;
-	options->order = g_strdup("123");
-	options->show_directory_date = FALSE;
-	options->show_marks = FALSE;
-	options->show_file_filter = FALSE;
-	options->show_thumbnails = FALSE;
-	options->style = 0;
-	options->show_info_pixel = FALSE;
-	options->selectable_toolbars_hidden = FALSE;
-	options->tools_float = FALSE;
-	options->tools_hidden = FALSE;
-	options->image_overlay.histogram_channel = HCHAN_RGB;
-	options->image_overlay.histogram_mode = 1;
-	options->image_overlay.state = OSD_SHOW_NOTHING;
-	options->animate = TRUE;
-	options->bars_state.hidden = FALSE;
-	options->log_window = {0, 0, 520, 400};
-	options->preferences_window.rect = {0, 0, 700, 600};
-	options->split_pane_sync = FALSE;
-	options->workspace = -1;
-	return options;
-}
-
 static void sync_options_with_current_state(ConfOptions *options)
 {
-	LayoutWindow *lw = nullptr;
+	LayoutWindow *lw = get_current_layout();
 
-	if (layout_valid(&lw))
+	if (lw)
 		{
 		layout_sync_options_with_current_state(lw);
 
@@ -374,32 +318,32 @@ static void sync_options_with_current_state(ConfOptions *options)
 
 void save_options(ConfOptions *options)
 {
-	gchar *rc_path;
-
 	sync_options_with_current_state(options);
 
-	rc_path = g_build_filename(get_rc_dir(), RC_FILE_NAME, NULL);
+	g_autofree gchar *rc_path = g_build_filename(get_rc_dir(), RC_FILE_NAME, NULL);
 	save_config_to_file(rc_path, options, nullptr);
-	g_free(rc_path);
 }
 
 gboolean load_options(ConfOptions *)
 {
 	gboolean success;
-	gchar *rc_path;
 
 	if (isdir(GQ_SYSTEM_WIDE_DIR))
 		{
-		rc_path = g_build_filename(GQ_SYSTEM_WIDE_DIR, RC_FILE_NAME, NULL);
+		g_autofree gchar *rc_path = g_build_filename(GQ_SYSTEM_WIDE_DIR, RC_FILE_NAME, NULL);
 		success = load_config_from_file(rc_path, TRUE);
 		DEBUG_1("Loading options from %s ... %s", rc_path, success ? "done" : "failed");
-		g_free(rc_path);
 		}
 
-	rc_path = g_build_filename(get_rc_dir(), RC_FILE_NAME, NULL);
+	g_autofree gchar *rc_path = g_build_filename(get_rc_dir(), RC_FILE_NAME, NULL);
 	success = load_config_from_file(rc_path, TRUE);
 	DEBUG_1("Loading options from %s ... %s", rc_path, success ? "done" : "failed");
-	g_free(rc_path);
-	return(success);
+	return success;
+}
+
+void set_default_image_overlay_template_string(ConfOptions *options)
+{
+	g_free(options->image_overlay.template_string);
+	options->image_overlay.template_string = g_strdup(DEFAULT_OVERLAY_INFO);
 }
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
