@@ -24,11 +24,11 @@
 
 #include <glib.h>
 
-struct FileData;
+class FileData;
 struct PanItem;
 struct PanWindow;
 
 void pan_calendar_update(PanWindow *pw, PanItem *pi_day);
-void pan_calendar_compute(PanWindow *pw, FileData *dir_fd, gint *width, gint *height);
+void pan_calendar_compute(PanWindow *pw, FileData *dir_fd, gint &width, gint &height);
 
 #endif
