@@ -29,26 +29,6 @@
 
 struct PanWindow;
 
-enum PanViewFilterMode {
-	PAN_VIEW_FILTER_REQUIRE,
-	PAN_VIEW_FILTER_EXCLUDE,
-	PAN_VIEW_FILTER_INCLUDE,
-	PAN_VIEW_FILTER_GROUP
-};
-
-struct PanViewFilterElement
-{
-	PanViewFilterMode mode;
-	gchar *keyword;
-	GRegex *kw_regex;
-};
-
-struct PanFilterCallbackState
-{
-	PanWindow *pw;
-	GList *filter_element;
-};
-
 struct PanViewFilterUi
 {
 	GtkWidget *filter_box;
@@ -58,16 +38,11 @@ struct PanViewFilterUi
 	GtkWidget *filter_button_arrow;
 	GtkWidget *filter_kw_hbox;
 	GtkWidget *filter_check_buttons[FILE_FORMAT_CLASSES];
-	GtkListStore *filter_mode_model;
 	GtkWidget *filter_mode_combo;
 	GList *filter_elements;  /**< List of #PanViewFilterElement. */
 	gint filter_classes;
 };
 
-void pan_filter_activate_cb(const gchar *text, gpointer data);
-void pan_filter_toggle_cb(GtkWidget *button, gpointer data);
-void pan_filter_toggle_button_cb(GtkWidget *button, gpointer data);
-
 /**
  * @headerfile pan_filter_ui_new
  * Creates a new #PanViewFilterUi instance and returns it.
@@ -76,9 +51,9 @@ PanViewFilterUi *pan_filter_ui_new(PanWindow *pw);
 
 /**
  * @headerfile pan_filter_ui_destroy
- * Destroys the specified #PanViewFilterUi and sets the pointer to NULL.
+ * Destroys the specified #PanViewFilterUi.
  */
-void pan_filter_ui_destroy(PanViewFilterUi **ui);
+void pan_filter_ui_destroy(PanViewFilterUi *ui);
 
 gboolean pan_filter_fd_list(GList **fd_list, GList *filter_elements, gint filter_classes);
 
