@@ -76,13 +76,6 @@ using PixbufRendererTileDisposeFunc = void (*)(PixbufRenderer *, gint, gint, gin
 
 using PixbufRendererPostProcessFunc = void (*)(PixbufRenderer *, GdkPixbuf **, gint, gint, gint, gint, gpointer);
 
-enum PixbufRendererScrollResetType {
-	PR_SCROLL_RESET_TOPLEFT = 0,
-	PR_SCROLL_RESET_CENTER,
-	PR_SCROLL_RESET_NOCHANGE,
-	PR_SCROLL_RESET_COUNT,
-};
-
 enum ImageRenderType {
 	TILE_RENDER_NONE = 0, /**< do nothing */
 	TILE_RENDER_AREA, /**< render an area of the tile */
@@ -97,8 +90,8 @@ enum OverlayRendererFlags {
 
 struct RendererFuncs
 {
-    void (*area_changed)(void *renderer, gint src_x, gint src_y, gint src_w, gint src_h); /**< pixbuf area changed */
-	void (*invalidate_region)(void *renderer, gint x, gint y, gint w, gint h);
+	void (*area_changed)(void *renderer, gint src_x, gint src_y, gint src_w, gint src_h); /**< pixbuf area changed */
+	void (*invalidate_region)(void *renderer, GdkRectangle region);
 	void (*scroll)(void *renderer, gint x_off, gint y_off); /**< scroll */
 	void (*update_viewport)(void *renderer); /**< window / wiewport / border color has changed */
 	void (*update_pixbuf)(void *renderer, gboolean lazy); /**< pixbuf has changed */
@@ -146,7 +139,7 @@ struct PixbufRenderer
 	gint y_scroll;
 
 	gdouble norm_center_x;	/**< coordinates of viewport center in the image, in range 0.0 - 1.0 */
-	gdouble norm_center_y;  /**< these coordinates are used for PR_SCROLL_RESET_NOCHANGE and should be preserved over periods with NULL pixbuf */
+	gdouble norm_center_y;  /**< these coordinates are used for ScrollReset::NOCHANGE and should be preserved over periods with NULL pixbuf */
 
 	gdouble subpixel_x_scroll; /**< subpixel scroll alignment, used to prevent accumulation of rounding errors */
 	gdouble subpixel_y_scroll;
@@ -162,7 +155,7 @@ struct PixbufRenderer
 	gboolean zoom_2pass;
 	gboolean zoom_expand;
 
-	PixbufRendererScrollResetType scroll_reset;
+	ScrollReset scroll_reset;
 
 	gboolean has_frame;
 
@@ -255,15 +248,8 @@ PixbufRenderer *pixbuf_renderer_new();
 
 void pixbuf_renderer_set_parent(PixbufRenderer *pr, GtkWindow *window);
 
-/**
- * @headerfile pixbuf_renderer_set_pixbuf
- * display a pixbuf
- */
 void pixbuf_renderer_set_pixbuf(PixbufRenderer *pr, GdkPixbuf *pixbuf, gdouble zoom);
 
-/**
- * @headerfile  pixbuf_renderer_set_pixbuf_lazy
- * same as pixbuf_renderer_set_pixbuf but waits with redrawing for pixbuf_renderer_area_changed */
 void pixbuf_renderer_set_pixbuf_lazy(PixbufRenderer *pr, GdkPixbuf *pixbuf, gdouble zoom, gint orientation, StereoPixbufData stereo_data);
 
 
@@ -271,18 +257,10 @@ GdkPixbuf *pixbuf_renderer_get_pixbuf(PixbufRenderer *pr);
 
 void pixbuf_renderer_set_orientation(PixbufRenderer *pr, gint orientation);
 
-/**
- * @headerfile pixbuf_renderer_set_stereo_data
- * sets the format of stereo data in the input pixbuf
- */
 void pixbuf_renderer_set_stereo_data(PixbufRenderer *pr, StereoPixbufData stereo_data);
 
 void pixbuf_renderer_set_post_process_func(PixbufRenderer *pr, PixbufRendererPostProcessFunc func, gpointer user_data, gboolean slow);
 
-/**
- * @headerfile pixbuf_renderer_set_tiles
- * display an on-request array of pixbuf tiles
- */
 void pixbuf_renderer_set_tiles(PixbufRenderer *pr, gint width, gint height,
 			       gint tile_width, gint tile_height, gint cache_size,
 			       PixbufRendererTileRequestFunc func_request,
@@ -292,17 +270,9 @@ void pixbuf_renderer_set_tiles(PixbufRenderer *pr, gint width, gint height,
 void pixbuf_renderer_set_tiles_size(PixbufRenderer *pr, gint width, gint height);
 gint pixbuf_renderer_get_tiles(PixbufRenderer *pr);
 
-/**
- * @headerfile pixbuf_renderer_move
- * move image data from source to pr, source is then set to NULL image
- */
 void pixbuf_renderer_move(PixbufRenderer *pr, PixbufRenderer *source);
 void pixbuf_renderer_copy(PixbufRenderer *pr, PixbufRenderer *source);
 
-/**
- * @headerfile pixbuf_renderer_area_changed
- * update region of existing image
- */
 void pixbuf_renderer_area_changed(PixbufRenderer *pr, gint x, gint y, gint width, gint height);
 
 /* scrolling */
@@ -329,16 +299,8 @@ void pixbuf_renderer_zoom_set_limits(PixbufRenderer *pr, gdouble min, gdouble ma
 gboolean pixbuf_renderer_get_image_size(PixbufRenderer *pr, gint *width, gint *height);
 gboolean pixbuf_renderer_get_scaled_size(PixbufRenderer *pr, gint *width, gint *height);
 
-/**
- * @headerfile pixbuf_renderer_get_visible_rect
- * region of image in pixel coordinates
- */
 gboolean pixbuf_renderer_get_visible_rect(PixbufRenderer *pr, GdkRectangle *rect);
 
-/**
- * @headerfile pixbuf_renderer_set_color
- *  background color
- */
 void pixbuf_renderer_set_color(PixbufRenderer *pr, GdkRGBA *color);
 
 /* overlay */
@@ -351,18 +313,13 @@ void pixbuf_renderer_overlay_remove(PixbufRenderer *pr, gint id);
 
 gboolean pixbuf_renderer_get_mouse_position(PixbufRenderer *pr, gint *x_pixel, gint *y_pixel);
 
-/**
- * @headerfile pixbuf_renderer_get_pixel_colors
- * x_pixel and y_pixel are the pixel coordinates see #pixbuf_renderer_get_mouse_position
- */
 gboolean pixbuf_renderer_get_pixel_colors(PixbufRenderer *pr, gint x_pixel, gint y_pixel,
-	 				gint *r_mouse, gint *g_mouse, gint *b_mouse);
+	 				gint *r_mouse, gint *g_mouse, gint *b_mouse, gint *a_mouse);
 
 void pixbuf_renderer_set_size_early(PixbufRenderer *pr, guint width, guint height);
 
 /* stereo */
 void pixbuf_renderer_stereo_set(PixbufRenderer *pr, gint stereo_mode);
-gint pixbuf_renderer_stereo_get(PixbufRenderer *pr);
 void pixbuf_renderer_stereo_fixed_set(PixbufRenderer *pr, gint width, gint height, gint x1, gint y1, gint x2, gint y2);
 
 /**
@@ -378,28 +335,20 @@ struct SourceTile
 };
 
 
-gboolean pr_clip_region(gint x, gint y, gint w, gint h,
-			       gint clip_x, gint clip_y, gint clip_w, gint clip_h,
-			       gint *rx, gint *ry, gint *rw, gint *rh);
 void pr_render_complete_signal(PixbufRenderer *pr);
 
 void pr_tile_coords_map_orientation(gint orientation,
-				     gdouble tile_x, gdouble tile_y, /**< coordinates of the tile */
-				     gdouble image_w, gdouble image_h,
-				     gdouble tile_w, gdouble tile_h,
-				     gdouble *res_x, gdouble *res_y);
-void pr_tile_region_map_orientation(gint orientation,
-				     gint area_x, gint area_y, /**< coordinates of the area inside tile */
-				     gint tile_w, gint tile_h,
-				     gint area_w, gint area_h,
-				     gint *res_x, gint *res_y,
-				     gint *res_w, gint *res_h);
-void pr_coords_map_orientation_reverse(gint orientation,
-				     gint area_x, gint area_y,
-				     gint tile_w, gint tile_h,
-				     gint area_w, gint area_h,
-				     gint *res_x, gint *res_y,
-				     gint *res_w, gint *res_h);
+                                    gdouble tile_x, gdouble tile_y, /**< coordinates of the tile */
+                                    gdouble image_w, gdouble image_h,
+                                    gdouble tile_w, gdouble tile_h,
+                                    gdouble &res_x, gdouble &res_y);
+GdkRectangle pr_tile_region_map_orientation(gint orientation,
+                                            GdkRectangle area, /**< coordinates of the area inside tile */
+                                            gint tile_w, gint tile_h);
+GdkRectangle pr_coords_map_orientation_reverse(gint orientation,
+                                               GdkRectangle area,
+                                               gint tile_w, gint tile_h);
+void pr_scale_region(GdkRectangle &region, gdouble scale);
 
 GList *pr_source_tile_compute_region(PixbufRenderer *pr, gint x, gint y, gint w, gint h, gboolean request);
 
