@@ -32,7 +32,6 @@
 
 #include <config.h>
 
-#include "debug.h"
 #include "intl.h"
 
 /**
@@ -83,12 +82,15 @@
  * @FIXME Low risk race conditions about ssi->file_name.
  */
 
-SecureSaveErrno secsave_errno = SS_ERR_NONE;
+namespace
+{
+
+SecureSaveErrno secsave_errno = SS_ERR_NONE; /**< internal secsave error number */
 
 
-/** Open a file for writing in a secure way. @returns a pointer to a
- * structure secure_save_info on success, or NULL on failure. */
-static SecureSaveInfo *
+/** Open a file for writing in a secure way.
+ * @returns a pointer to a structure secure_save_info on success, or NULL on failure. */
+SecureSaveInfo *
 secure_open_umask(const gchar *file_name)
 {
 	struct stat st;
@@ -103,7 +105,7 @@ secure_open_umask(const gchar *file_name)
 
 	ssi->secure_save = TRUE;
 	ssi->preserve_perms = TRUE;
-	ssi->unlink_on_error = TRUE;
+	ssi->remove_on_error = TRUE;
 
 	ssi->file_name = g_strdup(file_name);
 	if (!ssi->file_name) {
@@ -197,6 +199,8 @@ secure_open_umask(const gchar *file_name)
 	return g_steal_pointer(&ssi);
 }
 
+} // namespace
+
 SecureSaveInfo *
 secure_open(const gchar *file_name)
 {
@@ -228,7 +232,7 @@ secure_close(SecureSaveInfo *ssi)
 	{
 		if (ssi->tmp_file_name)
 			{
-			if (ret && ssi->unlink_on_error) unlink(ssi->tmp_file_name);
+			if (ret && ssi->remove_on_error) std::remove(ssi->tmp_file_name);
 			g_free(ssi->tmp_file_name);
 			}
 		g_free(ssi->file_name);
@@ -393,10 +397,16 @@ secure_fwrite(gconstpointer ptr, size_t size, size_t nmemb, SecureSaveInfo *ssi)
 	return ret;
 }
 
-gchar *
-secsave_strerror(SecureSaveErrno secsave_error)
+bool
+secsave_succeed()
 {
-	switch (secsave_error) {
+	return secsave_errno == SS_ERR_NONE;
+}
+
+const gchar *
+secsave_strerror()
+{
+	switch (secsave_errno) {
 	case SS_ERR_OPEN_READ:
 		return _("Cannot read the file");
 	case SS_ERR_STAT:
