@@ -43,8 +43,6 @@ enum SecureSaveErrno {
 	SS_ERR_OTHER,
 };
 
-extern SecureSaveErrno secsave_errno; /**< internal secsave error number */
-
 struct SecureSaveInfo {
 	FILE *fp; /**< file stream pointer */
 	gchar *file_name; /**< final file name */
@@ -53,7 +51,7 @@ struct SecureSaveInfo {
 	gboolean secure_save; /**< use secure save for this file, internal use only */
 	gboolean preserve_perms; /**< whether to preserve perms, TRUE by default */
 	gboolean preserve_mtime; /**< whether to preserve mtime, FALSE by default */
-	gboolean unlink_on_error; /**< whether to remove temporary file on save failure, TRUE by default */
+	gboolean remove_on_error; /**< whether to remove temporary file on save failure, TRUE by default */
 };
 
 SecureSaveInfo *secure_open(const gchar *);
@@ -63,10 +61,11 @@ gint secure_close(SecureSaveInfo *);
 gint secure_fputs(SecureSaveInfo *, const gchar *);
 gint secure_fputc(SecureSaveInfo *, gint);
 
-gint secure_fprintf(SecureSaveInfo *, const gchar *, ...);
+gint secure_fprintf(SecureSaveInfo *, const gchar *, ...) G_GNUC_PRINTF(2, 3);
 size_t secure_fwrite(gconstpointer ptr, size_t size, size_t nmemb, SecureSaveInfo *ssi);
 
-gchar *secsave_strerror(SecureSaveErrno);
+bool secsave_succeed();
+const gchar *secsave_strerror();
 
 #endif /* SECURE_SAVE_H */
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
