@@ -32,8 +32,6 @@
 #include "ui-bookmark.h"
 #include "ui-fileops.h"
 #include "ui-misc.h"
-#include "ui-utildlg.h"
-#include "utilops.h"
 
 namespace
 {
@@ -44,8 +42,8 @@ struct ShortcutsData
 	GtkWidget *bookmarks;
 	LayoutWindow *lw;
 
-	FileDialog *dialog;
-	GtkWidget *dialog_name_entry;
+	gchar *name;
+	GtkPopover *name_popover;
 
 	GtkWidget *add_button;
 };
@@ -67,81 +65,85 @@ void shortcuts_bookmark_select(const gchar *path, gpointer data)
 
 }
 
-void shortcuts_add_close(ShortcutsData *scd)
-{
-	if (scd->dialog) file_dialog_close(scd->dialog);
-	scd->dialog_name_entry = nullptr;
-	scd->dialog = nullptr;
-}
-
-void shortcuts_add_ok_cb(FileDialog *fd, gpointer data)
+void name_entry_activate_cb(GtkEntry *entry, gpointer data)
 {
 	auto scd = static_cast<ShortcutsData *>(data);
-	const gchar *name = gtk_entry_get_text(GTK_ENTRY(scd->dialog_name_entry));
-	gboolean empty_name = (name[0] == '\0');
 
-	name = gtk_entry_get_text(GTK_ENTRY(scd->dialog_name_entry));
+	g_free(scd->name);
+	scd->name = g_strdup(gtk_entry_get_text(entry));
 
-	if (empty_name)
+	gtk_popover_popdown(GTK_POPOVER(scd->name_popover));
+}
+
+void add_shortcut_cb(GtkFileChooser *chooser, gint response_id, gpointer data)
+{
+	auto scd = static_cast<ShortcutsData *>(data);
+
+	if (response_id == GTK_RESPONSE_ACCEPT)
 		{
-		name = filename_from_path(fd->dest_path);
+		gboolean empty_name = (scd->name == nullptr);
+
+		g_autoptr(GFile) file = gtk_file_chooser_get_file(chooser);
+		g_autofree gchar *selected_dir = g_file_get_path(file);
+
+		bookmark_list_add(scd->bookmarks, empty_name ? filename_from_path(selected_dir) : scd->name, selected_dir);
+
+		g_free(scd->name);
+		scd->name = nullptr;
 		}
 
-	bookmark_list_add(scd->bookmarks, name, fd->dest_path);
-
-	shortcuts_add_close(scd);
-}
-
-void shortcuts_add_cancel_cb(FileDialog *, gpointer data)
-{
-	auto scd = static_cast<ShortcutsData *>(data);
-
-	shortcuts_add_close(scd);
-}
-
-void shortcuts_add_cb(GtkWidget *button, gpointer data)
-{
-	auto scd = static_cast<ShortcutsData *>(data);
-	GtkWidget *hbox;
-	const gchar *title;
-
-	if (scd->dialog)
+	if (response_id == GQ_RESPONSE_NAME_CLICKED)
 		{
-		gtk_window_present(GTK_WINDOW(GENERIC_DIALOG(scd->dialog)->dialog));
+		gtk_popover_popup(GTK_POPOVER(scd->name_popover));
+		gq_gtk_widget_show_all(GTK_WIDGET(scd->name_popover));
+		gtk_widget_grab_focus(GTK_WIDGET(scd->name_popover));
+
 		return;
 		}
 
-	title = _("Add Shortcut");
-	scd->dialog = file_util_file_dlg(title,
-					"add_shortcuts", button,
-					shortcuts_add_cancel_cb, scd);
-	file_dialog_add_button(scd->dialog, GQ_ICON_OK, "OK", shortcuts_add_ok_cb, TRUE);
+	gq_gtk_widget_destroy(GTK_WIDGET(chooser));
+}
 
-	generic_dialog_add_message(GENERIC_DIALOG(scd->dialog), nullptr, title, nullptr, FALSE);
+void shortcuts_add_cb(GtkWidget *, gpointer data)
+{
+	auto scd = static_cast<ShortcutsData *>(data);
+	GtkWidget *dialog;
+	GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 
-	file_dialog_add_path_widgets(scd->dialog, nullptr, nullptr, "add_shortcuts", nullptr, nullptr);
+	dialog = gtk_file_chooser_dialog_new(_("Add Shortcut - Geeqie"), nullptr, action, _("_Cancel"), GTK_RESPONSE_CANCEL, _("Add"), GTK_RESPONSE_ACCEPT, _("Name"), GQ_RESPONSE_NAME_CLICKED, nullptr);
 
-	hbox = pref_box_new(GENERIC_DIALOG(scd->dialog)->vbox, FALSE, GTK_ORIENTATION_HORIZONTAL, PREF_PAD_GAP);
+	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), layout_get_path(get_current_layout()));
+	gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(dialog), layout_get_path(get_current_layout()), nullptr);
 
-	pref_label_new(hbox, _("Name:"));
+	gtk_widget_set_tooltip_text(gtk_dialog_get_widget_for_response(GTK_DIALOG(dialog), GQ_RESPONSE_NAME_CLICKED), _("Optional alias name for the shortcut.\nThis may be amended or added from the Shortcuts pane.\nIf none given, the basename of the folder is usedddd"));
 
-	scd->dialog_name_entry = gtk_entry_new();
-	gq_gtk_box_pack_start(GTK_BOX(hbox), scd->dialog_name_entry, TRUE, TRUE, 0);
-	generic_dialog_attach_default(GENERIC_DIALOG(scd->dialog), scd->dialog_name_entry);
-	gtk_widget_show(scd->dialog_name_entry);
+	GtkWidget *entry = gtk_entry_new();
+	GtkWidget *name_popover = gtk_popover_new(GTK_WIDGET(gtk_dialog_get_widget_for_response(GTK_DIALOG(dialog), GQ_RESPONSE_NAME_CLICKED)));
+	scd->name_popover = GTK_POPOVER(name_popover);
 
-	gtk_widget_show(GENERIC_DIALOG(scd->dialog)->dialog);
+	g_signal_connect(GTK_ENTRY(entry), "activate", G_CALLBACK(name_entry_activate_cb), scd);
+
+	gtk_popover_set_position(GTK_POPOVER(name_popover), GTK_POS_BOTTOM);
+	gq_gtk_container_add(GTK_WIDGET(name_popover), entry);
+	gtk_container_set_border_width(GTK_CONTAINER(name_popover), 6);
+
+	gtk_file_chooser_set_create_folders(GTK_FILE_CHOOSER(dialog), TRUE);
+
+	g_signal_connect(dialog, "response", G_CALLBACK(add_shortcut_cb), scd);
+
+	gq_gtk_widget_show_all(GTK_WIDGET(dialog));
 }
 
 void shortcuts_destroy(gpointer data)
 {
 	auto scd = static_cast<ShortcutsData *>(data);
 
-	shortcuts_add_close(scd);
-
+	g_free(scd->name);
 	g_free(scd);
 }
 
+} // namespace
+
 GtkWidget *shortcuts_new(LayoutWindow *lw)
 {
 	ShortcutsData *scd;
@@ -166,24 +168,8 @@ GtkWidget *shortcuts_new(LayoutWindow *lw)
 					_("Add Shortcut"),
 					G_CALLBACK(shortcuts_add_cb), scd);
 
+	gtk_widget_show(scd->vbox);
 	return scd->vbox;
 }
 
-GtkWidget *shortcuts_new_from_config(LayoutWindow *lw, const gchar **, const gchar **)
-{
-	GtkWidget *shortcuts_bar;
-
-	shortcuts_bar = shortcuts_new(lw);
-	gtk_widget_show(shortcuts_bar);
-
-	return shortcuts_bar;
-}
-
-} // namespace
-
-GtkWidget *shortcuts_new_default(LayoutWindow *lw)
-{
-	return shortcuts_new_from_config(lw, nullptr, nullptr);
-}
-
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
