@@ -33,8 +33,8 @@
 
 #include <config.h>
 
+#include "compat-deprecated.h"
 #include "compat.h"
-#include "debug.h"
 #include "history-list.h"
 #include "layout.h"
 #include "main-defines.h"
@@ -696,8 +696,8 @@ static void date_selection_popup_hide(DateSelection *ds)
 	if (gtk_widget_has_grab(ds->window))
 		{
 		gtk_grab_remove(ds->window);
-		gdk_keyboard_ungrab(GDK_CURRENT_TIME);
-		gdk_pointer_ungrab(GDK_CURRENT_TIME);
+		gq_gdk_keyboard_ungrab(GDK_CURRENT_TIME);
+		gq_gdk_pointer_ungrab(GDK_CURRENT_TIME);
 		}
 
 	gtk_widget_hide(ds->window);
@@ -840,21 +840,21 @@ static void date_selection_popup(DateSelection *ds)
 	x = wx + button_allocation.x + button_allocation.width - window_allocation.width;
 	y = wy + button_allocation.y + button_allocation.height;
 
-	if (y + window_allocation.height > gdk_screen_height())
+	if (y + window_allocation.height > gq_gdk_screen_height())
 		{
 		y = wy + button_allocation.y - window_allocation.height;
 		}
-	if (x < 0) x = 0;
-	if (y < 0) y = 0;
+	x = std::max(x, 0);
+	y = std::max(y, 0);
 
 	gq_gtk_window_move(GTK_WINDOW(ds->window), x, y);
 	gtk_widget_show(ds->window);
 
 	gtk_widget_grab_focus(ds->calendar);
-	gdk_pointer_grab(gtk_widget_get_window(ds->window), TRUE,
-			 static_cast<GdkEventMask>(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_MOTION_MASK),
-			 nullptr, nullptr, GDK_CURRENT_TIME);
-	gdk_keyboard_grab(gtk_widget_get_window(ds->window), TRUE, GDK_CURRENT_TIME);
+	gq_gdk_pointer_grab(gtk_widget_get_window(ds->window), TRUE,
+	                    static_cast<GdkEventMask>(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_MOTION_MASK),
+	                    nullptr, nullptr, GDK_CURRENT_TIME);
+	gq_gdk_keyboard_grab(gtk_widget_get_window(ds->window), TRUE, GDK_CURRENT_TIME);
 	gtk_grab_add(ds->window);
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ds->button), TRUE);
@@ -874,7 +874,7 @@ static void button_size_allocate_cb(GtkWidget *button, GtkAllocation *allocation
 {
 	auto spin = static_cast<GtkWidget *>(data);
 	GtkRequisition spin_requisition;
-	gtk_widget_get_requisition(spin, &spin_requisition);
+	gq_gtk_widget_get_requisition(spin, &spin_requisition);
 
 	if (allocation->height > spin_requisition.height)
 		{
@@ -894,7 +894,7 @@ static void spin_increase(GtkWidget *spin, gint value)
 {
 	GtkRequisition req;
 
-	gtk_widget_size_request(spin, &req);
+	gq_gtk_widget_size_request(spin, &req);
 	gtk_widget_set_size_request(spin, req.width + value, -1);
 }
 
@@ -1052,9 +1052,7 @@ static GList *pref_list_find(const gchar *group, const gchar *token)
 
 static gboolean pref_list_get(const gchar *group, const gchar *key, const gchar *marker, const gchar **result)
 {
-	gchar *token;
 	GList *work;
-	gboolean ret;
 
 	if (!group || !key || !marker)
 		{
@@ -1062,36 +1060,28 @@ static gboolean pref_list_get(const gchar *group, const gchar *key, const gchar
 		return FALSE;
 		}
 
-	token = g_strconcat(key, marker, NULL);
+	g_autofree gchar *token = g_strconcat(key, marker, NULL);
 
 	work = pref_list_find(group, token);
-	if (work)
-		{
-		*result = static_cast<const gchar *>(work->data) + strlen(token);
-		if (*result[0] == '\0') *result = nullptr;
-		ret = TRUE;
-		}
-	else
+	if (!work)
 		{
 		*result = nullptr;
-		ret = FALSE;
+		return FALSE;
 		}
 
-	g_free(token);
-
-	return ret;
+	*result = static_cast<const gchar *>(work->data) + strlen(token);
+	if (*result[0] == '\0') *result = nullptr;
+	return TRUE;
 }
 
 static void pref_list_set(const gchar *group, const gchar *key, const gchar *marker, const gchar *text)
 {
-	gchar *token;
-	gchar *path;
 	GList *work;
 
 	if (!group || !key || !marker) return;
 
-	token = g_strconcat(key, marker, NULL);
-	path = g_strconcat(token, text, NULL);
+	g_autofree gchar *token = g_strconcat(key, marker, NULL);
+	g_autofree gchar *path = g_strconcat(token, text, NULL);
 
 	work = pref_list_find(group, token);
 	if (work)
@@ -1100,8 +1090,7 @@ static void pref_list_set(const gchar *group, const gchar *key, const gchar *mar
 
 		if (text)
 			{
-			work->data = path;
-			path = nullptr;
+			work->data = g_steal_pointer(&path);
 
 			g_free(old_path);
 			}
@@ -1114,9 +1103,6 @@ static void pref_list_set(const gchar *group, const gchar *key, const gchar *mar
 		{
 		history_list_add_to_key(group, path, 0);
 		}
-
-	g_free(path);
-	g_free(token);
 }
 
 void pref_list_int_set(const gchar *group, const gchar *key, gint value)
@@ -1196,18 +1182,17 @@ GtkWidget *pref_color_button_new(GtkWidget *parent_box, const gchar *title, GdkR
  *-----------------------------------------------------------------------------
  */
 
-gchar *text_widget_text_pull(GtkWidget *text_widget)
+gchar *text_widget_text_pull(GtkWidget *text_widget, gboolean include_hidden_chars)
 {
 	if (GTK_IS_TEXT_VIEW(text_widget))
 		{
-		GtkTextBuffer *buffer;
+		GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_widget));
+
 		GtkTextIter start;
 		GtkTextIter end;
-
-		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_widget));
 		gtk_text_buffer_get_bounds(buffer, &start, &end);
 
-		return gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
+		return gtk_text_buffer_get_text(buffer, &start, &end, include_hidden_chars);
 		}
 
 	if (GTK_IS_ENTRY(text_widget))
@@ -1216,8 +1201,6 @@ gchar *text_widget_text_pull(GtkWidget *text_widget)
 		}
 
 	return nullptr;
-	
-
 }
 
 gchar *text_widget_text_pull_selected(GtkWidget *text_widget)
@@ -1335,14 +1318,13 @@ static gchar *get_action_label(GtkAction *action, const gchar *action_name)
  * @param
  * @returns std::vector<ActionItem>
  *
- * The list generated is used in the --remote --action-list command and
+ * The list generated is used in the --action-list command and
  * programmable mouse buttons 8 and 9.
  */
 std::vector<ActionItem> get_action_items()
 {
-	LayoutWindow *lw = nullptr;
-
-	if (!layout_valid(&lw))
+	LayoutWindow *lw = get_current_layout();
+	if (!lw)
 		{
 		return {};
 		}
@@ -1351,10 +1333,10 @@ std::vector<ActionItem> get_action_items()
 
 	for (GList *groups = gq_gtk_ui_manager_get_action_groups(lw->ui_manager); groups; groups = groups->next)
 		{
-		GtkActionGroup *action_group = GTK_ACTION_GROUP(groups->data);
+		GtkActionGroup *action_group = GQ_GTK_ACTION_GROUP(groups->data);
 		for (GList *actions = gq_gtk_action_group_list_actions(action_group); actions; actions = actions->next)
 			{
-			GtkAction *action = GTK_ACTION(actions->data);
+			GtkAction *action = GQ_GTK_ACTION(actions->data);
 
 			const gchar *accel_path = gq_gtk_action_get_accel_path(action);
 			if (accel_path && gtk_accel_map_lookup_entry(accel_path, nullptr))
@@ -1396,50 +1378,39 @@ std::vector<ActionItem> get_action_items()
 	return list_unique;
 }
 
-gboolean defined_mouse_buttons(GtkWidget *, GdkEventButton *event, gpointer data)
+bool defined_mouse_buttons(GdkEventButton *event, gpointer data)
 {
-	auto lw = static_cast<LayoutWindow *>(data);
-	GtkAction *action;
-	gboolean ret = FALSE;
+	bool ret = false;
+
+	const auto handle_button = [data](const gchar *action_name)
+	{
+		if (!action_name) return false;
+
+		auto *lw = static_cast<LayoutWindow *>(data);
+
+		if (g_strstr_len(action_name, -1, ".desktop") != nullptr)
+			{
+			file_util_start_editor_from_filelist(action_name, layout_selection_list(lw), layout_get_path(lw), lw->window);
+			}
+		else
+			{
+			GtkAction *action = gq_gtk_action_group_get_action(lw->action_group, action_name);
+			if (action)
+				{
+				gq_gtk_action_activate(action);
+				}
+			}
+
+		return true;
+	};
 
 	switch (event->button)
 		{
 		case MOUSE_BUTTON_8:
-			if (options->mouse_button_8)
-				{
-				if (g_strstr_len(options->mouse_button_8, -1, ".desktop") != nullptr)
-					{
-					file_util_start_editor_from_filelist(options->mouse_button_8, layout_selection_list(lw), layout_get_path(lw), lw->window);
-					ret = TRUE;
-					}
-				else
-					{
-					action = gq_gtk_action_group_get_action(lw->action_group, options->mouse_button_8);
-					if (action)
-						{
-						gq_gtk_action_activate(action);
-						}
-					ret = TRUE;
-					}
-				}
+			ret = handle_button(options->mouse_button_8);
 			break;
 		case MOUSE_BUTTON_9:
-			if (options->mouse_button_9)
-				{
-				if (g_strstr_len(options->mouse_button_9, -1, ".desktop") != nullptr)
-					{
-					file_util_start_editor_from_filelist(options->mouse_button_9, layout_selection_list(lw), layout_get_path(lw), lw->window);
-					}
-				else
-					{
-					action = gq_gtk_action_group_get_action(lw->action_group, options->mouse_button_9);
-					if (action)
-						{
-						gq_gtk_action_activate(action);
-						}
-					ret = TRUE;
-					}
-				}
+			ret = handle_button(options->mouse_button_9);
 			break;
 		default:
 			break;
