@@ -28,13 +28,8 @@
 GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 					 const gchar *filter, const gchar *filter_desc);
 
-void path_selection_sync_to_entry(GtkWidget *entry);
-
 void path_selection_add_select_func(GtkWidget *entry,
 				    void (*func)(const gchar *, gpointer), gpointer data);
-void path_selection_add_filter(GtkWidget *entry, const gchar *filter, const gchar *description, gint set);
-void path_selection_clear_filter(GtkWidget *entry);
-
 
 #endif
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
