@@ -25,7 +25,9 @@
 
 #include <glib-object.h>
 
+#include "compat-deprecated.h"
 #include "compat.h"
+#include "layout.h"
 #include "misc.h"
 #include "ui-misc.h"
 
@@ -38,8 +40,8 @@
 static void tree_edit_close(TreeEditData *ted)
 {
 	gtk_grab_remove(ted->window);
-	gdk_keyboard_ungrab(GDK_CURRENT_TIME);
-	gdk_pointer_ungrab(GDK_CURRENT_TIME);
+	gq_gdk_keyboard_ungrab(GDK_CURRENT_TIME);
+	gq_gdk_pointer_ungrab(GDK_CURRENT_TIME);
 
 	gq_gtk_widget_destroy(ted->window);
 
@@ -147,7 +149,7 @@ static gboolean tree_edit_by_path_idle_cb(gpointer data)
 	if (gtk_tree_view_column_cell_get_position(ted->column, ted->cell, &sx, &sw))
 		{
 		x += sx;
-		w = MAX(w - sx, sw);
+		w = std::max(w - sx, sw);
 		}
 
 	gdk_window_get_origin(gtk_widget_get_window(gtk_widget_get_parent(GTK_WIDGET(ted->tree))), &wx, &wy);
@@ -169,10 +171,10 @@ static gboolean tree_edit_by_path_idle_cb(gpointer data)
 	 */
 	gtk_widget_grab_focus(ted->entry);
 	gtk_grab_add(ted->window);
-	gdk_pointer_grab(gtk_widget_get_window(ted->window), TRUE,
-			 static_cast<GdkEventMask>(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_MOTION_MASK),
-			 nullptr, nullptr, GDK_CURRENT_TIME);
-	gdk_keyboard_grab(gtk_widget_get_window(ted->window), TRUE, GDK_CURRENT_TIME);
+	gq_gdk_pointer_grab(gtk_widget_get_window(ted->window), TRUE,
+	                    static_cast<GdkEventMask>(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_MOTION_MASK),
+	                    nullptr, nullptr, GDK_CURRENT_TIME);
+	gq_gdk_keyboard_grab(gtk_widget_get_window(ted->window), TRUE, GDK_CURRENT_TIME);
 
 	return G_SOURCE_REMOVE;
 }
@@ -229,6 +231,10 @@ gboolean tree_edit_by_path(GtkTreeView *tree, GtkTreePath *tpath, gint column, c
 	/* create the window */
 
 	ted->window = gtk_window_new(GTK_WINDOW_POPUP);
+
+LayoutWindow * lw = get_current_layout();
+	gtk_window_set_transient_for(GTK_WINDOW(ted->window), GTK_WINDOW(lw->window));
+
 	gtk_window_set_resizable(GTK_WINDOW(ted->window), FALSE);
 	g_signal_connect(G_OBJECT(ted->window), "button_press_event",
 			 G_CALLBACK(tree_edit_click_cb), ted);
@@ -413,15 +419,15 @@ void shift_color(GdkRGBA *src, gshort val, gint direction)
 	/* up or down ? */
 	if (direction < 0 || (direction == 0 &&(src->red + src->green + src->blue) / 3 > 1.0 / 2))
 		{
-		src->red = MAX(0 , src->red - cs);
-		src->green = MAX(0 , src->green - cs);
-		src->blue = MAX(0 , src->blue - cs);
+		src->red = std::max(0.0, src->red - cs);
+		src->green = std::max(0.0, src->green - cs);
+		src->blue = std::max(0.0, src->blue - cs);
 		}
 	else
 		{
-		src->red = MIN(1.0, src->red + cs);
-		src->green = MIN(1.0, src->green + cs);
-		src->blue = MIN(1.0, src->blue + cs);
+		src->red = std::min(1.0, src->red + cs);
+		src->green = std::min(1.0, src->green + cs);
+		src->blue = std::min(1.0, src->blue + cs);
 		}
 }
 
@@ -467,7 +473,7 @@ static gboolean widget_auto_scroll_cb(gpointer data)
 
 	if (sd->max_step < sd->region_size)
 		{
-		sd->max_step = MIN(sd->region_size, sd->max_step + 2);
+		sd->max_step = std::min(sd->region_size, sd->max_step + 2);
 		}
 
 	GdkWindow *window = gtk_widget_get_window(sd->widget);
@@ -477,7 +483,7 @@ static gboolean widget_auto_scroll_cb(gpointer data)
 		{
 		sd->timer_id = 0;
 		widget_auto_scroll_stop(sd->widget);
-		return FALSE;
+		return G_SOURCE_REMOVE;
 		}
 
 	gint h = gdk_window_get_height(window);
@@ -510,7 +516,7 @@ static gboolean widget_auto_scroll_cb(gpointer data)
 				{
 				sd->timer_id = 0;
 				widget_auto_scroll_stop(sd->widget);
-				return FALSE;
+				return G_SOURCE_REMOVE;
 				}
 
 			gtk_adjustment_set_value(sd->adj,
@@ -518,7 +524,7 @@ static gboolean widget_auto_scroll_cb(gpointer data)
 			}
 		}
 
-	return TRUE;
+	return G_SOURCE_CONTINUE;
 }
 
 /**
