@@ -21,9 +21,11 @@
 
 #include "ui-utildlg.h"
 
+#include <algorithm>
 #include <cstdio>
-#include <cstring>
 #include <ctime>
+#include <string>
+#include <vector>
 
 #include <gdk/gdk.h>
 #include <gio/gio.h>
@@ -32,7 +34,6 @@
 #include <config.h>
 
 #include "compat.h"
-#include "debug.h"
 #include "filedata.h"
 #include "intl.h"
 #include "main-defines.h"
@@ -48,7 +49,29 @@
 namespace
 {
 
-constexpr gint max_buffer_size = 16384;
+struct DialogWindow
+{
+	GdkRectangle rect;
+	gchar *title;
+	gchar *role;
+};
+
+DialogWindow *dialog_window_find_by_title_and_role(const std::vector<DialogWindow *> &dialog_windows,
+                                                   const gchar *title, const gchar *role)
+{
+	const auto dialog_window_has_title_and_role = [title, role](const DialogWindow *dw)
+	{
+		return g_strcmp0(dw->title, title) == 0 && g_strcmp0(dw->role, role) == 0;
+	};
+
+	auto it = std::find_if(dialog_windows.begin(), dialog_windows.end(),
+	                       dialog_window_has_title_and_role);
+
+	return (it != dialog_windows.end()) ? *it : nullptr;
+}
+
+std::vector<DialogWindow *> dialog_windows;
+
 } // namespace
 
 /*
@@ -57,70 +80,40 @@ constexpr gint max_buffer_size = 16384;
  *-----------------------------------------------------------------------------
  */
 
-struct DialogWindow
-{
-	GdkRectangle rect;
-	gchar *title;
-	gchar *role;
-};
-
-static GList *dialog_windows = nullptr;
-
 static void generic_dialog_save_window(const gchar *title, const gchar *role, GdkRectangle rect)
 {
-	GList *work;
-
-	work = g_list_first(dialog_windows);
-	while (work)
+	DialogWindow *dw = dialog_window_find_by_title_and_role(dialog_windows, title, role);
+	if (dw)
 		{
-		auto dw = static_cast<DialogWindow *>(work->data);
-		if (g_strcmp0(dw->title ,title) == 0 && g_strcmp0(dw->role, role) == 0)
-			{
-			dw->rect = rect;
-			return;
-			}
-		work = work->next;
+		dw->rect = rect;
+		return;
 		}
 
-	auto dw = g_new0(DialogWindow, 1);
+	dw = g_new0(DialogWindow, 1);
 	dw->rect = rect;
 	dw->title = g_strdup(title);
 	dw->role = g_strdup(role);
 
-	dialog_windows = g_list_append(dialog_windows, dw);
+	dialog_windows.push_back(dw);
 }
 
-static gboolean generic_dialog_find_window(const gchar *title, const gchar *role, GdkRectangle &rect)
+gboolean generic_dialog_find_window(const gchar *title, const gchar *role, GdkRectangle &rect)
 {
-	GList *work;
+	DialogWindow *dw = dialog_window_find_by_title_and_role(dialog_windows, title, role);
+	if (!dw) return FALSE;
 
-	work = g_list_first(dialog_windows);
-	while (work)
-		{
-		auto dw = static_cast<DialogWindow *>(work->data);
-
-		if (g_strcmp0(dw->title,title) == 0 && g_strcmp0(dw->role, role) == 0)
-			{
-			rect = dw->rect;
-			return TRUE;
-			}
-		work = work->next;
-		}
-	return FALSE;
+	rect = dw->rect;
+	return TRUE;
 }
 
 void generic_dialog_close(GenericDialog *gd)
 {
-	gchar *ident_string;
-	gchar *full_title;
-	gchar *actual_title;
-
 	/* The window title is modified in window.cc: window_new()
 	 * by appending the string " - Geeqie"
 	 */
-	ident_string = g_strconcat(" - ", GQ_APPNAME, NULL);
-	full_title = g_strdup(gtk_window_get_title(GTK_WINDOW(gd->dialog)));
-	actual_title = strndup(full_title, g_strrstr(full_title, ident_string) - full_title);
+	static const gchar *ident_string = " - " GQ_APPNAME;
+	g_autofree gchar *full_title = g_strdup(gtk_window_get_title(GTK_WINDOW(gd->dialog)));
+	g_autofree gchar *actual_title = strndup(full_title, g_strrstr(full_title, ident_string) - full_title);
 
 	GdkRectangle rect = window_get_root_origin_geometry(gtk_widget_get_window(gd->dialog));
 
@@ -128,9 +121,6 @@ void generic_dialog_close(GenericDialog *gd)
 
 	gq_gtk_widget_destroy(gd->dialog);
 	g_free(gd);
-	g_free(ident_string);
-	g_free(full_title);
-	g_free(actual_title);
 }
 
 static void generic_dialog_click_cb(GtkWidget *widget, gpointer data)
@@ -150,7 +140,7 @@ static gboolean generic_dialog_default_key_press_cb(GtkWidget *widget, GdkEventK
 {
 	auto gd = static_cast<GenericDialog *>(data);
 
-	if (event->keyval == GDK_KEY_Return && gtk_widget_has_focus(widget)
+	if ((event->keyval == (GDK_KEY_Return) || (event->keyval == GDK_KEY_KP_Enter)) && gtk_widget_has_focus(widget)
 	    && gd->default_cb)
 		{
 		gboolean auto_close;
@@ -330,12 +320,12 @@ void generic_dialog_windows_load_config(const gchar **attribute_names, const gch
 		if (READ_INT_FULL("w", dw->rect.width)) continue;
 		if (READ_INT_FULL("h", dw->rect.height)) continue;
 
-		log_printf("unknown attribute %s = %s\n", option, value);
+		config_file_error((std::string("Unknown attribute: ") + option + " = " + value).c_str());
 		}
 
 	if (dw->title && dw->title[0] != 0)
 		{
-		dialog_windows = g_list_append(dialog_windows, dw);
+		dialog_windows.push_back(dw);
 		}
 	else
 		{
@@ -347,30 +337,25 @@ void generic_dialog_windows_load_config(const gchar **attribute_names, const gch
 
 void generic_dialog_windows_write_config(GString *outstr, gint indent)
 {
-	GList *work;
+	if (!options->save_dialog_window_positions || dialog_windows.empty()) return;
 
-	if (options->save_dialog_window_positions && dialog_windows)
+	WRITE_NL(); WRITE_STRING("<dialogs>");
+	indent++;
+
+	for (const DialogWindow *dw : dialog_windows)
 		{
-		WRITE_NL(); WRITE_STRING("<%s>", "dialogs");
-		indent++;
-
-		work = g_list_first(dialog_windows);
-		while (work)
-			{
-			auto dw = static_cast<DialogWindow *>(work->data);
-			WRITE_NL(); WRITE_STRING("<window ");
-			write_char_option(outstr, indent + 1, "title", dw->title);
-			write_char_option(outstr, indent + 1, "role", dw->role);
-			WRITE_INT(dw->rect, x);
-			WRITE_INT(dw->rect, y);
-			write_int_option(outstr, indent, "w", dw->rect.width);
-			write_int_option(outstr, indent, "h", dw->rect.height);
-			WRITE_STRING("/>");
-			work = work->next;
-			}
-		indent--;
-		WRITE_NL(); WRITE_STRING("</%s>", "dialogs");
+		WRITE_NL(); WRITE_STRING("<window ");
+		write_char_option(outstr, "title", dw->title);
+		write_char_option(outstr, "role", dw->role);
+		WRITE_INT(dw->rect, x);
+		WRITE_INT(dw->rect, y);
+		write_int_option(outstr, "w", dw->rect.width);
+		write_int_option(outstr, "h", dw->rect.height);
+		WRITE_STRING("/>");
 		}
+
+	indent--;
+	WRITE_NL(); WRITE_STRING("</dialogs>");
 }
 
 static void generic_dialog_setup(GenericDialog *gd,
@@ -508,95 +493,46 @@ GenericDialog *warning_dialog(const gchar *heading, const gchar *text,
 
 /*
  *-----------------------------------------------------------------------------
- * AppImage version update notification message with fade-out
+ * AppImage version update notification message
  *-----------------------------------------------------------------------------
  *
  * If the current version is not on GitHub, assume a newer one is available
- * and show a fade-out message.
+ * and show a notification message.
  */
 
 struct AppImageData
 {
 	GThreadPool *thread_pool;
-	GtkWidget *window;
-	guint id;
 };
 
-static gboolean appimage_notification_close_cb(gpointer data)
+static void show_new_appimage_notification(GtkApplication *app)
 {
-	auto appimage_data = static_cast<AppImageData *>(data);
+	auto *notification = g_notification_new("Geeqie");
 
-	if (appimage_data->window && gtk_widget_get_opacity(appimage_data->window) != 0)
-		{
-		g_source_remove(appimage_data->id);
-		}
+	g_notification_set_title(notification, _("AppImage"));
+	g_notification_set_body(notification, _("A new Geeqie AppImage is available"));
+	g_notification_set_priority(notification, G_NOTIFICATION_PRIORITY_NORMAL);
+	g_notification_set_default_action(notification, "app.null");
 
-	if (appimage_data->window)
-		{
-		gq_gtk_widget_destroy(appimage_data->window);
-		}
+	g_application_send_notification(G_APPLICATION(app), "new-appimage-notification", notification);
 
-	g_thread_pool_free(appimage_data->thread_pool, TRUE, TRUE);
-	g_free(appimage_data);
-
-	return G_SOURCE_REMOVE;
+	g_object_unref(notification);
 }
 
-static gboolean appimage_notification_fade_cb(gpointer data)
-{
-	auto appimage_data = static_cast<AppImageData *>(data);
-
-	gtk_widget_set_opacity(appimage_data->window, (gtk_widget_get_opacity(appimage_data->window) - 0.02));
-
-	if (gtk_widget_get_opacity(appimage_data->window) == 0)
-		{
-		g_idle_add(appimage_notification_close_cb, data);
-
-		return FALSE;
-		}
-
-	return TRUE;
-}
-
-static gboolean user_close_cb(GtkWidget *, GdkEvent *, gpointer data)
-{
-	auto appimage_data = static_cast<AppImageData *>(data);
-
-	g_idle_add(appimage_notification_close_cb, appimage_data);
-
-	return FALSE;
-}
-
-static void show_notification_message(AppImageData *appimage_data)
-{
-	GtkBuilder *builder;
-
-	builder = gtk_builder_new_from_resource(GQ_RESOURCE_PATH_UI "/appimage-notification.ui");
-
-	appimage_data->window = GTK_WIDGET(gtk_builder_get_object(builder, "appimage_notification"));
-
-	GdkRectangle workarea;
-	gdk_monitor_get_workarea(gdk_display_get_primary_monitor(gdk_display_get_default()),
-                             &workarea);
-	gq_gtk_window_move(GTK_WINDOW(appimage_data->window), workarea.width * 0.8, workarea.height / 20);
-	g_signal_connect(appimage_data->window, "focus-in-event", G_CALLBACK(user_close_cb), appimage_data);
-	appimage_data->id = g_timeout_add(100, appimage_notification_fade_cb, appimage_data);
-
-	g_object_unref(builder);
-	gtk_widget_show(appimage_data->window);
-}
-
-static void appimage_notification_func(gpointer data, gpointer)
+static void new_appimage_notification_func(gpointer, gpointer user_data)
 {
 	FILE *pipe;
 	GNetworkMonitor *net_mon;
 	GSocketConnectable *geeqie_github;
-	auto appimage_data = static_cast<AppImageData *>(data);
+	auto app = static_cast<GtkApplication *>(user_data);
+	constexpr gint max_buffer_size = 16384;
 	char buffer[max_buffer_size];
 	char result[max_buffer_size];
 	gboolean internet_available = FALSE;
 
-	/* If this is a release version, do not check for updates */
+	/* If this is a release version, do not check for updates.
+	 * Non-release version is e.g. 2.5+git20241117-167271b8
+	 */
 	if (g_strrstr(VERSION, "git"))
 		{
 		net_mon = g_network_monitor_get_default();
@@ -626,34 +562,38 @@ static void appimage_notification_func(gpointer data, gpointer)
 
 				/* GitHub date looks like: "published_at": "2024-04-17T08:50:08Z" */
 				gchar *start_date = g_strstr_len(result, -1, "published_at");
-				start_date += 16; // skip 'published_at": "' part
-				start_date[10] = '\0'; // drop everything after YYYY-mm-dd part
 
-				std::tm github_version_date{};
-				strptime(start_date, "%Y-%m-%d", &github_version_date);
-
-				/* VERSION looks like: 2.0.1+git20220116-c791cbee */
-				g_auto(GStrv) version_split = g_strsplit_set(VERSION, "+-", -1);
-
-				std::tm current_version_date{};
-				strptime(version_split[1] + 3, "%Y-%m-%d", &current_version_date);
-
-				if (mktime(&github_version_date) > mktime(&current_version_date))
+				if (start_date)
 					{
-					show_notification_message(appimage_data);
+					start_date += 16; // skip 'published_at": "' part
+					start_date[10] = '\0'; // drop everything after YYYY-mm-dd part
+
+					std::tm github_version_date{};
+					strptime(start_date, "%Y-%m-%d", &github_version_date);
+
+					/* VERSION looks like: 2.0.1+git20220116-c791cbee */
+					g_auto(GStrv) version_split = g_strsplit_set(VERSION, "+-", -1);
+
+					std::tm current_version_date{};
+					strptime(version_split[1] + 3, "%Y%m%d", &current_version_date);
+
+					if (mktime(&github_version_date) > mktime(&current_version_date))
+						{
+						show_new_appimage_notification(app);
+						}
 					}
 				}
 			}
 		}
 }
 
-void appimage_notification()
+void new_appimage_notification(GtkApplication *app)
 {
 	AppImageData *appimage_data;
 
 	appimage_data = g_new0(AppImageData, 1);
 
-	appimage_data->thread_pool = g_thread_pool_new(appimage_notification_func, appimage_data, 1, FALSE, nullptr);
+	appimage_data->thread_pool = g_thread_pool_new(new_appimage_notification_func, app, 1, FALSE, nullptr);
 	g_thread_pool_push(appimage_data->thread_pool, appimage_data, nullptr);
 }
 
@@ -781,9 +721,8 @@ void file_dialog_sync_history(FileDialog *fdlg, gboolean dir_only)
 		}
 	else
 		{
-		gchar *buf = remove_level_from_path(fdlg->dest_path);
+		g_autofree gchar *buf = remove_level_from_path(fdlg->dest_path);
 		tab_completion_append_to_history(fdlg->entry, buf);
-		g_free(buf);
 		}
 }
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
