@@ -29,9 +29,10 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <glib-object.h>
 
-#include "debug.h"
+#include "compat-deprecated.h"
 #include "filedata.h"
 #include "layout.h"
+#include "misc.h"
 #include "options.h"
 #include "typedefs.h"
 #include "ui-fileops.h"
@@ -63,19 +64,12 @@ static void vdtree_row_expanded(GtkTreeView *treeview, GtkTreeIter *iter, GtkTre
  *----------------------------------------------------------------------------
  */
 
-static void set_cursor(GtkWidget *widget, GdkCursorType cursor_type)
+static void set_cursor(GtkWidget *widget, gint cursor_type)
 {
-	GdkCursor *cursor = nullptr;
-	GdkDisplay *display;
+	if (!widget) return;
 
-	if (!widget || !gtk_widget_get_window(widget)) return;
-
-	display = gdk_display_get_default();
-
-	if (cursor_type > -1) cursor = gdk_cursor_new_for_display(display, cursor_type);
-	gdk_window_set_cursor(gtk_widget_get_window(widget), cursor);
-	if (cursor) g_object_unref(G_OBJECT(cursor));
-	gdk_flush();
+	widget_set_cursor(widget, cursor_type);
+	gq_gdk_flush();
 }
 
 static void vdtree_busy_push(ViewDir *vd)
@@ -86,7 +80,7 @@ static void vdtree_busy_push(ViewDir *vd)
 
 static void vdtree_busy_pop(ViewDir *vd)
 {
-	if (VDTREE(vd)->busy_ref == 1) set_cursor(vd->view, GDK_CURSOR_IS_PIXMAP);
+	if (VDTREE(vd)->busy_ref == 1) set_cursor(vd->view, -1);
 	if (VDTREE(vd)->busy_ref > 0) VDTREE(vd)->busy_ref--;
 }
 
@@ -211,7 +205,7 @@ static gboolean vdtree_dnd_drop_expand_cb(gpointer data)
 		}
 
 	VDTREE(vd)->drop_expand_id = 0;
-	return FALSE;
+	return G_SOURCE_REMOVE;
 }
 
 static void vdtree_dnd_drop_expand_cancel(ViewDir *vd)
@@ -393,7 +387,6 @@ static void vdtree_add_by_data(ViewDir *vd, FileData *fd, GtkTreeIter *parent)
 	GtkTreeIter child;
 	GdkPixbuf *pixbuf;
 	GtkTreeIter empty;
-	gchar *link = nullptr;
 
 	if (!fd) return;
 
@@ -423,14 +416,11 @@ static void vdtree_add_by_data(ViewDir *vd, FileData *fd, GtkTreeIter *parent)
 	nd->expanded = FALSE;
 	nd->last_update = time(nullptr);
 
+	g_autofree gchar *link = nullptr;
 	if (islink(fd->path))
 		{
 		link = realpath(fd->path, nullptr);
 		}
-	else
-		{
-		link = nullptr;
-		}
 
 	store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vd->view)));
 	gtk_tree_store_append(store, &child, parent);
@@ -465,8 +455,6 @@ static void vdtree_add_by_data(ViewDir *vd, FileData *fd, GtkTreeIter *parent)
 			}
 		gtk_tree_path_free(tpath);
 		}
-
-	g_free(link);
 }
 
 gboolean vdtree_populate_path_by_iter(ViewDir *vd, GtkTreeIter *iter, gboolean force, FileData *target_fd)
@@ -479,7 +467,6 @@ gboolean vdtree_populate_path_by_iter(ViewDir *vd, GtkTreeIter *iter, gboolean f
 	GtkTreeIter child;
 	NodeData *nd;
 	gboolean add_hidden = FALSE;
-	gchar *link = nullptr;
 
 	store = gtk_tree_view_get_model(GTK_TREE_VIEW(vd->view));
 	gtk_tree_model_get(store, iter, DIR_COLUMN_POINTER, &nd, -1);
@@ -530,19 +517,17 @@ gboolean vdtree_populate_path_by_iter(ViewDir *vd, GtkTreeIter *iter, gboolean f
 	if (add_hidden)
 		{
 		gint n;
-		gchar *name8;
 
 		n = strlen(nd->fd->path) + 1;
 
 		while (target_fd->path[n] != '\0' && target_fd->path[n] != G_DIR_SEPARATOR) n++;
-		name8 = g_strndup(target_fd->path, n);
+
+		g_autofree gchar *name8 = g_strndup(target_fd->path, n);
 
 		if (isdir(name8))
 			{
 			list = g_list_prepend(list, file_data_new_dir(name8));
 			}
-
-		g_free(name8);
 		}
 
 	old = nullptr;
@@ -582,14 +567,11 @@ gboolean vdtree_populate_path_by_iter(ViewDir *vd, GtkTreeIter *iter, gboolean f
 
 				gtk_tree_store_set(GTK_TREE_STORE(store), &child, DIR_COLUMN_NAME, fd->name, -1);
 
+				g_autofree gchar *link = nullptr;
 				if (islink(fd->path))
 					{
 					link = realpath(fd->path, nullptr);
 					}
-				else
-					{
-					link = nullptr;
-					}
 
 				gtk_tree_store_set(GTK_TREE_STORE(store), &child, DIR_COLUMN_LINK, link, -1);
 
@@ -628,8 +610,6 @@ gboolean vdtree_populate_path_by_iter(ViewDir *vd, GtkTreeIter *iter, gboolean f
 	nd->expanded = TRUE;
 	nd->last_update = current_time;
 
-	g_free(link);
-
 	return TRUE;
 }
 
@@ -785,13 +765,6 @@ void vdtree_refresh(ViewDir *vd)
 	vdtree_populate_path(vd, vd->dir_fd, FALSE, TRUE);
 }
 
-const gchar *vdtree_row_get_path(ViewDir *, gint)
-{
-/** @FIXME no get row path */
-	log_printf("FIXME: no get row path\n");
-	return nullptr;
-}
-
 /*
  *----------------------------------------------------------------------------
  * callbacks
@@ -848,6 +821,8 @@ gboolean vdtree_press_key_cb(GtkWidget *widget, GdkEventKey *event, gpointer dat
 					}
 				}
 			break;
+		default:
+			break;
 		}
 
 	return FALSE;
