@@ -31,8 +31,8 @@
 
 #include <config.h>
 
+#include "compat-deprecated.h"
 #include "compat.h"
-#include "debug.h"
 #include "dnd.h"
 #include "dupe.h"
 #include "editors.h"
@@ -328,20 +328,15 @@ static gboolean vd_rename_cb(TreeEditData *td, const gchar *, const gchar *new_n
 {
 	auto vd = static_cast<ViewDir *>(data);
 	FileData *fd;
-	gchar *new_path;
-	gchar *base;
 
 	fd = vd_get_fd_from_tree_path(vd, GTK_TREE_VIEW(vd->view), td->path);
 	if (!fd) return FALSE;
 
-	base = remove_level_from_path(fd->path);
-	new_path = g_build_filename(base, new_name, NULL);
-	g_free(base);
+	g_autofree gchar *base = remove_level_from_path(fd->path);
+	g_autofree gchar *new_path = g_build_filename(base, new_name, NULL);
 
 	file_util_rename_dir(fd, new_path, vd->view, vd_rename_finished_cb, vd);
 
-	g_free(new_path);
-
 	return FALSE;
 }
 
@@ -453,8 +448,6 @@ static void vd_drop_menu_filter_cb(GtkWidget *widget, gpointer data)
 GtkWidget *vd_drop_menu(ViewDir *vd, gint active)
 {
 	GtkWidget *menu;
-	GList *editors_list = editor_list_get();
-	GList *work = editors_list;
 
 	menu = popup_menu_short_lived();
 	g_signal_connect(G_OBJECT(menu), "destroy",
@@ -464,19 +457,16 @@ GtkWidget *vd_drop_menu(ViewDir *vd, gint active)
 				      G_CALLBACK(vd_drop_menu_copy_cb), vd);
 	menu_item_add_sensitive(menu, _("_Move"), active, G_CALLBACK(vd_drop_menu_move_cb), vd);
 
-	while (work)
+	EditorsList editors_list = editor_list_get();
+	for (const EditorDescription *editor : editors_list)
 		{
-		GtkWidget *item;
-		auto editor = static_cast<const EditorDescription *>(work->data);
-		work = work->next;
-
 		if (!editor_is_filter(editor->key)) continue;
-		item = menu_item_add_sensitive(menu, editor->name, active, G_CALLBACK(vd_drop_menu_filter_cb), vd);
+
+		GtkWidget *item = menu_item_add_sensitive(menu, editor->name, active,
+		                                          G_CALLBACK(vd_drop_menu_filter_cb), vd);
 		g_object_set_data_full(G_OBJECT(item), "filter_key", g_strdup(editor->key), g_free);
 		}
 
-	g_list_free(editors_list);
-
 	menu_item_add_divider(menu);
 	menu_item_add_icon(menu, _("Cancel"), GQ_ICON_CANCEL, nullptr, vd);
 
@@ -492,19 +482,16 @@ GtkWidget *vd_drop_menu(ViewDir *vd, gint active)
 static void vd_pop_menu_up_cb(GtkWidget *, gpointer data)
 {
 	auto vd = static_cast<ViewDir *>(data);
-	gchar *path;
 
 	if (!vd->dir_fd || strcmp(vd->dir_fd->path, G_DIR_SEPARATOR_S) == 0) return;
-	path = remove_level_from_path(vd->dir_fd->path);
 
 	if (vd->select_func)
 		{
+		g_autofree gchar *path = remove_level_from_path(vd->dir_fd->path);
 		FileData *fd = file_data_new_dir(path);
 		vd->select_func(vd, fd, vd->select_data);
 		file_data_unref(fd);
 		}
-
-	g_free(path);
 }
 
 static void vd_pop_menu_slide_cb(GtkWidget *, gpointer data)
@@ -1144,7 +1131,7 @@ static GdkRGBA *vd_color_shifted(GtkWidget *widget)
 		GtkStyleContext *style_context;
 
 		style_context = gtk_widget_get_style_context(widget);
-		gtk_style_context_get_background_color(style_context, GTK_STATE_FLAG_NORMAL, &color);
+		gq_gtk_style_context_get_background_color(style_context, GTK_STATE_FLAG_NORMAL, &color);
 
 		shift_color(&color, -1, 0);
 		done = widget;
@@ -1171,7 +1158,7 @@ gboolean vd_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 	GtkTreePath *tpath;
 	FileData *fd = nullptr;
 
-	if (defined_mouse_buttons(widget, bevent, vd->layout))
+	if (defined_mouse_buttons(bevent, vd->layout))
 		{
 		return TRUE;
 		}
@@ -1267,13 +1254,12 @@ static void vd_notify_cb(FileData *fd, NotifyType type, gpointer data)
 {
 	auto vd = static_cast<ViewDir *>(data);
 	gboolean refresh;
-	gchar *base;
 
 	if (!S_ISDIR(fd->mode)) return; /* this gives correct results even on recently deleted files/directories */
 
 	DEBUG_1("Notify vd: %s %04x", fd->path, type);
 
-	base = remove_level_from_path(fd->path);
+	g_autofree gchar *base = remove_level_from_path(fd->path);
 
 	if (vd->type == DIRVIEW_LIST)
 		{
@@ -1288,16 +1274,14 @@ static void vd_notify_cb(FileData *fd, NotifyType type, gpointer data)
 			{
 			if (!refresh && fd->change->dest)
 				{
-				gchar *dest_base = remove_level_from_path(fd->change->dest);
+				g_autofree gchar *dest_base = remove_level_from_path(fd->change->dest);
 				refresh = (strcmp(dest_base, vd->dir_fd->path) == 0);
-				g_free(dest_base);
 				}
 
 			if (!refresh && fd->change->source)
 				{
-				gchar *source_base = remove_level_from_path(fd->change->source);
+				g_autofree gchar *source_base = remove_level_from_path(fd->change->source);
 				refresh = (strcmp(source_base, vd->dir_fd->path) == 0);
-				g_free(source_base);
 				}
 			}
 
@@ -1316,7 +1300,5 @@ static void vd_notify_cb(FileData *fd, NotifyType type, gpointer data)
 
 		file_data_unref(base_fd);
 		}
-
-	g_free(base);
 }
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
