@@ -28,7 +28,7 @@
 #include <glib-object.h>
 
 #include "collect.h"
-#include "debug.h"
+#include "compat-deprecated.h"
 #include "dnd.h"
 #include "filedata.h"
 #include "intl.h"
@@ -48,20 +48,20 @@
 /* Index to tree store */
 enum {
 	FILE_COLUMN_POINTER = 0,
-	FILE_COLUMN_VERSION,
-	FILE_COLUMN_THUMB,
-	FILE_COLUMN_FORMATTED,
-	FILE_COLUMN_FORMATTED_WITH_STARS,
-	FILE_COLUMN_NAME,
-	FILE_COLUMN_SIDECARS,
-	FILE_COLUMN_STAR_RATING,
-	FILE_COLUMN_SIZE,
-	FILE_COLUMN_DATE,
-	FILE_COLUMN_EXPANDED,
-	FILE_COLUMN_COLOR,
-	FILE_COLUMN_MARKS,
+	FILE_COLUMN_VERSION = 1,
+	FILE_COLUMN_THUMB = 2,
+	FILE_COLUMN_FORMATTED = 3,
+	FILE_COLUMN_FORMATTED_WITH_STARS = 4,
+	FILE_COLUMN_NAME = 5,
+	FILE_COLUMN_SIDECARS = 6,
+	FILE_COLUMN_STAR_RATING = 7,
+	FILE_COLUMN_SIZE = 8,
+	FILE_COLUMN_DATE = 9,
+	FILE_COLUMN_EXPANDED = 10,
+	FILE_COLUMN_COLOR = 11,
+	FILE_COLUMN_MARKS = 12,
 	FILE_COLUMN_MARKS_LAST = FILE_COLUMN_MARKS + FILEDATA_MARKS_SIZE - 1,
-	FILE_COLUMN_COUNT
+	FILE_COLUMN_COUNT = 22
 };
 
 
@@ -69,13 +69,13 @@ enum {
 enum {
 	FILE_VIEW_COLUMN_MARKS = 0,
 	FILE_VIEW_COLUMN_MARKS_LAST = FILE_VIEW_COLUMN_MARKS + FILEDATA_MARKS_SIZE - 1,
-	FILE_VIEW_COLUMN_THUMB,
-	FILE_VIEW_COLUMN_FORMATTED,
-	FILE_VIEW_COLUMN_FORMATTED_WITH_STARS,
-	FILE_VIEW_COLUMN_STAR_RATING,
-	FILE_VIEW_COLUMN_SIZE,
-	FILE_VIEW_COLUMN_DATE,
-	FILE_VIEW_COLUMN_COUNT
+	FILE_VIEW_COLUMN_THUMB = 10,
+	FILE_VIEW_COLUMN_FORMATTED = 11,
+	FILE_VIEW_COLUMN_FORMATTED_WITH_STARS = 12,
+	FILE_VIEW_COLUMN_STAR_RATING = 13,
+	FILE_VIEW_COLUMN_SIZE = 14,
+	FILE_VIEW_COLUMN_DATE = 15,
+	FILE_VIEW_COLUMN_COUNT = 16
 };
 
 
@@ -537,7 +537,7 @@ gboolean vflist_release_cb(ViewFile *vf, GtkWidget *widget, GdkEventButton *beve
 	GtkTreeIter iter;
 	FileData *fd = nullptr;
 
-	if (defined_mouse_buttons(widget, bevent, vf->layout))
+	if (defined_mouse_buttons(bevent, vf->layout))
 		{
 		return TRUE;
 		}
@@ -633,18 +633,15 @@ static gboolean vflist_select_idle_cb(gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 
-	if (!vf->layout)
+	if (vf->layout)
 		{
-		VFLIST(vf)->select_idle_id = 0;
-		return G_SOURCE_REMOVE;
-		}
+		vf_send_update(vf);
 
-	vf_send_update(vf);
-
-	if (VFLIST(vf)->select_fd)
-		{
-		vflist_select_image(vf, VFLIST(vf)->select_fd);
-		VFLIST(vf)->select_fd = nullptr;
+		if (VFLIST(vf)->select_fd)
+			{
+			vflist_select_image(vf, VFLIST(vf)->select_fd);
+			VFLIST(vf)->select_fd = nullptr;
+			}
 		}
 
 	VFLIST(vf)->select_idle_id = 0;
@@ -729,13 +726,11 @@ static gchar* vflist_get_formatted(ViewFile *vf, const gchar *name, const gchar
 static void vflist_set_expanded(ViewFile *vf, GtkTreeIter *iter, gboolean expanded)
 {
 	GtkTreeStore *store;
-	gchar *name;
-	gchar *sidecars;
-	gchar *size;
-	gchar *time;
-	gchar *formatted;
-	gchar *formatted_with_stars;
-	gchar *star_rating;
+	g_autofree gchar *name = nullptr;
+	g_autofree gchar *sidecars = nullptr;
+	g_autofree gchar *size = nullptr;
+	g_autofree gchar *time = nullptr;
+	g_autofree gchar *star_rating = nullptr;
 	store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vf->listview)));
 
 	gtk_tree_model_get(GTK_TREE_MODEL(store), iter,
@@ -746,8 +741,8 @@ static void vflist_set_expanded(ViewFile *vf, GtkTreeIter *iter, gboolean expand
 					FILE_COLUMN_STAR_RATING, &star_rating,
 					-1);
 
-	formatted = vflist_get_formatted(vf, name, sidecars, size, time, expanded, nullptr);
-	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, star_rating);
+	g_autofree gchar *formatted = vflist_get_formatted(vf, name, sidecars, size, time, expanded, nullptr);
+	g_autofree gchar *formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, star_rating);
 
 	gtk_tree_store_set(store, iter, FILE_COLUMN_FORMATTED, formatted,
 					FILE_COLUMN_EXPANDED, expanded,
@@ -755,49 +750,34 @@ static void vflist_set_expanded(ViewFile *vf, GtkTreeIter *iter, gboolean expand
 	gtk_tree_store_set(store, iter, FILE_COLUMN_FORMATTED_WITH_STARS, formatted_with_stars,
 					FILE_COLUMN_EXPANDED, expanded,
 					-1);
-	g_free(time);
-	g_free(size);
-	g_free(sidecars);
-	g_free(name);
-	g_free(formatted);
-	g_free(formatted_with_stars);
 }
 
 static void vflist_setup_iter(ViewFile *vf, GtkTreeStore *store, GtkTreeIter *iter, FileData *fd)
 {
-	gchar *size;
-	gchar *sidecars = nullptr;
-	gchar *name;
 	const gchar *time = text_from_time(fd->date);
 	const gchar *link = islink(fd->path) ? GQ_LINK_STR : "";
 	const gchar *disabled_grouping;
-	gchar *formatted;
-	gchar *formatted_with_stars;
 	gboolean expanded = FALSE;
-	gchar *star_rating;
+	g_autofree gchar *star_rating = nullptr;
 
 	if (options->show_star_rating && fd->rating != STAR_RATING_NOT_READ)
 		{
  		star_rating = convert_rating_to_stars(fd->rating);
 		}
-	else
-		{
-		star_rating = nullptr;
-		}
 
 	if (fd->sidecar_files) /* expanded has no effect on files without sidecars */
 		{
 		gtk_tree_model_get(GTK_TREE_MODEL(store), iter, FILE_COLUMN_EXPANDED, &expanded, -1);
 		}
 
-	sidecars = file_data_sc_list_to_string(fd);
+	g_autofree gchar *sidecars = file_data_sc_list_to_string(fd);
 
 	disabled_grouping = fd->disable_grouping ? _(" [NO GROUPING]") : "";
-	name = g_strdup_printf("%s%s%s", link, fd->name, disabled_grouping);
-	size = text_from_size(fd->size);
+	g_autofree gchar *name = g_strdup_printf("%s%s%s", link, fd->name, disabled_grouping);
+	g_autofree gchar *size = text_from_size(fd->size);
 
-	formatted = vflist_get_formatted(vf, name, sidecars, size, time, expanded, nullptr);
-	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, star_rating);
+	g_autofree gchar *formatted = vflist_get_formatted(vf, name, sidecars, size, time, expanded, nullptr);
+	g_autofree gchar *formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, star_rating);
 
 	gtk_tree_store_set(store, iter, FILE_COLUMN_POINTER, fd,
 					FILE_COLUMN_VERSION, fd->version,
@@ -835,10 +815,6 @@ static void vflist_setup_iter(ViewFile *vf, GtkTreeStore *store, GtkTreeIter *it
 		gtk_tree_store_set(store, iter, FILE_COLUMN_MARKS + i, file_data_get_mark(fd, i), -1);
 	}
 #endif
-	g_free(size);
-	g_free(sidecars);
-	g_free(name);
-	g_free(formatted);
 }
 
 static void vflist_setup_iter_recursive(ViewFile *vf, GtkTreeStore *store, GtkTreeIter *parent_iter, GList *list, GList *selected, gboolean force)
@@ -1127,21 +1103,19 @@ void vflist_set_star_fd(ViewFile *vf, FileData *fd)
 {
 	GtkTreeStore *store;
 	GtkTreeIter iter;
-	gchar *name;
-	gchar *sidecars;
-	gchar *size;
-	gchar *time;
-	gchar *star_rating;
-	gchar *formatted_with_stars;
 	gboolean expanded;
 
 	if (!fd || vflist_find_row(vf, fd, &iter) < 0) return;
 
-	star_rating = metadata_read_rating_stars(fd);
+	g_autofree gchar *star_rating = metadata_read_rating_stars(fd);
 
 	store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vf->listview)));
 	gtk_tree_store_set(store, &iter, FILE_COLUMN_STAR_RATING, star_rating, -1);
 
+	g_autofree gchar *name = nullptr;
+	g_autofree gchar *sidecars = nullptr;
+	g_autofree gchar *size = nullptr;
+	g_autofree gchar *time = nullptr;
 	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
 					FILE_COLUMN_NAME, &name,
 					FILE_COLUMN_SIDECARS, &sidecars,
@@ -1150,21 +1124,16 @@ void vflist_set_star_fd(ViewFile *vf, FileData *fd)
 					FILE_COLUMN_EXPANDED, &expanded,
 					-1);
 
-	formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, star_rating);
+	g_autofree gchar *formatted_with_stars = vflist_get_formatted(vf, name, sidecars, size, time, expanded, star_rating);
 
 	gtk_tree_store_set(store, &iter, FILE_COLUMN_FORMATTED_WITH_STARS, formatted_with_stars,
 					FILE_COLUMN_EXPANDED, expanded,
 					-1);
-
-	g_free(star_rating);
-	g_free(formatted_with_stars);
 }
 
 FileData *vflist_star_next_fd(ViewFile *vf)
 {
 	GtkTreePath *tpath;
-	FileData *fd = nullptr;
-	FileData *nfd = nullptr;
 
 	/* first check the visible files */
 
@@ -1179,63 +1148,33 @@ FileData *vflist_star_next_fd(ViewFile *vf)
 		gtk_tree_path_free(tpath);
 		tpath = nullptr;
 
-		while (!fd && valid && tree_view_row_get_visibility(GTK_TREE_VIEW(vf->listview), &iter, FALSE) == 0)
+		while (valid && tree_view_row_get_visibility(GTK_TREE_VIEW(vf->listview), &iter, FALSE) == 0)
 			{
-			gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &nfd, -1);
+			FileData *fd = nullptr;
+			gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &fd, -1);
 
-			if (nfd && nfd->rating == STAR_RATING_NOT_READ)
+			if (fd && fd->rating == STAR_RATING_NOT_READ)
 				{
-				fd = nfd;
+				return fd;
 				}
 
 			valid = gtk_tree_model_iter_next(store, &iter);
 			}
-
-		if (fd)
-			{
-			vf->stars_filedata = fd;
-
-			if (vf->stars_id == 0)
-				{
-				vf->stars_id = g_idle_add_full(G_PRIORITY_LOW, vf_stars_cb, vf, nullptr);
-				}
-			}
 		}
 
 	/* then find first undone */
 
-	if (!fd)
+	for (GList *work = vf->list; work; work = work->next)
 		{
-		GList *work = vf->list;
+		auto *fd = static_cast<FileData *>(work->data);
 
-		while (work && !fd)
+		if (fd && fd->rating == STAR_RATING_NOT_READ)
 			{
-			auto fd_p = static_cast<FileData *>(work->data);
-
-			if (fd_p && fd_p->rating == STAR_RATING_NOT_READ)
-				{
-				fd = fd_p;
-				}
-			else
-				{
-				fd = nullptr;
-				}
-
-			work = work->next;
-			}
-
-		if (fd)
-			{
-			vf->stars_filedata = fd;
-
-			if (vf->stars_id == 0)
-				{
-				vf->stars_id = g_idle_add_full(G_PRIORITY_LOW, vf_stars_cb, vf, nullptr);
-				}
+			return fd;
 			}
 		}
 
-	return fd;
+	return nullptr;
 }
 
 /*
@@ -1305,40 +1244,30 @@ gboolean vflist_is_selected(ViewFile *vf, FileData *fd)
 
 guint vflist_selection_count(ViewFile *vf, gint64 *bytes)
 {
-	GtkTreeModel *store;
-	GtkTreeSelection *selection;
-	GList *slist;
-	guint count;
-
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(vf->listview));
-	slist = gtk_tree_selection_get_selected_rows(selection, &store);
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(vf->listview));
 
 	if (bytes)
 		{
 		gint64 b = 0;
-		GList *work;
 
-		work = slist;
-		while (work)
+		GtkTreeModel *store;
+		GList *slist = gtk_tree_selection_get_selected_rows(selection, &store);
+		for (GList *work = slist; work; work = work->next)
 			{
 			auto tpath = static_cast<GtkTreePath *>(work->data);
 			GtkTreeIter iter;
-			FileData *fd;
-
 			gtk_tree_model_get_iter(store, &iter, tpath);
+
+			FileData *fd;
 			gtk_tree_model_get(store, &iter, FILE_COLUMN_POINTER, &fd, -1);
 			b += fd->size;
-
-			work = work->next;
 			}
+		g_list_free_full(slist, reinterpret_cast<GDestroyNotify>(gtk_tree_path_free));
 
 		*bytes = b;
 		}
 
-	count = g_list_length(slist);
-	g_list_free_full(slist, reinterpret_cast<GDestroyNotify>(gtk_tree_path_free));
-
-	return count;
+	return gtk_tree_selection_count_selected_rows(selection);
 }
 
 GList *vflist_selection_get_list(ViewFile *vf)
@@ -1408,7 +1337,8 @@ void vflist_selection_foreach(ViewFile *vf, const ViewFile::SelectionCallback &f
 	GtkTreeIter iter;
 	FileData *fd_n;
 
-	for (GList *work = gtk_tree_selection_get_selected_rows(selection, &store); work; work = work->next)
+	GList *slist = gtk_tree_selection_get_selected_rows(selection, &store);
+	for (GList *work = slist; work; work = work->next)
 		{
 		auto *tpath = static_cast<GtkTreePath *>(work->data);
 
@@ -1417,6 +1347,7 @@ void vflist_selection_foreach(ViewFile *vf, const ViewFile::SelectionCallback &f
 
 		func(fd_n);
 		}
+	g_list_free_full(slist, reinterpret_cast<GDestroyNotify>(gtk_tree_path_free));
 }
 
 void vflist_select_all(ViewFile *vf)
@@ -1775,8 +1706,10 @@ gboolean vflist_refresh(ViewFile *vf)
 			}
 
 		vf->list = file_data_filter_marks_list(vf->list, vf_marks_get_filter(vf));
+
+		g_autoptr(GRegex) filter = vf_file_filter_get_filter(vf);
 		vf->list = g_list_first(vf->list);
-		vf->list = file_data_filter_file_filter_list(vf->list, vf_file_filter_get_filter(vf));
+		vf->list = file_data_filter_file_filter_list(vf->list, filter);
 
 		vf->list = g_list_first(vf->list);
 		vf->list = file_data_filter_class_list(vf->list, vf_class_get_filter(vf));
@@ -1809,7 +1742,7 @@ static GdkRGBA *vflist_listview_color_shifted(GtkWidget *widget)
 		{
 		GtkStyle *style;
 
-		style = gtk_widget_get_style(widget);
+		style = gq_gtk_widget_get_style(widget);
 		convert_gdkcolor_to_gdkrgba(&style->base[GTK_STATE_NORMAL], &color);
 
 		shift_color(&color, -1, 0);
