@@ -27,7 +27,6 @@
 
 #include "archives.h"
 #include "compat.h"
-#include "debug.h"
 #include "dnd.h"
 #include "dupe.h"
 #include "filedata.h"
@@ -397,13 +396,12 @@ static void vf_drag_data_received(GtkWidget *, GdkDragContext *,
 	if (!fd) return;
 
 	/* Add keywords to file */
-	auto str = reinterpret_cast<gchar *>(gtk_selection_data_get_text(selection));
+	g_autofree auto str = reinterpret_cast<gchar *>(gtk_selection_data_get_text(selection));
 	GList *kw_list = string_to_keywords_list(str);
 
 	metadata_append_list(fd, KEYWORD_KEY, kw_list);
 
 	g_list_free_full(kw_list, g_free);
-	g_free(str);
 }
 
 static void vf_dnd_init(ViewFile *vf)
@@ -493,14 +491,12 @@ static void vf_pop_menu_open_archive_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
 	LayoutWindow *lw_new;
-	gchar *dest_dir;
 
-	dest_dir = open_archive(vf->click_fd);
+	g_autofree gchar *dest_dir = open_archive(vf->click_fd);
 	if (dest_dir)
 		{
 		lw_new = layout_new_from_default();
 		layout_set_path(lw_new, dest_dir);
-		g_free(dest_dir);
 		}
 	else
 		{
@@ -790,13 +786,13 @@ GtkWidget *vf_pop_menu(ViewFile *vf)
 	if (vf->clicked_mark > 0)
 		{
 		gint mark = vf->clicked_mark;
-		gchar *str_set_mark = g_strdup_printf(_("_Set mark %d"), mark);
-		gchar *str_res_mark = g_strdup_printf(_("_Reset mark %d"), mark);
-		gchar *str_toggle_mark = g_strdup_printf(_("_Toggle mark %d"), mark);
-		gchar *str_sel_mark = g_strdup_printf(_("_Select mark %d"), mark);
-		gchar *str_sel_mark_or = g_strdup_printf(_("_Add mark %d"), mark);
-		gchar *str_sel_mark_and = g_strdup_printf(_("_Intersection with mark %d"), mark);
-		gchar *str_sel_mark_minus = g_strdup_printf(_("_Unselect mark %d"), mark);
+		g_autofree gchar *str_set_mark = g_strdup_printf(_("_Set mark %d"), mark);
+		g_autofree gchar *str_res_mark = g_strdup_printf(_("_Reset mark %d"), mark);
+		g_autofree gchar *str_toggle_mark = g_strdup_printf(_("_Toggle mark %d"), mark);
+		g_autofree gchar *str_sel_mark = g_strdup_printf(_("_Select mark %d"), mark);
+		g_autofree gchar *str_sel_mark_or = g_strdup_printf(_("_Add mark %d"), mark);
+		g_autofree gchar *str_sel_mark_and = g_strdup_printf(_("_Intersection with mark %d"), mark);
+		g_autofree gchar *str_sel_mark_minus = g_strdup_printf(_("_Unselect mark %d"), mark);
 
 		g_assert(mark >= 1 && mark <= FILEDATA_MARKS_SIZE);
 
@@ -824,14 +820,6 @@ GtkWidget *vf_pop_menu(ViewFile *vf)
 					G_CALLBACK(vf_pop_menu_sel_mark_minus_cb), vf);
 
 		menu_item_add_divider(menu);
-
-		g_free(str_set_mark);
-		g_free(str_res_mark);
-		g_free(str_toggle_mark);
-		g_free(str_sel_mark);
-		g_free(str_sel_mark_and);
-		g_free(str_sel_mark_or);
-		g_free(str_sel_mark_minus);
 		}
 
 	vf->editmenu_fd_list = vf_pop_menu_file_list(vf);
@@ -1069,18 +1057,15 @@ static gboolean vf_marks_tooltip_cb(GtkWidget *widget,
 static void vf_file_filter_save_cb(GtkWidget *, gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
-	gchar *entry_text;
-	gchar *remove_text = nullptr;
 
-	entry_text = g_strdup(gq_gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(vf->file_filter.combo)))));
+	g_autofree gchar *entry_text = g_strdup(gq_gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(vf->file_filter.combo)))));
 
 	if (entry_text[0] == '\0' && vf->file_filter.last_selected >= 0)
 		{
 		gtk_combo_box_set_active(GTK_COMBO_BOX(vf->file_filter.combo), vf->file_filter.last_selected);
-		remove_text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(vf->file_filter.combo));
+		g_autofree gchar *remove_text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(vf->file_filter.combo));
 		history_list_item_remove("file_filter", remove_text);
 		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(vf->file_filter.combo), vf->file_filter.last_selected);
-		g_free(remove_text);
 
 		gtk_combo_box_set_active(GTK_COMBO_BOX(vf->file_filter.combo), -1);
 		vf->file_filter.last_selected = - 1;
@@ -1115,8 +1100,6 @@ static void vf_file_filter_save_cb(GtkWidget *, gpointer data)
 			}
 		}
 	vf_refresh(vf);
-
-	g_free(entry_text);
 }
 
 static void vf_file_filter_cb(GtkWidget *, gpointer data)
@@ -1196,71 +1179,53 @@ static gboolean vf_file_filter_class_cb(GtkWidget *widget, gpointer data)
 	return TRUE;
 }
 
-static gboolean vf_file_filter_class_set_all_cb(GtkWidget *widget, gpointer data)
+static gboolean vf_file_filter_class_set_all(GtkWidget *widget, gpointer data, gboolean state)
 {
-	auto vf = static_cast<ViewFile *>(data);
-	GtkWidget *parent;
-	GList *children;
-	gint i;
-	gboolean state;
+	GtkWidget *parent = gtk_widget_get_parent(widget);
+	g_autoptr(GList) children = gtk_container_get_children(GTK_CONTAINER(parent));
 
-	if (g_strcmp0(_("Select all"), gtk_menu_item_get_label(GTK_MENU_ITEM(widget))) == 0)
+	GList *work = children;
+	for (gboolean &class_filter : options->class_filter)
 		{
-		state = TRUE;
-		}
-	else
-		{
-		state = FALSE;
-		}
+		class_filter = state;
 
-	for (i = 0; i < FILE_FORMAT_CLASSES; i++)
-		{
-		options->class_filter[i] = state;
-		}
-
-	i = 0;
-	parent = gtk_widget_get_parent(widget);
-	children = gtk_container_get_children(GTK_CONTAINER(parent));
-	for (GList *work = children; work; work = work->next)
-		{
-		if (i < FILE_FORMAT_CLASSES)
+		if (work)
 			{
 			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(work->data), state);
+			work = work->next;
 			}
-		i++;
 		}
-	g_list_free(children);
-	vf_refresh(vf);
+
+	vf_refresh(static_cast<ViewFile *>(data));
 
 	return TRUE;
 }
 
+static gboolean vf_file_filter_class_select_all_cb(GtkWidget *widget, gpointer data)
+{
+	return vf_file_filter_class_set_all(widget, data, TRUE);
+}
+
+static gboolean vf_file_filter_class_select_none_cb(GtkWidget *widget, gpointer data)
+{
+	return vf_file_filter_class_set_all(widget, data, FALSE);
+}
+
 static GtkWidget *class_filter_menu (ViewFile *vf)
 {
-	GtkWidget *menu;
-	GtkWidget *menu_item;
-	int i;
+	GtkWidget *menu = gtk_menu_new();
 
-	menu = gtk_menu_new();
-
-	for (i = 0; i < FILE_FORMAT_CLASSES; i++)
-	    {
-		menu_item = gtk_check_menu_item_new_with_label(format_class_list[i]);
+	for (int i = 0; i < FILE_FORMAT_CLASSES; i++)
+		{
+		GtkWidget *menu_item = gtk_check_menu_item_new_with_label(format_class_list[i]);
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), options->class_filter[i]);
 		g_signal_connect(G_OBJECT(menu_item), "toggled", G_CALLBACK(vf_file_filter_class_cb), vf);
 		gtk_menu_shell_append(GTK_MENU_SHELL (menu), menu_item);
 		gtk_widget_show(menu_item);
 		}
 
-	menu_item = gtk_menu_item_new_with_label(_("Select all"));
-	gtk_menu_shell_append(GTK_MENU_SHELL (menu), menu_item);
-	gtk_widget_show(menu_item);
-	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(vf_file_filter_class_set_all_cb), vf);
-
-	menu_item = gtk_menu_item_new_with_label(_("Select none"));
-	gtk_menu_shell_append(GTK_MENU_SHELL (menu), menu_item);
-	gtk_widget_show(menu_item);
-	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(vf_file_filter_class_set_all_cb), vf);
+	menu_item_add_simple(menu, _("Select all"), G_CALLBACK(vf_file_filter_class_select_all_cb), vf);
+	menu_item_add_simple(menu, _("Select none"), G_CALLBACK(vf_file_filter_class_select_none_cb), vf);
 
 	return menu;
 }
@@ -1625,11 +1590,13 @@ void vf_star_cleanup(ViewFile *vf)
 
 void vf_star_stop(ViewFile *vf)
 {
-	 vf_star_cleanup(vf);
+	vf_star_cleanup(vf);
 }
 
 static void vf_set_star_fd(ViewFile *vf, FileData *fd)
 {
+	if (!fd) return;
+
 	switch (vf->type)
 		{
 		case FILEVIEW_LIST: vflist_set_star_fd(vf, fd); break;
@@ -1638,12 +1605,7 @@ static void vf_set_star_fd(ViewFile *vf, FileData *fd)
 		}
 }
 
-static void vf_star_do(ViewFile *vf, FileData *fd)
-{
-	if (!fd) return;
-
-	vf_set_star_fd(vf, fd);
-}
+static gboolean vf_stars_cb(gpointer data);
 
 static gboolean vf_star_next(ViewFile *vf)
 {
@@ -1663,31 +1625,35 @@ static gboolean vf_star_next(ViewFile *vf)
 		return FALSE;
 		}
 
+	vf->stars_filedata = fd;
+
+	if (vf->stars_id == 0)
+		{
+		vf->stars_id = g_idle_add_full(G_PRIORITY_LOW, vf_stars_cb, vf, nullptr);
+		}
+
 	return TRUE;
 }
 
-gboolean vf_stars_cb(gpointer data)
+static gboolean vf_stars_cb(gpointer data)
 {
 	auto vf = static_cast<ViewFile *>(data);
+
 	FileData *fd = vf->stars_filedata;
+	if (!fd) return G_SOURCE_REMOVE;
 
-	if (fd)
+	read_rating_data(fd);
+
+	vf_set_star_fd(vf, fd);
+
+	if (!vf_star_next(vf))
 		{
-		read_rating_data(fd);
-
-		vf_star_do(vf, fd);
-
-		if (vf_star_next(vf))
-			{
-			return G_SOURCE_CONTINUE;
-			}
-
 		vf->stars_filedata = nullptr;
 		vf->stars_id = 0;
 		return G_SOURCE_REMOVE;
 		}
 
-	return G_SOURCE_REMOVE;
+	return G_SOURCE_CONTINUE;
 }
 
 void vf_star_update(ViewFile *vf)
@@ -1739,31 +1705,22 @@ guint vf_marks_get_filter(ViewFile *vf)
 
 GRegex *vf_file_filter_get_filter(ViewFile *vf)
 {
-	GRegex *ret = nullptr;
-	GError *error = nullptr;
-	gchar *file_filter_text = nullptr;
-
 	if (!gtk_widget_get_visible(vf->file_filter.combo))
 		{
 		return g_regex_new("", static_cast<GRegexCompileFlags>(0), static_cast<GRegexMatchFlags>(0), nullptr);
 		}
 
-	file_filter_text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(vf->file_filter.combo));
-
-	if (file_filter_text[0] != '\0')
+	g_autofree gchar *file_filter_text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(vf->file_filter.combo));
+	if (file_filter_text[0] == '\0')
 		{
-		ret = g_regex_new(file_filter_text, vf->file_filter.case_sensitive ? static_cast<GRegexCompileFlags>(0) : G_REGEX_CASELESS, static_cast<GRegexMatchFlags>(0), &error);
-		if (error)
-			{
-			log_printf("Error: could not compile regular expression %s\n%s\n", file_filter_text, error->message);
-			g_error_free(error);
-			error = nullptr;
-			ret = g_regex_new("", static_cast<GRegexCompileFlags>(0), static_cast<GRegexMatchFlags>(0), nullptr);
-			}
-		g_free(file_filter_text);
+		return g_regex_new("", static_cast<GRegexCompileFlags>(0), static_cast<GRegexMatchFlags>(0), nullptr);
 		}
-	else
+
+	g_autoptr(GError) error = nullptr;
+	GRegex *ret = g_regex_new(file_filter_text, vf->file_filter.case_sensitive ? static_cast<GRegexCompileFlags>(0) : G_REGEX_CASELESS, static_cast<GRegexMatchFlags>(0), &error);
+	if (error)
 		{
+		log_printf("Error: could not compile regular expression %s\n%s\n", file_filter_text, error->message);
 		ret = g_regex_new("", static_cast<GRegexCompileFlags>(0), static_cast<GRegexMatchFlags>(0), nullptr);
 		}
 
@@ -1845,6 +1802,10 @@ void vf_notify_cb(FileData *fd, NotifyType type, gpointer data)
 	gboolean refresh;
 
 	auto interested = static_cast<NotifyType>(NOTIFY_CHANGE | NOTIFY_REREAD | NOTIFY_GROUPING);
+	if (options->show_star_rating)
+		{
+		interested = static_cast<NotifyType>(interested | NOTIFY_METADATA);
+		}
 	if (vf->marks_enabled) interested = static_cast<NotifyType>(interested | NOTIFY_MARKS | NOTIFY_METADATA);
 	/** @FIXME NOTIFY_METADATA should be checked by the keyword-to-mark functions and converted to NOTIFY_MARKS only if there was a change */
 
@@ -1854,25 +1815,22 @@ void vf_notify_cb(FileData *fd, NotifyType type, gpointer data)
 
 	if (!refresh)
 		{
-		gchar *base = remove_level_from_path(fd->path);
+		g_autofree gchar *base = remove_level_from_path(fd->path);
 		refresh = (g_strcmp0(base, vf->dir_fd->path) == 0);
-		g_free(base);
 		}
 
 	if ((type & NOTIFY_CHANGE) && fd->change)
 		{
 		if (!refresh && fd->change->dest)
 			{
-			gchar *dest_base = remove_level_from_path(fd->change->dest);
+			g_autofree gchar *dest_base = remove_level_from_path(fd->change->dest);
 			refresh = (g_strcmp0(dest_base, vf->dir_fd->path) == 0);
-			g_free(dest_base);
 			}
 
 		if (!refresh && fd->change->source)
 			{
-			gchar *source_base = remove_level_from_path(fd->change->source);
+			g_autofree gchar *source_base = remove_level_from_path(fd->change->source);
 			refresh = (g_strcmp0(source_base, vf->dir_fd->path) == 0);
-			g_free(source_base);
 			}
 		}
 
