@@ -30,7 +30,6 @@
 
 #include <config.h>
 
-#include "debug.h"
 #include "intl.h"
 #include "main-defines.h"
 #include "main.h"
@@ -111,14 +110,12 @@ gchar *HtmlBrowser::command_result() const
 
 GtkWidget *window_new(const gchar *role, const gchar *icon, const gchar *icon_file, const gchar *subtitle)
 {
-	gchar *title;
+	g_autofree gchar *title = nullptr;
 	GtkWidget *window;
 
-#if HAVE_GTK4
-	window = gtk_window_new();
-#else
-	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-#endif
+	GApplication *app = g_application_get_default();
+	window = gtk_application_window_new(GTK_APPLICATION(app));
+
 	if (!window) return nullptr;
 
 	if (subtitle)
@@ -131,7 +128,6 @@ GtkWidget *window_new(const gchar *role, const gchar *icon, const gchar *icon_fi
 		}
 
 	gtk_window_set_title(GTK_WINDOW(window), title);
-	g_free(title);
 
 	window_set_icon(window, icon, icon_file);
 	gtk_window_set_role(GTK_WINDOW(window), role);
@@ -183,8 +179,7 @@ gboolean window_maximized(GtkWidget *window)
 
 static int help_browser_command(const gchar *command, const gchar *path)
 {
-	gchar *result;
-	gchar *buf;
+	g_autofree gchar *result = nullptr;
 	gchar *begin;
 	gchar *end;
 	int retval = -1;
@@ -193,7 +188,7 @@ static int help_browser_command(const gchar *command, const gchar *path)
 
 	DEBUG_1("Help command pre \"%s\", \"%s\"", command, path);
 
-	buf = g_strdup(command);
+	g_autofree gchar *buf = g_strdup(command);
 	begin = strstr(buf, "%s");
 	if (begin)
 		{
@@ -207,14 +202,12 @@ static int help_browser_command(const gchar *command, const gchar *path)
 		{
 		result = g_strdup_printf("%s \"%s\" &", command, path);
 		}
-	g_free(buf);
 
 	DEBUG_1("Help command post [%s]", result);
 
 	retval = runcmd(result);
 	DEBUG_1("Help command exit code: %d", retval);
 
-	g_free(result);
 	return retval;
 }
 
@@ -249,22 +242,20 @@ static void help_browser_run(const gchar *path)
  *-----------------------------------------------------------------------------
  */
 
-static GtkWidget *help_window = nullptr;
-
-static void help_window_destroy_cb(GtkWidget *, gpointer)
+static void help_window_destroy_cb(GtkWidget *, gpointer data)
 {
-	help_window = nullptr;
+	auto **help_window = static_cast<GtkWidget **>(data);
+	*help_window = nullptr;
 }
 
 void help_window_show(const gchar *key)
 {
-	gchar *path;
-
 	if (key && strstr(key, ".html") != nullptr)
 		{
-		path = g_build_filename(gq_htmldir, key, NULL);
+		g_autofree gchar *path = g_build_filename(gq_htmldir, key, NULL);
 		if (!isfile(path))
 			{
+			g_free(path);
 			if (g_strcmp0(key, "index.html") == 0)
 				{
 				path = g_build_filename("https://www.geeqie.org/help/", "GuideIndex.html", NULL);
@@ -275,10 +266,10 @@ void help_window_show(const gchar *key)
 				}
 			}
 		help_browser_run(path);
-		g_free(path);
 		return;
 		}
 
+	static GtkWidget *help_window = nullptr;
 	if (help_window)
 		{
 		gtk_window_present(GTK_WINDOW(help_window));
@@ -286,48 +277,31 @@ void help_window_show(const gchar *key)
 		return;
 		}
 
-	if (!strcmp(key, "release_notes"))
-		{
-		path = g_build_filename(gq_helpdir, "README.html", NULL);
-		if (isfile(path))
+	const auto open_help_file = [key](const gchar *html_file, const gchar *text_file)
+	{
+		g_autofree gchar *html_path = g_build_filename(gq_helpdir, html_file, NULL);
+		if (isfile(html_path))
 			{
-			g_free(path);
-			path = g_build_filename("file://", gq_helpdir, "README.html", NULL);
-			help_browser_run(path);
-			g_free(path);
+			g_autofree gchar *url = g_strdup_printf("file://%s", html_path);
+			help_browser_run(url);
 			}
 		else
 			{
-			g_free(path);
-			path = g_build_filename(gq_helpdir, "README.md", NULL);
-			help_window = help_window_new(_("Help"), "help", path, key);
-			g_free(path);
+			g_autofree gchar *text_path = g_build_filename(gq_helpdir, text_file, NULL);
+			help_window = help_window_new(_("Help"), "help", text_path, key);
 
 			g_signal_connect(G_OBJECT(help_window), "destroy",
-					 G_CALLBACK(help_window_destroy_cb), NULL);
+			                 G_CALLBACK(help_window_destroy_cb), &help_window);
 			}
+	};
+
+	if (!strcmp(key, "release_notes"))
+		{
+		open_help_file("README.html", "README.md");
 		}
 	else
 		{
-		path = g_build_filename(gq_helpdir, "ChangeLog.html", NULL);
-		if (isfile(path))
-			{
-			g_free(path);
-			path = g_build_filename("file://", gq_helpdir, "ChangeLog.html", NULL);
-			help_browser_run(path);
-			g_free(path);
-			}
-		else
-			{
-			g_free(path);
-			path = g_build_filename(gq_helpdir, "ChangeLog", NULL);
-			help_window = help_window_new(_("Help"), "help", path, key);
-			g_free(path);
-
-			g_signal_connect(G_OBJECT(help_window), "destroy",
-					 G_CALLBACK(help_window_destroy_cb), NULL);
-			}
-
+		open_help_file("ChangeLog.html", "ChangeLog");
 		}
 }
 
@@ -337,53 +311,39 @@ void help_window_show(const gchar *key)
  *-----------------------------------------------------------------------------
  */
 
-struct HelpSearchData {
-	GenericDialog *gd;
-	GtkWidget *edit_widget;
-	gchar *text_entry;
-};
-
-static void help_search_window_show_icon_press(GtkEntry *, GtkEntryIconPosition, GdkEvent *, gpointer userdata)
+static void help_search_window_show_icon_press(GtkEntry *edit_widget, GtkEntryIconPosition, GdkEvent *, gpointer)
 {
-	auto hsd = static_cast<HelpSearchData *>(userdata);
-
-	g_free(hsd->text_entry);
-	hsd->text_entry = g_strdup("");
-	gq_gtk_entry_set_text(GTK_ENTRY(hsd->edit_widget), hsd->text_entry);
+	gq_gtk_entry_set_text(edit_widget, "");
 }
 
 static void help_search_window_ok_cb(GenericDialog *, gpointer data)
 {
-	auto hsd = static_cast<HelpSearchData *>(data);
-	gchar *search_command;
+	auto *edit_widget = GTK_ENTRY(data);
 
-	search_command = g_strconcat(options->help_search_engine,
-						gq_gtk_entry_get_text(GTK_ENTRY(hsd->edit_widget)),
-						NULL);
+	g_autofree gchar *search_command = g_strconcat(options->help_search_engine,
+	                                               gq_gtk_entry_get_text(edit_widget),
+	                                               NULL);
 	help_browser_run(search_command);
-	g_free(search_command);
-
-	g_free(hsd);
-}
-
-static void help_search_window_cancel_cb(GenericDialog *, gpointer data)
-{
-	auto hsd = static_cast<HelpSearchData *>(data);
-
-	g_free(hsd);
 }
 
 void help_search_window_show()
 {
-	GenericDialog *gd;
 	GtkWidget *table;
 	GtkWidget *label1;
 	GtkWidget *label2;
 
-	auto hsd = g_new0(HelpSearchData, 1);
-	hsd->gd = gd = generic_dialog_new(_("On-line help search"), "help_search",
-				nullptr, TRUE,
-				help_search_window_cancel_cb, hsd);
+	GtkWidget *edit_widget = gtk_entry_new();
+	gtk_entry_set_icon_from_icon_name(GTK_ENTRY(edit_widget),
+	                                  GTK_ENTRY_ICON_SECONDARY, GQ_ICON_CLEAR);
+	gtk_entry_set_icon_tooltip_text(GTK_ENTRY(edit_widget),
+	                                GTK_ENTRY_ICON_SECONDARY, _("Clear"));
+	gtk_widget_set_size_request(edit_widget, 300, -1);
+	g_signal_connect(GTK_ENTRY(edit_widget), "icon-press",
+	                 G_CALLBACK(help_search_window_show_icon_press), nullptr);
+
+	GenericDialog *gd = generic_dialog_new(_("On-line help search"), "help_search",
+	                                       nullptr, TRUE,
+	                                       nullptr, edit_widget);
 	generic_dialog_add_message(gd, nullptr, _("Search the on-line help files.\n"), nullptr, FALSE);
 
 	generic_dialog_add_button(gd, GQ_ICON_OK, "OK",
@@ -401,46 +361,31 @@ void help_search_window_show()
 
 	table = pref_table_new(gd->vbox, 3, 1, FALSE, TRUE);
 	pref_table_label(table, 0, 0, _("Search terms:"), GTK_ALIGN_END);
-	hsd->edit_widget = gtk_entry_new();
-	gtk_widget_set_size_request(hsd->edit_widget, 300, -1);
-	gq_gtk_grid_attach_default(GTK_GRID(table), hsd->edit_widget, 1, 2, 0, 1);
-	generic_dialog_attach_default(gd, hsd->edit_widget);
-	gtk_widget_show(hsd->edit_widget);
+	gq_gtk_grid_attach_default(GTK_GRID(table), edit_widget, 1, 2, 0, 1);
+	generic_dialog_attach_default(gd, edit_widget);
+	gtk_widget_show(edit_widget);
 
-	gtk_entry_set_icon_from_icon_name(GTK_ENTRY(hsd->edit_widget),
-						GTK_ENTRY_ICON_SECONDARY, GQ_ICON_CLEAR);
-	gtk_entry_set_icon_tooltip_text (GTK_ENTRY(hsd->edit_widget),
-						GTK_ENTRY_ICON_SECONDARY, _("Clear"));
-	g_signal_connect(GTK_ENTRY(hsd->edit_widget), "icon-press",
-						G_CALLBACK(help_search_window_show_icon_press), hsd);
-
-	gtk_widget_grab_focus(hsd->edit_widget);
+	gtk_widget_grab_focus(edit_widget);
 
 	gtk_widget_show(gd->dialog);
 }
 
 void help_pdf()
 {
-	GError *error = nullptr;
-	gchar *path;
-	gchar *command;
-
-	path = g_build_filename(gq_helpdir, "help.pdf", NULL);
+	g_autofree gchar *path = g_build_filename(gq_helpdir, "help.pdf", NULL);
 
 	if (!isfile(path))
 		{
+		g_free(path);
 		path = g_build_filename("https://www.geeqie.org/help-pdf/help.pdf", NULL);
 		}
 
-	command = g_strdup_printf("xdg-open %s", path);
+	g_autofree gchar *command = g_strdup_printf("xdg-open %s", path);
 
+	g_autoptr(GError) error = nullptr;
 	if (!g_spawn_command_line_async(command, &error))
 		{
 		log_printf(_("Warning: Failed to execute command: %s\n"), error->message);
-		g_error_free(error);
 		}
-
-	g_free(path);
-	g_free(command);
 }
 /* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
